PRINT N'Altering [dbo].[AccountSubAccountVCCBatchSummaryClose]...';


GO
ALTER PROCEDURE [dbo].[AccountSubAccountVCCBatchSummaryClose](
	@BatchId VARCHAR(50) = NULL, 
	@ClosedDate DATETIME = NULL
)
AS
BEGIN
	-- ESTE SP TIENE LA FINALIDAD DE ACTUALIZAR LOS SALDOS DE LAS CARTERAS DEL VCC
	SET XACT_ABORT, NOCOUNT ON

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY

		-- INICIAMOS LA TRANSACCION
		BEGIN TRANSACTION

		-- VERIFICAMOS QUE TENEMOS LOS PARAMETROS NECESARIOS
		IF @BatchId IS NULL 
			-- LANZAMOS MENSAJE AVISANDO QUE FALTA ACCOUNTID
			RAISERROR('DATABABASE_ERROR_MISSING_PARAMETERS', 16, 16)

		-- DECLARAMOS UNA VARIABLE PARA ALMACENAR LOS SYSTEMSERVICE RELACIONADOS AL BATCH
		DECLARE MY_CURSOR CURSOR 
		LOCAL STATIC READ_ONLY FORWARD_ONLY
			FOR 
			-- CONSULTAMOS LOS SYSTEMSERVICE DEL BATCH PARA LA ACTUALIZACION DE SALDOS
			SELECT ASAVST.[InstallationId],
					ASAVST.[VCC], 
					ASAVST.[SystemService], 
					ASAVST.[Rate]
				FROM [dbo].[AccountSubAccountVCCSummaryTransaction] ASAVST 
				WHERE ASAVST.[BatchID] = @BatchId

		-- DECLARARAMOS UNA VARIABLE PARA INDICAR EL BACTH ACTUAL QUE ESTAMOS RECORRIENDO
		DECLARE @CURRENT_INSTALLATION_ID VARCHAR(50) = ''
		DECLARE @CURRENT_VCC VARCHAR(50) = ''
		DECLARE @CURRENT_SYSTEM_SERVICE VARCHAR(50) = ''
		DECLARE @CURRENT_RATE DECIMAL(18, 5) = 0

		-- DECLARAMOS VARIABLES DE AYUDA QUE CAMBIARAN EN EL BUCLE
		DECLARE @MOVEMENT_DESCRIPTION VARCHAR(250) = NULL
		DECLARE @BATCH_TOTAL_STEPS INT = 0
		DECLARE @BATCH_CURRENT_STEP INT = 0
		DECLARE @INITIAL_CALCULATED_AMOUNT DECIMAL(18,5) = 0
		DECLARE @FINAL_CALCULATED_AMOUNT DECIMAL(18,5) = 0

		
		-- ABRIMOS Y RECORREMOS EL ARRAY DE CAMPOS A ACTUALIZAR
		OPEN MY_CURSOR
		FETCH NEXT FROM MY_CURSOR INTO @CURRENT_INSTALLATION_ID, @CURRENT_VCC, @CURRENT_SYSTEM_SERVICE, @CURRENT_RATE
		WHILE @@FETCH_STATUS = 0
		BEGIN 
			-- OBTENEMOS LOS DATOS GENERALES DEL BATCH EN LA TABLA DE MOVIMIENTOS
			SELECT @BATCH_CURRENT_STEP = MAX(ASAVBM.[CurrentStep]),
					@BATCH_TOTAL_STEPS = ASAVBM.[TotalSteps]
				FROM [dbo].[AccountSubAccountVCCBalanceMovement] ASAVBM (NOLOCK)
				WHERE ASAVBM.[BatchID] = @BatchId
				GROUP BY ASAVBM.[InstallationId], ASAVBM.[VCC], ASAVBM.[TotalSteps]

			-- OBTENEMOS LOS DATOS ESPECIFICOS DEL BATCH-SYSTEMSERVICE EN LA TABLA DE MOVIMIENTOS
			SELECT @INITIAL_CALCULATED_AMOUNT = ASAVBM.[Amount]
				FROM [dbo].[AccountSubAccountVCCBalanceMovement] ASAVBM (NOLOCK)
				WHERE ASAVBM.[BatchID] = @BatchId
					AND ASAVBM.[InstallationId] = @CURRENT_INSTALLATION_ID
					AND ASAVBM.[Wallet] = @CURRENT_SYSTEM_SERVICE

			-- PARA REGISTRAR LOS MOVIMIENTOS AUMENTAMOS EL CURRENT_STEP PARA REGISTRAR EL SIGUIENTE MOVIMIENTO
			SET @BATCH_CURRENT_STEP = @BATCH_CURRENT_STEP + 1

			-- DEFINIMOS LA DESCRIPCION DEL MOVIMIENTO DEBITAJE
			SET @MOVEMENT_DESCRIPTION = CONCAT('VCC ', @CURRENT_VCC, ': DEBT $', @CURRENT_RATE, ' FROM ', @CURRENT_SYSTEM_SERVICE, ' WALLET ON CLOSING BATCH')

			-- INSERTAMOS EN LA TABLA ACCOUNSUBACCOUNTVCCBALANCEMOVEMENT EL REGISTRO PARA EL COBRO A LA CARTERA RECORRIDA
			INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
						([ID] ,[InstallationId] ,[AccountId] ,[SubAccountId] ,[VCC]
						,[Wallet] ,[Type] ,[CurrentStep] ,[TotalSteps] ,[Description]
						,[BatchId] ,[Amount]
						,[PrevAvailableBalance] ,[PostAvailableBalance] 
						,[PrevCountableBalance] ,[PostCountableBalance] 
						,[TimeStamp])
				SELECT @BatchId ,AVB.[InstallationId] ,AVB.[AccountId] ,AVB.[SubAccountId] ,AVB.[VCC]
						,AVB.[SystemService] ,'DEBT' ,@BATCH_CURRENT_STEP ,@BATCH_TOTAL_STEPS ,@MOVEMENT_DESCRIPTION
						,@BatchId ,@CURRENT_RATE
						,AVB.[AvailableBalance] ,AVB.[AvailableBalance]
						,AVB.[CountableBalance] ,AVB.[CountableBalance] - @CURRENT_RATE 
						,GETUTCDATE()
					FROM [dbo].[AccountVCCBalance] AVB (NOLOCK)
					WHERE [InstallationId] = @CURRENT_INSTALLATION_ID 
						AND [VCC] = @CURRENT_VCC
						AND [SystemService] = @CURRENT_SYSTEM_SERVICE
						--AND [CountableBalance] > @FINAL_CALCULATED_AMOUNT

			-- DESBLOQUEMOS EL REGISTRO DE SALDOS DEL SERVICO USANDO LOS MISMO PARAMETROS DE BUSQUEDA Y ACTUALIZAMOS EL SALDO DISPONIBLE Y CONTABLE
			UPDATE [dbo].[AccountVCCBalance] 
				SET [IsLocked] = 0, 
					[AvailableBalance] = [AvailableBalance] + (@INITIAL_CALCULATED_AMOUNT - @CURRENT_RATE),
					[RetainedBalance] = [RetainedBalance] - @INITIAL_CALCULATED_AMOUNT,
					[CountableBalance] = [CountableBalance] - @CURRENT_RATE,
					--[LastModifiedByUserId] = @UserApi,
					[LastModifiedDate] = GETUTCDATE(),
					[TimeStamp] = GETUTCDATE() 
				WHERE [InstallationId] = @CURRENT_INSTALLATION_ID 
					AND [VCC] = @CURRENT_VCC
					AND [SystemService] = @CURRENT_SYSTEM_SERVICE

			-- AUMENTAMOS EL CURRENT_STEP PARA REGISTRAR EL SIGUIENTE MOVIMIENTO
			SET @BATCH_CURRENT_STEP = @BATCH_CURRENT_STEP + 1

			-- DEFINIMOS LA DESCRIPCION DEL MOVIMIENTO LIBERAR
			SET @MOVEMENT_DESCRIPTION = CONCAT('VCC ', @CURRENT_VCC, ': RELEASE $', (@INITIAL_CALCULATED_AMOUNT - @CURRENT_RATE), ' FROM ', @CURRENT_SYSTEM_SERVICE, ' WALLET ON CLOSING BATCH')
			
			-- INSERTAMOS EN LA TABLA ACCOUNSUBACCOUNTVCCBALANCEMOVEMENT EL REGISTRO PARA LA LIBERACION DE SALDO RETENIDO A LA CARTERA RECORRIDA
			INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
						([ID] ,[InstallationId] ,[AccountId] ,[SubAccountId] ,[VCC]
						,[Wallet] ,[Type] ,[CurrentStep] ,[TotalSteps] ,[Description]
						,[BatchId] ,[Amount]
						,[PrevAvailableBalance] ,[PostAvailableBalance] 
						,[PrevCountableBalance] ,[PostCountableBalance] 
						,[TimeStamp])
				SELECT @BatchId ,AVB.[InstallationId] ,AVB.[AccountId] ,AVB.[SubAccountId] ,AVB.[VCC]
						,AVB.[SystemService] ,'RELEASE' ,@BATCH_CURRENT_STEP ,@BATCH_TOTAL_STEPS ,@MOVEMENT_DESCRIPTION
						,@BatchId ,@INITIAL_CALCULATED_AMOUNT - @CURRENT_RATE
						,AVB.[AvailableBalance] ,AVB.[AvailableBalance] + (@INITIAL_CALCULATED_AMOUNT - @CURRENT_RATE)
						,AVB.[CountableBalance] ,AVB.[CountableBalance] 
						,GETUTCDATE()
					FROM [dbo].[AccountVCCBalance] AVB (NOLOCK)
					WHERE [InstallationId] = @CURRENT_INSTALLATION_ID 
						AND [VCC] = @CURRENT_VCC
						AND [SystemService] = @CURRENT_SYSTEM_SERVICE

			-- ACTUALIZAR EL BATCH COMO ISFINISHED TRUE EN CASO NO HAYA SIDO MARCADO TODAVIA
			UPDATE [dbo].[AccountSubAccountVCCSummaryTransaction]
				SET [IsFinished] = 1,
					[EndDate] = CASE WHEN @ClosedDate IS NULL THEN [EndDate] ELSE @ClosedDate END,
					[Status] = CASE WHEN [Status] LIKE 'CANCELLED%' THEN 'FINISHED_BY_CANCELLED' ELSE 'FINISHED' END, 
					--[Status] = 'FINISHED', -- ESTE ESTADO SE CAMBIARA PARA DAR OPCION A FINISHED COMO COMPLETADO SIN INTERVENSION O FINISHED_BY_USER CUANDO EL USUARIO DECIDE TERMINARLO
					[LasModifiedDate] = GETUTCDATE(),
					[TimeStamp] = GETUTCDATE()
				WHERE [InstallationId] = @CURRENT_INSTALLATION_ID 
					AND [BatchID] = @BatchId
					AND [VCC] = @CURRENT_VCC
					AND [SystemService] = @CURRENT_SYSTEM_SERVICE
					--AND [IsFinished] = 0
					AND [Status] NOT IN ('FINISHED', 'FINISHED_BY_CANCELLED')

			FETCH NEXT FROM MY_CURSOR INTO @CURRENT_INSTALLATION_ID, @CURRENT_VCC, @CURRENT_SYSTEM_SERVICE, @CURRENT_RATE
		END
		CLOSE MY_CURSOR
		DEALLOCATE MY_CURSOR

		-- RETORNAMOS RESULTADO 1
		SELECT 1 'result'

		-- COMPLETAMOS LA TRANSACCION
		COMMIT TRANSACTION

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH

	SET XACT_ABORT, NOCOUNT OFF
END
GO
PRINT N'Altering [dbo].[GetAccountSubAccountVCCBalance]...';


GO
ALTER PROCEDURE [dbo].[GetAccountSubAccountVCCBalance] (
	@PlatformHash VARCHAR(100) = NULL,
	@InstallationId VARCHAR(50) = NULL,
	@AccountId VARCHAR(50) = NULL,
	@SubAccountId VARCHAR(50) = NULL,
	@VCC VARCHAR(50) = NULL,
	@SystemService VARCHAR(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	-- VERIFICAMOS SI EXISTE UNA PLATAFORMA CON EL PLATFORMHASH INGRESADO
	IF NOT EXISTS(
		SELECT P.[PlatformHash] 
			FROM dbo.[Platform] P 
			WHERE P.[PlatformHash] = @PlatformHash
				AND P.[InstallationId] = @InstallationId
	)
	BEGIN
		-- SI NO EXISTE LA PLATAFORMA ENVIAMOS UN MENSAJE DE ERROR
		RAISERROR('DATABABASE_ERROR_NOT_FOUND_PLATAFORM', 16, 16)
		RETURN
	END

	-- VERIFICAMOS SI SUBACCOUNID ES NULL PARA LA VALIDACION DE CUENTA/SUBCUENTA
	IF @SubAccountId IS NULL 
	BEGIN
		-- SI SUBCUENTA ES NULL, SOLO VERIFICAMOS SI EXISTE LA CUENTA
		IF NOT EXISTS(
			SELECT A.[AccountId] 
				FROM dbo.[Account] A 
				WHERE A.[PlatformHash] = @PlatformHash
					AND A.[InstallationId] = @InstallationId 
					AND A.[AccountId] = @AccountId
					AND A.[SubAccountId] IS NULL
		)
		BEGIN
			-- SI NO EXISTE LA CUENTA ENVIAMOS UN MENSAJE DE ERROR
			RAISERROR('DATABABASE_ERROR_NOT_FOUND_ACCOUNT', 16, 16)
			RETURN
		END
		ELSE 
		BEGIN
			-- VERIFICAMOS SI EXISTE EL VCC ASOCIADO A LA CUENTA
			IF NOT EXISTS(
				SELECT AV.[VCC] 
					FROM dbo.[AccountVCC] AV 
					WHERE AV.[InstallationId] = @InstallationId 
						AND AV.[AccountId] = @AccountId
						AND AV.[SubAccountId] IS NULL
						AND AV.[VCC] = @VCC
			)	
			BEGIN
				-- SI NO EXISTE LA CUENTA ENVIAMOS UN MENSAJE DE ERROR
				RAISERROR('DATABABASE_ERROR_NOT_FOUND_ACCOUNT_VCC', 16, 16)
				RETURN
			END
		END
	END
	ELSE
	BEGIN 
		-- SI SUBCUENTA ES DIFERENTE DE NULL, VERIFICAMOS SI EXISTE LA CUENTA CON SUBCUENTA ASOCIADA
		IF NOT EXISTS(
			SELECT A.[SubAccountId] 
				FROM dbo.[Account] A 
				WHERE A.[PlatformHash] = @PlatformHash
					AND A.[InstallationId] = @InstallationId 
					AND A.[AccountId] = @AccountId
					AND A.[SubAccountId] = @SubAccountId
		)
		BEGIN
			-- SI NO EXISTE LA PLATAFORMA ENVIAMOS UN MENSAJE DE ERROR
			RAISERROR('DATABABASE_ERROR_NOT_FOUND_SUBACCOUNT', 16, 16)
			RETURN
		END
		ELSE 
		BEGIN
			-- VERIFICAMOS SI EXISTE EL VCC ASOCIADO A LA CUENTA-SUBCUENTA
			IF NOT EXISTS(
				SELECT AV.[VCC] 
					FROM dbo.[AccountVCC] AV 
					WHERE AV.[InstallationId] = @InstallationId 
						AND AV.[AccountId] = @AccountId
						AND AV.[SubAccountId] = @SubAccountId
						AND AV.[VCC] = @VCC
			)	
			BEGIN
				-- SI NO EXISTE LA CUENTA ENVIAMOS UN MENSAJE DE ERROR
				RAISERROR('DATABABASE_ERROR_NOT_FOUND_SUBACCOUNT_VCC', 16, 16)
				RETURN
			END
		END
	END;

	-- DECLARAMOS LAS CTE QUE JUNTARAN LA DATA REQUERIDA PARA EL RESULT
	WITH CTE_BATCHES_DETAILED_RATED AS (
		-- HACEMOS SELECT A LOS BATCHES CON POR BATACHID, SYSTEMSERVICE, CON SALDO RETENIDO Y SALDO CONSUMIDO
		SELECT ASAVST.[InstallationId],
				ASAVBM.[AccountId],
				ASAVBM.[SubAccountId],
				ASAVST.[VCC],
				ASAVST.[BatchID],
				ASAVST.[SystemService],
				ASAVBM.[Amount] AS 'Estimated',
				SUM(ASAVDT.[Rate]) AS 'Rated',
				ASAVST.[CreatedDate],
				COUNT(*) AS 'Details'
			FROM [dbo].[AccountSubAccountVCCSummaryTransaction] ASAVST (NOLOCK)
			LEFT JOIN [dbo].[AccountSubAccountVCCDetailedTransaction] ASAVDT (NOLOCK) 
				ON ASAVDT.[BatchId] = ASAVST.[BatchID]
					AND ASAVDT.[SystemService] = ASAVST.[SystemService]
			LEFT JOIN [dbo].[AccountSubAccountVCCBalanceMovement] ASAVBM (NOLOCK)
				ON ASAVBM.[VCC] = ASAVST.[VCC]
					AND ASAVBM.[BatchId] = ASAVST.[BatchID]
					AND ASAVBM.[Wallet] = ASAVST.[SystemService]
			WHERE ASAVST.[InstallationId] = @InstallationId
				AND ASAVBM.[AccountId] = @AccountId
				AND (
					(@SubAccountId IS NULL AND ASAVBM.[SubAccountId] IS NULL)
					OR (ASAVBM.[SubAccountId] = @SubAccountId)
				)
				AND ASAVBM.[VCC] = @VCC
				AND ASAVBM.[Type] = 'RETAIN'
				--AND (ASAVST.[Status] <> 'FINISHED' AND ASAVST.[Status] NOT LIKE 'CANCELLED')
				AND (ASAVST.[Status] NOT LIKE 'FINISHED%')
				AND GETUTCDATE() < DATEADD(DAY, 5, ASAVST.[CreatedDate])
				AND ASAVDT.[IsDemo] = 0
			GROUP BY ASAVST.[InstallationId],
						ASAVBM.[AccountId],
						ASAVBM.[SubAccountId],
						ASAVST.[VCC],
						ASAVST.[BatchID],
						ASAVST.[SystemService],
						ASAVBM.[Amount],
						ASAVST.[CreatedDate]
	),
	CTE_BATCHES_SUMMARY_RATED AS (
		-- HACEMOS SELECT AL RESULTADO ANTERIOR PARA SUMARIZAR EL RATED, ESTIMATED Y DIFFERENCE
		SELECT CTEBDR.[InstallationId],
				CTEBDR.[AccountId],
				CTEBDR.[SubAccountId],
				CTEBDR.[VCC],
				CTEBDR.[SystemService],
				SUM(CTEBDR.[Estimated]) AS 'Estimated',
				SUM(CTEBDR.[Rated]) AS 'Rated'
			FROM CTE_BATCHES_DETAILED_RATED CTEBDR 
			GROUP BY CTEBDR.[InstallationId],
						CTEBDR.[AccountId],
						CTEBDR.[SubAccountId],
						CTEBDR.[VCC],
						CTEBDR.[SystemService]
	)
	
	-- HACEMOS LA BUSQUEDA DE LOS SALDOS DEL CUENTA-SUBCUENTA-VCC
	SELECT AVB.[AccountId]														'accountId', 
			AVB.[SubAccountId]													'subAccountId',
			AVB.[SystemService]													'systemService',
			AVB.[CountableBalance]												'countableBalance',
			AVB.[AvailableBalance]												'availableBalance',
			AVB.[RetainedBalance]												'retainedBalance',
			-- ISNULL(CTEBMR.[Estimated], 0)										'estimated',
			-- ISNULL(CTEBMR.[Rated], 0)											'rated',
			ISNULL(ABS(CTEBMR.[Estimated] - CTEBMR.[Rated]), 0)					'difference',
			CASE WHEN CTEBMR.[Rated] > CTEBMR.[Estimated] THEN 1 ELSE 0 END		'exceeded'
		FROM dbo.[AccountVCCBalance] AVB (NOLOCK)
		LEFT JOIN CTE_BATCHES_SUMMARY_RATED CTEBMR 
			ON CTEBMR.[InstallationId] = AVB.[InstallationId]
				AND CTEBMR.[AccountId] = AVB.[AccountId]
				AND (
					(CTEBMR.[SubAccountId] IS NULL AND AVB.[SubAccountId] IS NULL)
					OR (CTEBMR.[SubAccountId] = AVB.[SubAccountId])
				)
				AND CTEBMR.[VCC] = AVB.[VCC]
				AND CTEBMR.[SystemService] = AVB.[SystemService]
		WHERE AVB.[InstallationId] = @InstallationId 
			AND AVB.[AccountId] = @AccountId
			AND (
				(@SubAccountId IS NULL AND AVB.[SubAccountId] IS NULL)
				OR (AVB.[SubAccountId] = @SubAccountId)
			) 
			AND AVB.[VCC] = @VCC
			AND AVB.[SystemService] = ISNULL(@SystemService, AVB.[SystemService]) 

	SET NOCOUNT OFF
END
GO
PRINT N'Altering [dbo].[GetPlatformCancelledBatches]...';


GO
-- ESTA FUNCION RETORNA LAS TARIFAS SETTEADAS DE UNA CUENTA O SUBCUENTA
ALTER PROCEDURE [dbo].[GetPlatformCancelledBatches] (
	@PlatformHash VARCHAR(100) = NULL,
	@InstallationId VARCHAR(100) = NULL
	)
AS
BEGIN
	-- ESTE SP TIENE LA FINALIDAD DE ACTUALIZAR LOS SALDOS DE LAS CARTERAS DEL VCC
	SET XACT_ABORT, NOCOUNT ON

	-- ESTADOS DE LOS BATCHASUMMARY:
	-- CREATED: ESTADO POR DEFECTO AL CREAR EL BATCH
	-- RUNNING: ESTADO CUANDO EL BATCH TIENE 1 O MAS DETALLES (BATCHDETAILED)
	-- PAUSED_BY_USER: ESTADO CUANDO EL BATCH ES PAUSADO POR EL USUARIO POR ALGUN MOTIVO
	-- PAUSED_BY_INSUFFICIENT_BALANCE: ESTADO CUANDO EL BATCH ES PAUSADO POR EL SISTEMA POR NO TENER SALDO DISPONIBLE
	-- CLOSED: ESTADO CUANDO EL BATCH TERMINO SIN PROBLEMA NI INTERVENSION
	-- CANCELLED_BY_TIMEOUT: ESTADO CUANDO EL BATCH ES TERMINADO POR EL SISTEMA POR SUPERAR 72 HORAS DESDE LA PAUSA
	-- CANCELLED_BY_USER: ESTADO CUANDO EL BATCH ES TERMINADO POR EL USUARIO POR ALGUN MOTIVO
	
	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY

		-- SCRIPT PARA RECUPERAR LOS BATCH ABIERTOS, QUE HAN SIDO CANCELADOS
		SELECT ASAVST.[InstallationId],
				ASAVST.[BatchId],
				ASAVST.[CreatedDate],
				ASAVST.[BatchTypeName],
				ASAVST.[BatchTypeId],
				ASAVST.[VCC],
				ASAVST.[SystemService],
				ASAVST.[Status]
			FROM [dbo].[AccountSubAccountVCCSummaryTransaction] ASAVST (NOLOCK)
			WHERE ASAVST.[InstallationId] = @InstallationId
				--AND ASAVST.[IsFinished] = 0
				--AND ASAVDT.[IsValid] = 1
				--AND ASAVDT.[IsFinished]= 1
				AND ASAVST.[Status] LIKE 'CANCELLED%'

    END TRY
    BEGIN CATCH
        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH

	SET XACT_ABORT, NOCOUNT OFF
END
GO
PRINT N'Altering [dbo].[GetPlatformCurrentBatches]...';


GO
-- ESTA FUNCION RETORNA LAS TARIFAS SETTEADAS DE UNA CUENTA O SUBCUENTA
ALTER PROCEDURE [dbo].[GetPlatformCurrentBatches] (
	@PlatformHash VARCHAR(100) = NULL,
	@InstallationId VARCHAR(100) = NULL
	)
AS
BEGIN
	-- ESTE SP TIENE LA FINALIDAD DE ACTUALIZAR LOS SALDOS DE LAS CARTERAS DEL VCC
	SET XACT_ABORT, NOCOUNT ON

	-- ESTADOS DE LOS BATCHASUMMARY:
	-- CREATED: ESTADO POR DEFECTO AL CREAR EL BATCH
	-- RUNNING: ESTADO CUANDO EL BATCH TIENE 1 O MAS DETALLES (BATCHDETAILED)
	-- PAUSED_BY_USER: ESTADO CUANDO EL BATCH ES PAUSADO POR EL USUARIO POR ALGUN MOTIVO
	-- PAUSED_BY_INSUFFICIENT_BALANCE: ESTADO CUANDO EL BATCH ES PAUSADO POR EL SISTEMA POR NO TENER SALDO DISPONIBLE
	-- CLOSED: ESTADO CUANDO EL BATCH TERMINO SIN PROBLEMA NI INTERVENSION
	-- CANCELLED_BY_TIMEOUT: ESTADO CUANDO EL BATCH ES TERMINADO POR EL SISTEMA POR SUPERAR 72 HORAS DESDE LA PAUSA
	-- CANCELLED_BY_USER: ESTADO CUANDO EL BATCH ES TERMINADO POR EL USUARIO POR ALGUN MOTIVO
	
	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY

		-- INICIAMOS LA TRANSACCION
		BEGIN TRANSACTION

		-- SCRIPT PARA RECUPERAR LOS SALDOS ACTUALES DE LA PLATAFORMA
		SELECT AVB.[InstallationId],
				AVB.[AccountId],
				AVB.[SubAccountId],
				AVB.[VCC],
				AVB.[SystemService],
				AVB.[CountableBalance],
				AVB.[AvailableBalance],
				AVB.[RetainedBalance]
			FROM [dbo].[AccountVCCBalance] AVB (NOLOCK)
			WHERE AVB.[InstallationId] = @InstallationId
				AND AVB.[VCC] IN (
					SELECT ASAVST.[VCC] 
						FROM [dbo].[AccountSubAccountVCCSummaryTransaction] ASAVST (NOLOCK)
						WHERE ASAVST.[InstallationId] = AVB.[InstallationId]
							AND ASAVST.[IsFinished] = 0
				)

		-- DEFINIMOS EL TIEMPO EN HORAS PARA CERRAR UN BATCH AUTOMATICAMENTE (SI SE MODIFICA, MODIFICAR TAMBIEN EN EL SP GETBATCHBALANCEDETAIL)
		DECLARE @HOURS_TO_CLOSE_PAUSED_BATCH_SUMMARY DECIMAL(18,5) = 72

		-- HACEMO UN UPDATE A LA TABLA SUMMARY PARA MARCAR COMO CANCELLED_BY_TIMEOUT LOS BATCHES PAUSADOS CON 72 HORAS O MAS
		UPDATE [dbo].[AccountSubAccountVCCSummaryTransaction] 
			SET [Status] = 'CANCELLED_BY_TIMEOUT',
				[LastModifiedByUserId] = 'system',
				[LasModifiedDate] = CURRENT_TIMESTAMP,
				[TimeStamp] =  CURRENT_TIMESTAMP
			WHERE [Status] LIKE 'PAUSED%' 
				AND GETUTCDATE() >= DATEADD(HOUR , @HOURS_TO_CLOSE_PAUSED_BATCH_SUMMARY , [LasModifiedDate]) 

		-- DECLARAMOS UNA VARIABLE PARA ALMACENAR LOS BATCH CREATED QUE TIENEN DETALLE
		DECLARE MY_CURSOR CURSOR 
		LOCAL STATIC READ_ONLY FORWARD_ONLY
			FOR 
			-- SCRIPT PARA RETORNAR BATCHES CON ESTADO CREATED QUE TIENEN 1 O MAS DETALLES
			SELECT ASAVST.[BatchId],
					ASAVST.[VCC],
					ASAVST.[SystemService]
				FROM [dbo].[AccountSubAccountVCCSummaryTransaction] ASAVST (NOLOCK) 
				LEFT JOIN [dbo].[AccountSubAccountVCCDetailedTransaction] ASAVDT 
					ON ASAVDT.[InstallationId] = ASAVST.[InstallationId]
						AND ASAVDT.[VCC] = ASAVST.[VCC]
						AND ASAVDT.[BatchId] = ASAVST.[BatchId]
						AND ASAVDT.[SystemService] = ASAVST.[SystemService]
				WHERE ASAVST.[InstallationId] = @InstallationId
					--AND ASAVST.[IsFinished] = 0 <--HUBO UN PROBLEMA CUANDO UN COGNITIVO LLENO DE FALLIDOS AL PRIMER LANZAMIENTO
					AND ASAVST.[Status] = 'CREATED'
					AND ASAVDT.[BatchId] IS NOT NULL
				GROUP BY ASAVST.[InstallationId],
					ASAVST.[BatchId],
					ASAVST.[VCC],
					ASAVST.[SystemService],
					ASAVST.[CreatedDate]
				ORDER BY ASAVST.[CreatedDate] ASC

		-- DECLARARAMOS UNA VARIABLE PARA INDICAR EL BACTH ACTUAL QUE ESTAMOS RECORRIENDO
		DECLARE @CURRENT_BATCH_ID VARCHAR(50) = ''
		DECLARE @CURRENT_BATCH_VCC VARCHAR(50) = ''
		DECLARE @CURRENT_BATCH_SYSTEM_SERVICE VARCHAR(50) = ''

		-- ABRIMOS Y RECORREMOS EL ARRAY DE PRODUCTOS
		OPEN MY_CURSOR
		FETCH NEXT FROM MY_CURSOR INTO @CURRENT_BATCH_ID, @CURRENT_BATCH_VCC, @CURRENT_BATCH_SYSTEM_SERVICE
		WHILE @@FETCH_STATUS = 0
		BEGIN 
			-- ACTUALIZAMOS A ESTADO RUNNING CADA BATCH DE ESTADO CREATED QUE YA TIENE 1 REGISTRO EN DETAILED
			UPDATE [dbo].[AccountSubAccountVCCSummaryTransaction]
				SET [Status] = 'RUNNING'
				WHERE [InstallationId] = @InstallationId
					AND [BatchId] = @CURRENT_BATCH_ID
					AND [VCC] = @CURRENT_BATCH_VCC
					AND [SystemService] = @CURRENT_BATCH_SYSTEM_SERVICE

			FETCH NEXT FROM MY_CURSOR INTO @CURRENT_BATCH_ID, @CURRENT_BATCH_VCC, @CURRENT_BATCH_SYSTEM_SERVICE
		END
		CLOSE MY_CURSOR
		DEALLOCATE MY_CURSOR

		-- SCRIPT PARA RECUPERAR LOS BATCH ABIERTOS, EL ACUMULADO DE LOS DETALLES CERRADOS Y EL RETENIDO GENERAL DEL BATCH
		SELECT ASAVST.[InstallationId],
				ASAVST.[BatchId],
				ASAVST.[CreatedDate],
				ASAVST.[BatchTypeName],
				ASAVST.[BatchTypeId],
				ASAVBM.[AccountId],
				ASAVBM.[SubAccountId],
				ASAVST.[VCC],
				ASAVST.[SystemService],
				SUM(ASAVDT.[Rate]) 'RATED',
				ASAVBM.[Amount] 'RETAINED',
				COUNT(ASAVDT.[BatchId]) 'DETAILS',
				ASAVST.[Status],
				AVB.[CountableBalance],
				AVB.[AvailableBalance],
				AVB.[RetainedBalance]
			FROM [dbo].[AccountSubAccountVCCSummaryTransaction] ASAVST (NOLOCK)
			LEFT JOIN [dbo].[AccountSubAccountVCCDetailedTransaction] ASAVDT 
				ON ASAVDT.[InstallationId] = ASAVST.[InstallationId]
					AND ASAVDT.[VCC] = ASAVST.[VCC]
					AND ASAVDT.[BatchId] = ASAVST.[BatchId]
					AND ASAVDT.[SystemService] = ASAVST.[SystemService]
			INNER JOIN [dbo].[AccountSubAccountVCCBalanceMovement] ASAVBM (NOLOCK) 
				ON ASAVBM.[InstallationId] = ASAVST.[InstallationId]
					AND ASAVBM.[VCC] = ASAVST.[VCC]
					AND ASAVBM.BatchId = ASAVST.[BatchId]
					AND ASAVBM.[Wallet] = ASAVST.[SystemService]
			INNER JOIN [dbo].[AccountVCCBalance] AVB  (NOLOCK) 
				ON AVB.[InstallationId] = ASAVST.[InstallationId]
					AND AVB.[AccountId] = ASAVBM.[AccountId]
					AND AVB.[VCC] = ASAVST.[VCC]
					AND AVB.[SystemService] = ASAVST.[SystemService]
			WHERE ASAVST.[InstallationId] = @InstallationId
				AND (ASAVST.[Status] IN ('CREATED', 'RUNNING')
					OR ASAVST.[Status] LIKE 'CANCELLED%')
				--AND ASAVST.[IsFinished] = 0
				--AND ASAVDT.[IsValid] = 1
				--AND ASAVDT.[IsFinished]= 1
				AND ASAVBM.[Type] = 'RETAIN'
				AND GETUTCDATE() <= DATEADD(HOUR , 98 , CASE WHEN ASAVST.[LasModifiedDate] IS NULL THEN GETUTCDATE() ELSE ASAVST.[LasModifiedDate] END) 
			GROUP BY ASAVST.[InstallationId],
				ASAVST.[BatchId],
				ASAVST.[BatchTypeName],
				ASAVST.[BatchTypeId],
				ASAVBM.[AccountId],
				ASAVBM.[SubAccountId],
				ASAVST.[VCC],
				ASAVST.[SystemService],
				ASAVST.[CreatedDate],
				ASAVBM.[Amount],
				ASAVST.[Status],
				AVB.[CountableBalance],
				AVB.[AvailableBalance],
				AVB.[RetainedBalance]
			ORDER BY ASAVST.[CreatedDate] DESC

		-- COMPLETAMOS LA TRANSACCION
		COMMIT TRANSACTION

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH

	SET XACT_ABORT, NOCOUNT OFF
END
GO
PRINT N'Update complete.';


GO
