PRINT N'Modificando [dbo].[ActivatePlatform]...';


GO
ALTER PROCEDURE [dbo].[ActivatePlatform]
(
	@PlatformHash			VARCHAR(100),
	@InstallationId			VARCHAR(50),
	@MacAddress				VARCHAR(50),
	@PublicIp				VARCHAR(50),
	@User					VARCHAR(50),
	@Token					VARCHAR(50)
)
AS
BEGIN
	SET XACT_ABORT, NOCOUNT ON;

	-- ABRIMOS EL TRY PARA LA TRANSACCION
    BEGIN TRY
        BEGIN TRANSACTION

		-- DECLARAMOS UNA VARIABLE PARA ALMACENAR UN FLAG SI EXISTE EL PLATFORM HASH
		DECLARE @PLATFORMHASH_EXISTS BIT

		-- DECLARAMOS UNA VARIABLE PARA ALMACENAR UN FLAG SI EXISTE LA CUENTA PRINCIPAL O CUENTA DE ACTIVACION
		DECLARE @MAINACCOUNT VARCHAR(50)

		-- SETTEAMOS UN VALOR FALSE POR DEFECTO
		SET @PLATFORMHASH_EXISTS = 0

		-- SETTEAMOS UN VALOR FALSE POR DEFECTO
		SET @MAINACCOUNT = NULL

		-- VALIDAMOS SI YA EXISTE UNA PLATAFORMA CON EL INSTALLATION ID
		/*
			IF EXISTS (
				SELECT P.[InstallationId]
					FROM [dbo].[Platform] P (NOLOCK)
					WHERE P.[InstallationId] = @InstallationId 
			)
				-- LANZAMOS MESAJE AVISANDO QUE NO SE ENCONTRO LA CUENTA
				RAISERROR('DATABABASE_ERROR_INSTALLATION_ID_REPEAT', 16, 16)*/

		-- VALIDAMOS QUE EL PLATFORMHASH ESTA REGISTRADO EN LA TABLA PLATFORM
		SELECT TOP 1 @PLATFORMHASH_EXISTS = CASE WHEN P.[PlatformHash] IS NOT NULL THEN 1 END
			FROM dbo.[Platform] P (NOLOCK) 
			WHERE P.[PlatformHash] = @PlatformHash 

		-- VALIDAMOS QUE EL USUARIO, TOKEN E INSTALLATIONID COINCIDEN CON LOS DE LA TABLA ACCOUNT
		SELECT TOP 1 @MAINACCOUNT = A.[AccountId] 
			FROM dbo.[Account] A (NOLOCK)
			WHERE A.[PlatformHash] = @PlatformHash 
				AND A.[User] = @User
				AND A.[Token] = @Token
				AND A.[AccountLevel] = 1 
				AND (
					A.[InstallationId] IS NULL 
					OR 
					(A.[InstallationId] IS NOT NULL AND A.[InstallationId] = @InstallationId)
				)
				AND A.[SubAccountId] IS NULL

		-- VERIFICAMOS QUE AMBOS RESULTADOS EXISTAN
		IF @PLATFORMHASH_EXISTS = 1 AND @MAINACCOUNT IS NOT NULL
		BEGIN
		
			-- ACTUALIZAMOS LA PLATAFORMA CON SUS DATOS DE INFRAESTRUCTURA
			UPDATE [dbo].[Platform] 
				SET [InstallationId] = @InstallationId,
					[MacAddress] = @MacAddress,
					[PublicIp] = @PublicIp,
					[IsActivated] = 1,
					[ActivationDate] = GETUTCDATE(),
					[LastModifiedDate] = GETUTCDATE(),
					[LastModifiedByUserId] = @User,
					[TimeStamp] = GETUTCDATE()
				WHERE
					[PlatformHash] = @PlatformHash

			IF (@@ROWCOUNT = 0) 
				RAISERROR('DATABASE_ERROR_PLATFORM_UPDATE_FAILED', 16, 16)
				

			-- ACTUALIZAMOS LA COLUMNA INSTALLATIONID EN LA TABLA PLATFORMDATA
			UPDATE dbo.[PlatformData] 
				SET [InstallationId] = @InstallationId,
					[LastModifiedDate] = GETUTCDATE(),
					[LastModifiedByUserId] = @User,
					[TimeStamp] = GETUTCDATE()
				WHERE [PlatformHash] = @PlatformHash

			IF (@@ROWCOUNT = 0) 
				RAISERROR('DATABASE_ERROR_PLATFORMDATA_UPDATE_FAILED', 16, 16)

			-- ACTUALIZAMOS LA COLUMNA INSTALLATIONID EN LA TABLA ACCOUNT PARA LA MAIN ACCOUNT
			UPDATE dbo.[Account] 
				SET [InstallationId] = @InstallationId,
					[LastModifiedDate] = GETUTCDATE(),
					[LastModifiedByUserId] = @User,
					[TimeStamp] = GETUTCDATE()
				WHERE [PlatformHash] = @PlatformHash
					AND [User] = @User
					AND [Token] = @Token
					AND [AccountLevel] = 1
		
			IF (@@ROWCOUNT = 0) 
				RAISERROR('DATABASE_ERROR_ACCOUNT_UPDATE_FAILED', 16, 16)


			-- ACTUALIZAMOS LA COLUMNA INSTALLATIONID EN LA TABLA ACCOUNTBALANCE PARA LA MAIN ACCOUNT
			UPDATE dbo.[AccountBalance] 
				SET [InstallationId] = @InstallationId,
					[LastModifiedDate] = GETUTCDATE(),
					[LastModifiedByUserId] = @User,
					[TimeStamp] = GETUTCDATE()
				WHERE [PlatformHash] = @PlatformHash 
					AND [AccountId] = @MAINACCOUNT

			IF (@@ROWCOUNT = 0) 
				RAISERROR('DATABASE_ERROR_ACCOUNTBALANCE_UPDATE_FAILED', 16, 16)

			-- ACTUALIZAMOS LA COLUMNA INSTALLATIONID EN LA TABLA DE MOVIMIENTOS
			UPDATE [dbo].[AccountSubAccountVCCBalanceMovement]
				SET [InstallationId] = @InstallationId
				WHERE (
						[InstallationId] IS NULL 
						OR 
						([InstallationId] IS NOT NULL AND [InstallationId] = @InstallationId)
					)
					AND [AccountId] = @MAINACCOUNT
					AND [SubAccountId] IS NULL
					
			IF (@@ROWCOUNT = 0) 
				RAISERROR('DATABASE_ERROR_ACCOUNTSUBACCOUNTVCCBALANCEMOVEMENT_UPDATE_FAILED', 16, 16)

			-- ACTUALIZAMOS LA COLUMNA INSTALLATIONID EN LA TABLA PLATFORMLICENSE
			UPDATE dbo.[PlatformLicense] 
				SET [InstallationId] = @InstallationId,
					[LastModifiedDate] = GETUTCDATE(),
					[LastModifiedByUserId] = @User,
					[TimeStamp] = GETUTCDATE()
				WHERE [PlatformHash] = @PlatformHash 

			IF (@@ROWCOUNT = 0) 
				RAISERROR('DATABASE_ERROR_PLATFORMLICENSE_UPDATE_FAILED', 16, 16)

			-- SELECT 1 AS 'result'
			SELECT P.[InstallationId] 'installationId', 
					P.[MacAddress] 'macAddress', 
					P.[BaseBoardSerialNumber] 'baseBoardSerialNumber',
					P.[PublicIp] 'publicIp',
					P.[IsActivated] 'isActivated'
				FROM [dbo].[Platform] P
				WHERE P.[PlatformHash] = @PlatformHash 

		END
		ELSE
		BEGIN 
			CREATE TABLE #DUMMY ( A Int, B VarChar(42) )
			SELECT * FROM #DUMMY
		END

	-- CONSOLIDAMOS LA TRANSACCION
		COMMIT TRANSACTION

    -- TERMINAMOS EL TRY PARA LA TRANSACCION
    END TRY
	BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH

   SET NOCOUNT OFF
END
GO
PRINT N'Modificando [dbo].[GetCognitiveAnalysisAverageRequestPerMinute]...';


GO
ALTER PROCEDURE [dbo].[GetCognitiveAnalysisAverageRequestPerMinute](
	@InstallationId VARCHAR(50) = NULL,
	@VCC VARCHAR(50) = NULL
)
AS
BEGIN
	-- ESTE SP TIENE LA FINALIDAD DE RETORNAR UN NUMERO PROMEDIO DE REQUEST DE ANALISIS COGNITIVO EN UN MINUTO
	-- USADO POR [dbo].[AccountSubAccountVCCBatchSummaryCreate] Y [dbo].[GetAccountVccRates]
	SET XACT_ABORT, NOCOUNT ON

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY
		-- DECLARAMOS UNA VARIABLE PARA ALMACENAR EL VALOR DE SEGUNDOS POR REQUEST
		DECLARE @SECONDS_PER_REQUEST DECIMAL(5,2) = 0

		-- DECLARAMOS UNA VARIABLE PARA ALMACENAR EL VALOR DE REQUEST POR MINUTO
		DECLARE @REQUEST_PER_MINUTE DECIMAL(5,2) = 0
		
		-- *TASA DE SEGURO DE FACTURACION(DECIMAL): SI SE CAMBIA ESTE VALOR, SE DEBERA CAMBIAR TAMBIEN EN EL SP [dbo].[AccountSubAccountVCCBatchSummaryCreate]
		DECLARE @COGNITIVE_ANALYSIS_BILLING_SECURE_RATE DECIMAL(5, 2) = 0.15 -- <-SE BORRARA

		-- DECLARAMOS LAS VARIABLES PARA LOS RESULTADOS TOTALES
		DECLARE @TOTAL_AUDIOS DECIMAL(18, 5) = 0
		DECLARE @TOTAL_DURATION_TIME DECIMAL(18, 5) = 0
		DECLARE @TOTAL_VALID_DURATION_TIME DECIMAL(18, 5) = 0
		DECLARE @TOTAL_ROUNDED_DURATION_TIME DECIMAL(18, 5) = 0
		DECLARE @TOTAL_VALID_REQUEST DECIMAL(18, 5) = 0
					
		-- RECUPERAMOS LOS VALORES TOTALES DE LOS ULTIMOS SIETE DIAS
		SELECT TOP 100 @TOTAL_AUDIOS = SUM(ASAVST.[TotalAudio]), 
				@TOTAL_DURATION_TIME = SUM(ASAVST.[DurationTime] + ASAVST.[AlreadyDurationTime]),
				@TOTAL_VALID_DURATION_TIME = SUM(CASE WHEN (ASAVST.[ValidDurationTime] + ASAVST.[AlreadyTranscribedDurationTime]) > 60 
												THEN (ASAVST.[ValidDurationTime] + ASAVST.[AlreadyTranscribedDurationTime])
												ELSE 60 END), 
				@TOTAL_ROUNDED_DURATION_TIME = SUM(CASE WHEN (ASAVST.[RoundedDurationTime] + ASAVST.[AlreadyTranscribedRoundedDurationTime]) > 60 
													THEN (ASAVST.[RoundedDurationTime] + ASAVST.[AlreadyTranscribedRoundedDurationTime]) 
													ELSE 60 END), 
				/*
				@TOTAL_VALID_REQUEST = SUM(CASE WHEN (ASAVDT.[TotalValidRequest] + ASAVDT.[TotalInvalidRequest]) < (ASAVST.[TotalAudio]) 
											THEN ASAVST.[TotalAudio] 
											ELSE (ASAVDT.[TotalValidRequest] + ASAVDT.[TotalInvalidRequest]) END)
				*/
				@TOTAL_VALID_REQUEST = SUM(CASE WHEN (ASAVDT.[TotalValidRequest]) < (ASAVST.[TotalAudio]) 
											THEN ASAVST.[TotalAudio] 
											ELSE (ASAVDT.[TotalValidRequest]) END)
			FROM [dbo].[AccountSubAccountVCCSummaryTransaction] ASAVST (NOLOCK) 
			INNER JOIN [dbo].[AccountSubAccountVCCDetailedTransaction] ASAVDT (NOLOCK) 
				ON ASAVDT.[InstallationId] = ASAVST.[InstallationId]
					AND ASAVDT.[VCC] = ASAVST.[VCC]
					AND ASAVDT.[BatchId] = ASAVST.[BatchID]
					AND ASAVDT.[SystemService] = ASAVST.[SystemService]
			WHERE ASAVST.[InstallationId] = @InstallationId
				AND ASAVST.[VCC] = @VCC 
				AND ASAVST.[SystemService] = 'COGNITIVE_ANALYSIS'
				AND ASAVDT.[Description] = 'SUCCED'
				AND ASAVST.[CreatedDate] >= '2020-05-01T00:00:00Z'

		/*
		SELECT @TOTAL_AUDIOS 'audios', 
				@TOTAL_VALID_REQUEST 'requests', 
				@TOTAL_DURATION_TIME 'durationTime',
				@TOTAL_VALID_DURATION_TIME 'validTime', 
				@TOTAL_ROUNDED_DURATION_TIME 'roundedTime', 
				(60.00 * @TOTAL_VALID_REQUEST) / @TOTAL_VALID_DURATION_TIME 'validTimeAverage',
				(60.00 * @TOTAL_VALID_REQUEST) / @TOTAL_ROUNDED_DURATION_TIME 'roundedTimeAverage'
		*/
			
		-- VALIDAMOS QUE LOS VALORES SEAN VALIDOS
		IF @TOTAL_AUDIOS IS NULL OR @TOTAL_AUDIOS = 0 
			OR @TOTAL_DURATION_TIME IS NULL OR @TOTAL_DURATION_TIME = 0 
			OR @TOTAL_VALID_DURATION_TIME IS NULL OR @TOTAL_VALID_DURATION_TIME = 0 
			OR @TOTAL_ROUNDED_DURATION_TIME IS NULL OR @TOTAL_ROUNDED_DURATION_TIME = 0
			OR @TOTAL_VALID_REQUEST  IS NULL OR @TOTAL_VALID_REQUEST = 0
		BEGIN
			-- TOMAMOS EL VALOR POR DEFECTO (O VALOR BASE) PARA SEGUNDOS POR REQUEST
			SET @SECONDS_PER_REQUEST = 30

			-- TOMAMOS EL VALOR POR DEFECTO (O VALOR BASE) PARA REQUEST POR MINUTO
			SET @REQUEST_PER_MINUTE = 2
		END
		ELSE
		BEGIN
			-- CALCULAMOSEL VALOR PROMEDIO DE SEGUNDOS POR REQUEST SEGUN EL HISTORICO
			SET @SECONDS_PER_REQUEST = @TOTAL_DURATION_TIME / @TOTAL_VALID_REQUEST

			-- CALCULAMOS EL VALOR PROMEDIO DE REQUEST POR MINUTO SEGUN EL HISTORICO
			--SET @REQUEST_PER_MINUTE = CEILING ((60.00 * @TOTAL_VALID_REQUEST) / @TOTAL_VALID_DURATION_TIME)
			SET @REQUEST_PER_MINUTE = (60.00 * @TOTAL_VALID_REQUEST) / @TOTAL_VALID_DURATION_TIME
			--SET @REQUEST_PER_MINUTE = CEILING ((60.00 * @TOTAL_VALID_REQUEST) / @TOTAL_ROUNDED_DURATION_TIME)
		END

		-- INSERTAMOS EL NUEVO PROMEDIO DE REQUEST EN LA TABLA HISTORICA
		INSERT INTO [dbo].[CognitiveAnalysisHistory] 
			([InstallationId], [VCC], [AverageRequestPerMinute])
		VALUES (@InstallationId, @VCC, @REQUEST_PER_MINUTE)
		
		-- RECUPERAMOS EL VALOR DE REQUEST PROMEDIO POR MINUTO
		SELECT @SECONDS_PER_REQUEST 'CAASPR', 
				@REQUEST_PER_MINUTE 'CAARPM', 
				@COGNITIVE_ANALYSIS_BILLING_SECURE_RATE 'CABSR'

	END TRY
    BEGIN CATCH
        -- IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH

	SET XACT_ABORT, NOCOUNT OFF
END
GO
PRINT N'Actualizando [dbo].[AccountSubAccountVCCBatchSummaryCreate]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[AccountSubAccountVCCBatchSummaryCreate]';


GO
PRINT N'Actualizando [dbo].[GetAccountVccRates]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetAccountVccRates]';


GO
PRINT N'Actualización completada.';


GO

