/*
Se está quitando la columna [dbo].[PlatformData].[CurrencyCode]; puede que se pierdan datos.
*/

IF EXISTS (select top 1 1 from [dbo].[PlatformData])
    RAISERROR (N'Se detectaron filas. La actualización del esquema va a terminar debido a una posible pérdida de datos.', 16, 127) WITH NOWAIT

GO
PRINT N'Quitando [dbo].[DF_Account_TranscriptionRatePerHour]...';


GO
ALTER TABLE [dbo].[Account] DROP CONSTRAINT [DF_Account_TranscriptionRatePerHour];


GO
PRINT N'Quitando [dbo].[DF_Account_TranscriptionRatePerSecond]...';


GO
ALTER TABLE [dbo].[Account] DROP CONSTRAINT [DF_Account_TranscriptionRatePerSecond];


GO
PRINT N'Quitando [dbo].[DF_Account_SemanticAnalysisRatePerTransaction]...';


GO
ALTER TABLE [dbo].[Account] DROP CONSTRAINT [DF_Account_SemanticAnalysisRatePerTransaction];


GO
PRINT N'Quitando [dbo].[DF_Account_CognitiveAnalysisRatePerTransaction]...';


GO
ALTER TABLE [dbo].[Account] DROP CONSTRAINT [DF_Account_CognitiveAnalysisRatePerTransaction];


GO
PRINT N'Quitando [dbo].[DF_Account_Enabled]...';


GO
ALTER TABLE [dbo].[Account] DROP CONSTRAINT [DF_Account_Enabled];


GO
PRINT N'Quitando [dbo].[DF_Account_CreatedDate]...';


GO
ALTER TABLE [dbo].[Account] DROP CONSTRAINT [DF_Account_CreatedDate];


GO
PRINT N'Iniciando recompilación de la tabla [dbo].[Account]...';


GO
BEGIN TRANSACTION;

SET TRANSACTION ISOLATION LEVEL SERIALIZABLE;

SET XACT_ABORT ON;

CREATE TABLE [dbo].[tmp_ms_xx_Account] (
    [PlatformHash]                        VARCHAR (100)   NULL,
    [ID]                                  VARCHAR (100)   NOT NULL,
    [InstallationId]                      VARCHAR (100)   NULL,
    [AccountId]                           VARCHAR (50)    NOT NULL,
    [SubAccountId]                        VARCHAR (50)    NULL,
    [User]                                VARCHAR (50)    NOT NULL,
    [Token]                               VARCHAR (50)    NOT NULL,
    [AccountLevel]                        INT             NOT NULL,
    [CountryCode]                         VARCHAR (10)    NULL,
    [CurrencyCode]                        VARCHAR (10)    NULL,
    [TranscriptionRatePerHour]            DECIMAL (18, 5) CONSTRAINT [DF_Account_TranscriptionRatePerHour] DEFAULT ((0)) NOT NULL,
    [TranscriptionRatePerSecond]          DECIMAL (18, 5) CONSTRAINT [DF_Account_TranscriptionRatePerSecond] DEFAULT ((0)) NOT NULL,
    [SemanticAnalysisRatePerTransaction]  DECIMAL (18, 5) CONSTRAINT [DF_Account_SemanticAnalysisRatePerTransaction] DEFAULT ((0)) NOT NULL,
    [CognitiveAnalysisRatePerTransaction] DECIMAL (18, 5) CONSTRAINT [DF_Account_CognitiveAnalysisRatePerTransaction] DEFAULT ((0)) NOT NULL,
    [Enabled]                             BIT             CONSTRAINT [DF_Account_Enabled] DEFAULT ((1)) NOT NULL,
    [CreatedDate]                         DATETIME        CONSTRAINT [DF_Account_CreatedDate] DEFAULT (getutcdate()) NULL,
    [CreatedByUserId]                     VARCHAR (50)    NULL,
    [LastModifiedDate]                    DATETIME        NULL,
    [LastModifiedByUserId]                VARCHAR (50)    NULL,
    [TimeStamp]                           DATETIME        NULL,
    CONSTRAINT [tmp_ms_xx_constraint_PK_Account1] PRIMARY KEY CLUSTERED ([ID] ASC)
);

IF EXISTS (SELECT TOP 1 1 
           FROM   [dbo].[Account])
    BEGIN
        INSERT INTO [dbo].[tmp_ms_xx_Account] ([ID], [PlatformHash], [InstallationId], [AccountId], [SubAccountId], [User], [Token], [AccountLevel], [CurrencyCode], [TranscriptionRatePerHour], [TranscriptionRatePerSecond], [SemanticAnalysisRatePerTransaction], [CognitiveAnalysisRatePerTransaction], [Enabled], [CreatedDate], [CreatedByUserId], [LastModifiedDate], [LastModifiedByUserId], [TimeStamp])
        SELECT   [ID],
                 [PlatformHash],
                 [InstallationId],
                 [AccountId],
                 [SubAccountId],
                 [User],
                 [Token],
                 [AccountLevel],
                 [CurrencyCode],
                 [TranscriptionRatePerHour],
                 [TranscriptionRatePerSecond],
                 [SemanticAnalysisRatePerTransaction],
                 [CognitiveAnalysisRatePerTransaction],
                 [Enabled],
                 [CreatedDate],
                 [CreatedByUserId],
                 [LastModifiedDate],
                 [LastModifiedByUserId],
                 [TimeStamp]
        FROM     [dbo].[Account]
        ORDER BY [ID] ASC;
    END

DROP TABLE [dbo].[Account];

EXECUTE sp_rename N'[dbo].[tmp_ms_xx_Account]', N'Account';

EXECUTE sp_rename N'[dbo].[tmp_ms_xx_constraint_PK_Account1]', N'PK_Account', N'OBJECT';

COMMIT TRANSACTION;

SET TRANSACTION ISOLATION LEVEL READ COMMITTED;


GO
PRINT N'Modificando [dbo].[AccountSubAccountVCCBalanceMovement]...';


GO
ALTER TABLE [dbo].[AccountSubAccountVCCBalanceMovement] ALTER COLUMN [Description] NVARCHAR (250) NULL;


GO
PRINT N'Modificando [dbo].[PlatformData]...';


GO
ALTER TABLE [dbo].[PlatformData] DROP COLUMN [CurrencyCode];


GO
PRINT N'Modificando [dbo].[AccountSubAccountActivate]...';


GO
ALTER PROCEDURE [dbo].[AccountSubAccountActivate] (
	@InstallationId VARCHAR(50) = NULL,
	@AccountId VARCHAR(50) = NULL,
	@SubAccountId VARCHAR(50) = NULL,
	@LastModifiedByUserId VARCHAR(50) = NULL
	)
AS
BEGIN
	-- ESTE SP TIENE POR FINALIDAD DESACTIVAR UNA CUENTA/SUBCUENTA (ENABLED = 0)
	SET XACT_ABORT, NOCOUNT ON

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY
        -- INICIAMOS LA TRANSACCION
		BEGIN TRANSACTION

		-- VERIFICAMOS QUE TENEMOS LOS PARAMETROS NECESARIOS
		IF @InstallationId IS NULL OR @AccountId IS NULL
			-- LANZAMOS MENSAJE AVISANDO QUE FALTA ACCOUNTID
			RAISERROR('DATABASE_ERROR_MISSING_PARAMETERS', 16, 16)

		-- VALIDAMOS SI EXISTE LA INSTALLACION 
		IF NOT EXISTS (
			SELECT P.[IsActivated] 
				FROM [dbo].[Platform] P (NOLOCK) 
				WHERE P.[InstallationId] = @InstallationId
		)
			-- LANZAMOS MENSAJE AVISANDO QUE FALTA ACCOUNTID
			RAISERROR('DATABASE_ERROR_INSTALLATION_DOES_NOT_EXIST', 16, 16)

		-- VALIDAMOS SI LA INSTALACION ESTA ACTIVDA
		IF NOT EXISTS (
			SELECT P.[IsActivated] 
				FROM [dbo].[Platform] P (NOLOCK) 
				WHERE P.[InstallationId] = @InstallationId
					AND P.[IsActivated] = 1
		)
			-- LANZAMOS MENSAJE AVISANDO QUE FALTA ACCOUNTID
			RAISERROR('DATABASE_ERROR_INSTALLATION_IS_NOT_ACTIVATED', 16, 16)

		-- VALIDAMOS SI EXISTE LA CUENTA
		IF NOT EXISTS (
			SELECT 1 
				FROM [dbo].[Account] A (NOLOCK)
				WHERE A.[InstallationId] = @InstallationId 
					AND A.[AccountId] = @AccountId
		)
			-- LANZAMOS MENSAJE AVISANDO QUE FALTA ACCOUNTID
			RAISERROR('DATABASE_ERROR_ACCOUNT_DOES_NOT_EXIST', 16, 16)

		-- CONTINUAMOS EL PROCEDIMIENTO DEPENDIENDO SI TENEMOS SUBCUENTAID
		IF @SubAccountId IS NULL
		BEGIN
			-- EN CASO ESTEMOS DESACTIVANDO UNA CUENTA, BUSCAMOS TODOS LOS VCC ASOCIADOS A ELLA

			-- ACTUALIZAMOS EL CAMPO ENABLED DE LA TABLA ACCOUNT
			UPDATE [dbo].[Account]
				SET [Enabled] = 1,
					[LastModifiedByUserId] = @LastModifiedByUserId,
					[LastModifiedDate] = GETUTCDATE()
				WHERE [InstallationId] = @InstallationId 
					AND [AccountId] = @AccountId
					AND [SubAccountId] IS NULL
			-- VERIFICAMOS SI EL REGISTRO FUE ACTUALIZADO
			IF @@ROWCOUNT = 0
				RAISERROR('DATABASE_ERROR_UPDATE_FAILED', 16, 16)

		END
		ELSE
		BEGIN
			-- EN CASO ESTEMOS DESACTIVANDO UNA SUBCUENTA, SOLO BUSCAMOS UN UNICO VCC

			-- VALIDAMOS SI EXISTE LA SUBCUENTA
			IF NOT EXISTS (
				SELECT 1 
					FROM [dbo].[Account] A (NOLOCK)
					WHERE A.[InstallationId] = @InstallationId 
						AND A.[AccountId] = @AccountId
						AND A.[SubAccountId] = @SubAccountId
			)
				-- LANZAMOS MENSAJE AVISANDO QUE FALTA ACCOUNTID
				RAISERROR('DATABASE_ERROR_SUBACCOUNT_DOES_NOT_EXIST', 16, 16)

			-- ACTUALIZAMOS EL CAMPO ENABLED DE LA TABLA ACCOUNT
			UPDATE [dbo].[Account]
				SET [Enabled] = 1,
					[LastModifiedByUserId] = @LastModifiedByUserId,
					[LastModifiedDate] = GETUTCDATE()
				WHERE [InstallationId] = @InstallationId 
					AND [AccountId] = @AccountId
					AND [SubAccountId] = @SubAccountId 
			-- VERIFICAMOS SI EL REGISTRO FUE ACTUALIZADO
			IF @@ROWCOUNT = 0
				RAISERROR('DATABASE_ERROR_UPDATE_FAILED', 16, 16)

		END

		-- RETORNAMOS RESULTADO 1
		SELECT 1 'result'

		-- COMPLETAMOS LA TRANSACCION
		COMMIT TRANSACTION;

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION;

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH

	SET XACT_ABORT, NOCOUNT OFF
END
GO
PRINT N'Modificando [dbo].[AccountSubAccountVCCBalanceAllocation]...';


GO
ALTER PROCEDURE [dbo].[AccountSubAccountVCCBalanceAllocation] (
	@Id VARCHAR(100) = NULL,
	@Step INT = 0,
	@PlatformHash VARCHAR(100) = NULL,
	@InstallationId VARCHAR(50) = NULL,
	@AccountId VARCHAR(50) = NULL,
	@SubAccountId VARCHAR(50) = NULL,
	@VCC VARCHAR(50) = NULL,
	@Balance DECIMAL(18,5) = 0,
	@Target VARCHAR(50) = NULL,
	@Operation VARCHAR(50) = NULL,
	@User VARCHAR(50) = NULL
	)
AS
BEGIN
	-- ESTE SP TIENE LA FINALIDAD DE ACTUALIZAR LOS SALDOS DE LAS CARTERAS DEL VCC
	SET XACT_ABORT, NOCOUNT ON

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY

		-- INICIAMOS LA TRANSACCION
		BEGIN TRANSACTION

		-- VERIFICAMOS SI ACCOUNTID TIENE VALOR
		IF @AccountId IS NULL
			-- LANZAMOS MENSAJE AVISANDO QUE FALTA ACCOUNTID
			RAISERROR('DATABASE_ERROR_MISSING_ACCOUNT_IDENTIFIER_PARAMETER', 16, 16)

		-- VERIFICAMOS QUE TENEMOS LOS PARAMETROS NECESARIOS
		IF @PlatformHash IS NULL OR @InstallationId IS NULL OR @AccountId IS NULL OR @VCC IS NULL
			-- LANZAMOS MENSAJE AVISANDO QUE FALTA ACCOUNTID
			RAISERROR('DATABASE_ERROR_MISSING_PARAMETERS', 16, 16)

		IF @Operation <> 'INCREASE' AND @Operation <> 'DECREASE'
			-- LANZAMOS MENSAJE AVISANDO QUE HAN SOLICITADO UNA OPERACION DESCONOCIDA
			RAISERROR('DATABASE_ERROR_UNKNOWN_OPERATION', 16, 16)

		-- DECLARAMOS LA VARIABLE EN LA QUE ESCRIBIREMOS LA DESCRIPCION DEL MOVIMIENTO
		DECLARE @MOVEMENT_DESCRIPTION NVARCHAR(250) = NULL

		-- SIEMPRE REGISTRAMOS LOS MOVIMIENTOS INCLUSO SI EL SALDO A AGREGAR ES CERO
		IF @Balance >= 0
		BEGIN
			-- VALIDAMOS SI EXISTE LA CUENTA INGRESADA
			IF NOT EXISTS (
				SELECT AB.[ID] 
					FROM [dbo].[AccountBalance] AB (NOLOCK)
					WHERE AB.[PlatformHash] = @PlatformHash
						AND AB.[InstallationId] = @InstallationId 
						AND AB.[AccountId] = @AccountId
						AND AB.[SubAccountId] IS NULL
			)
				-- LANZAMOS MESAJE AVISANDO QUE NO SE ENCONTRO LA CUENTA
				RAISERROR('DATABASE_ERROR_NOT_FOUND_ACCOUNT', 16, 16)

			-- EN CASO SUBACCOUNT SEA DIFERENTE DE NULO
			IF @SubAccountId IS NOT NULL 
				IF NOT EXISTS (
					-- BUSCAMOS EL IDENTIFICADOR UNICO DE LA SUBCUENTA EN LA TABLA ACCOUNTBALANCE
					SELECT AB.[ID] 
						FROM [dbo].[AccountBalance] AB (NOLOCK)
						WHERE AB.[PlatformHash] = @PlatformHash
							AND AB.[InstallationId] = @InstallationId 
							AND AB.[AccountId] = @AccountId
							AND AB.[SubAccountId] = @SubAccountId
				)
					-- LANZAMOS MESAJE AVISANDO QUE NO SE ENCONTRO LA CUENTA
					RAISERROR('DATABASE_ERROR_NOT_FOUND_SUBACCOUNT', 16, 16)

			-- VALIDAMOS SI EXISTE EL VCC CUANDO SUBACCOUNT ES NULO
			IF @SubAccountId IS NULL
			BEGIN
				-- VALIDAMOS SI EXISTE EL VCC
				IF NOT EXISTS (
					SELECT 1 
						FROM [dbo].[AccountVCC] AV (NOLOCK)
						WHERE AV.[InstallationId] = @InstallationId 
							AND AV.[AccountId] = @AccountId
							AND AV.[VCC] = @VCC
				)
					-- LANZAMOS MESAJE AVISANDO QUE NO SE ENCONTRO EL VCC
					RAISERROR('DATABASE_ERROR_NOT_FOUND_VCC', 16, 16)
			END
			-- VALIDAMOS SI EXISTE EL VCC CUANDO SUBACCOUNT ES DIFERENTE DE NULO
			ELSE 
			BEGIN
				-- VALIDAMOS SI EXISTE EL VCC
				IF NOT EXISTS (
					SELECT 1 
						FROM [dbo].[AccountVCC] AV (NOLOCK)
						WHERE AV.[InstallationId] = @InstallationId 
							AND AV.[AccountId] = @AccountId
							AND AV.[SubAccountId] = @SubAccountId
							AND AV.[VCC] = @VCC
				)
					-- LANZAMOS MESAJE AVISANDO QUE NO SE ENCONTRO EL VCC
					RAISERROR('DATABASE_ERROR_NOT_FOUND_VCC', 16, 16)
			END

			-- PARA VALIDAR QUE EXISTE LA CARTERA SOLICITADA BASTA CON CONSULTAR CUENTA Y VCC
			IF NOT EXISTS (
				SELECT 1 
					FROM [dbo].[AccountVCCBalance] AB (NOLOCK)
					WHERE AB.[InstallationId] = @InstallationId 
						AND AB.[AccountId] = @AccountId
						AND AB.[VCC] = @VCC
						AND AB.[SystemService] = @Target
			)
				-- LANZAMOS MESAJE AVISANDO QUE NO SE ENCONTRO LA CUENTA
				RAISERROR('DATABASE_ERROR_NOT_FOUND_TARGET', 16, 16)
				
			-- PASADAS LAS VALIDACIONES BASICAS, APUNTAMOS A HACER LA ACTUALIZACION DE SALDOS

			-- OBTENEMOS EL CURRENCYCODE Y CURRENCYSYMBOL DE LA CUENTA
			DECLARE @CURRENCY_CODE VARCHAR(5) = NULL
			DECLARE @CURRENCY_SYMBOL NVARCHAR(10) = NULL

			SELECT @CURRENCY_CODE = A.[CurrencyCode],
					@CURRENCY_SYMBOL = C.[Symbol]
				FROM [dbo].[Account] A (NOLOCK)
				INNER JOIN [dbo].[Currency] C (NOLOCK)
					ON C.[Code] = A.[CurrencyCode]
				WHERE A.[InstallationId] = @InstallationId 
					AND A.[AccountId] = @AccountId
					AND A.[SubAccountId] IS NULL

			-- PARA EL CASO QUE ESTEMOS REDUCIENDO EL SALDO, SALE DE TARGET -> FREE
			IF @Operation = 'DECREASE'
			BEGIN
				-- VALIDAMOS QUE EL VCC TIENE SADO SUFICIENTE EL EL SERVICIO SELECCIONADO
				IF 1 = (
					SELECT 1
					FROM [dbo].[AccountVCCBalance] (NOLOCK)
					WHERE [InstallationId] = @InstallationId 
						AND [AccountId] = @AccountId
						AND [VCC] = @VCC
						AND [SystemService] = @Target
						AND [AvailableBalance] >= @Balance
				)
				BEGIN
					-- BLOQUEMOS TEMPORALMENTE EL REGISTRO DE SALDOS DEL SERVICO USANDO LOS MISMO PARAMETROS DE BUSQUEDA Y 
					-- ASEGURANDONOS QUE EL SALDO DISPONIBLE AUN ES MAYOR QUE EL SALDO A SETTEAR
					UPDATE [dbo].[AccountVCCBalance] 
						SET [IsLocked] = 1,
							[LastModifiedByUserId] = @User,
							[LastModifiedDate] = GETUTCDATE(),
							[TimeStamp] = GETUTCDATE() 
						WHERE [InstallationId] = @InstallationId 
							AND [AccountId] = @AccountId
							AND [VCC] = @VCC
							AND [SystemService] = @Target
							AND [AvailableBalance] >= @Balance
	
					IF @@ROWCOUNT = 0
						RAISERROR('DATABASE_ERROR_INSUFFICIENT_SERVICE_BALANCE', 16, 16)

					-- UNA VEZ BLOQUEADO EL REGISTRO DE LA TABLA ACCOUNTVCCBALANCE, REGISTRAMOS LAS SALIDAS Y ENTRADAS EN 
					-- LA TABLA DE MOVIMIENTOS ACCOUNSUBACCOUNTVCCBALANCEMOVEMENT (MOVIMIENTO DE SERVICIO TARGET -> FREE)
					-- DEFINIMOS LA DESCRIPCION DEL MOVIMIENTO PARA ESTE BLOQUE
					SET @MOVEMENT_DESCRIPTION = CONCAT('VCC ', @VCC, ': DISCOUNT ', @CURRENCY_CODE, ': ', ISNULL(@CURRENCY_SYMBOL, ''), ' ', @Balance, ' FROM ', @Target, ' WALLET TO FREE WALLET FROM BILLING BALANCE ALLOCATION MODULE') 

					-- INSERTAMOS EN LA TABLA ACCOUNSUBACCOUNTVCCBALANCEMOVEMENT EL REGISTRO PARA LA SALIDA DE LA CARTERA TARGET
					INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
								([ID], [InstallationId], [AccountId], [SubAccountId], [VCC]
								,[Wallet] ,[Type] ,[CurrentStep] ,[TotalSteps] ,[Description]
								,[CurrencyCode] ,[Amount] ,[PrevAvailableBalance] ,[PostAvailableBalance] ,[PrevCountableBalance] 
								,[PostCountableBalance], [CreatedDate])
						SELECT @Id ,AVB.[InstallationId] ,AVB.[AccountId] ,AVB.[SubAccountId] ,AVB.[VCC]
								,AVB.[SystemService] ,'DISCOUNT' ,(@Step * 2) + 1 ,6 ,@MOVEMENT_DESCRIPTION
								,@CURRENCY_CODE ,@Balance ,AVB.[AvailableBalance] ,AVB.[AvailableBalance] - @Balance, AVB.[CountableBalance] 
								,AVB.[CountableBalance] - @Balance ,GETUTCDATE()
							FROM [dbo].[AccountVCCBalance] AVB (NOLOCK)
							WHERE [InstallationId] = @InstallationId 
								AND [AccountId] = @AccountId
								AND [VCC] = @VCC
								AND [SystemService] = @Target
								AND [AvailableBalance] > @Balance

					-- INSERTAMOS EN LA TABLA ACCOUNSUBACCOUNTVCCBALANCEMOVEMENT EL REGISTRO PARA LA ENTRADA A LA CARTERA FREE
					INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
								([ID] ,[InstallationId] ,[AccountId] ,[SubAccountId] ,[VCC]
								,[Wallet] ,[Type] ,[CurrentStep] ,[TotalSteps] ,[Description]
								,[CurrencyCode] ,[Amount] ,[PrevAvailableBalance] ,[PostAvailableBalance] ,[PrevCountableBalance] 
								,[PostCountableBalance] ,[CreatedDate])
						SELECT @Id ,AVB.[InstallationId] ,AVB.[AccountId] ,AVB.[SubAccountId] ,AVB.[VCC]
								,AVB.[SystemService] ,'CHARGE' ,(@Step * 2) + 2 ,6 ,@MOVEMENT_DESCRIPTION
								,@CURRENCY_CODE ,@Balance ,AVB.[AvailableBalance] ,AVB.[AvailableBalance] + @Balance ,AVB.[CountableBalance] 
								,AVB.[CountableBalance] + @Balance ,GETUTCDATE()
							FROM [dbo].[AccountVCCBalance] AVB (NOLOCK)
							WHERE [InstallationId] = @InstallationId 
								AND [AccountId] = @AccountId
								AND [VCC] = @VCC
								AND [SystemService] = 'FREE'

					-- DESBLOQUEMOS EL REGISTRO DE SALDOS DEL SERVICO USANDO LOS MISMO PARAMETROS DE BUSQUEDA Y ACTUALIZAMOS EL SALDO DISPONIBLE Y CONTABLE
					UPDATE [dbo].[AccountVCCBalance] 
						SET [IsLocked] = 0, 
							[AvailableBalance] = [AvailableBalance] - @Balance,
							[CountableBalance] = [CountableBalance] - @Balance,
							[LastModifiedByUserId] = @User,
							[LastModifiedDate] = GETUTCDATE(),
							[TimeStamp] = GETUTCDATE() 
						WHERE [InstallationId] = @InstallationId 
							AND [AccountId] = @AccountId
							AND [VCC] = @VCC
							AND [SystemService] = @Target
							AND [AvailableBalance] >= @Balance

					IF @@ROWCOUNT = 0
						RAISERROR('DATABASE_ERROR_INSUFFICIENT_SERVICE_BALANCE', 16, 16)

					-- ACTUALIZAMOS EL REGISTRO DE SALDO DEL FREE
					UPDATE [dbo].[AccountVCCBalance] 
						SET [AvailableBalance] = [AvailableBalance] + @Balance,
							[CountableBalance] = [CountableBalance] + @Balance,
							[LastModifiedByUserId] = @User,
							[LastModifiedDate] = GETUTCDATE(),
							[TimeStamp] = GETUTCDATE() 
						WHERE [InstallationId] = @InstallationId 
							AND [AccountId] = @AccountId
							AND [VCC] = @VCC
							AND [SystemService] = 'FREE'

				END 
				ELSE
					-- LANZAMOS MESAJE AVISANDO QUE NO HAY SUFICIENTE SALDO DISPONIBLE EN EL SERVICIO DEL SISTEMA
					RAISERROR('DATABASE_ERROR_INSUFFICIENT_SERVICE_AVAILABLE_BALANCE', 16, 16)

			END 

			-- PARA EL CASO QUE ESTEMOS AUMENTANDO EL SALDO, SALE DE  FREE -> TARGET
			ELSE IF @Operation = 'INCREASE' 
			BEGIN
				-- VALIDAMOS QUE EL VCC TIENE SADO SUFICIENTE EN LA CARTERA FREE
				IF 1 = (
					SELECT 1
					FROM [dbo].[AccountVCCBalance] (NOLOCK)
					WHERE [InstallationId] = @InstallationId 
						AND [AccountId] = @AccountId
						AND [VCC] = @VCC
						AND [SystemService] = 'FREE'
						AND [AvailableBalance] >= @Balance
				)
				BEGIN
					-- BLOQUEMOS TEMPORALMENTE EL REGISTRO DE SALDOS DE LA CARTERA FREE USANDO LOS MISMO PARAMETROS DE BUSQUEDA Y 
					-- ASEGURANDONOS QUE EL SALDO DISPONIBLE AUN ES MAYOR QUE EL SALDO A SETTEAR
					UPDATE [dbo].[AccountVCCBalance] 
						SET [IsLocked] = 1,
							[LastModifiedByUserId] = @User,
							[LastModifiedDate] = GETUTCDATE(),
							[TimeStamp] = GETUTCDATE() 
						WHERE [InstallationId] = @InstallationId 
							AND [AccountId] = @AccountId
							AND [VCC] = @VCC
							AND [SystemService] = 'FREE'
							AND [AvailableBalance] >= @Balance
	
					IF @@ROWCOUNT = 0
						RAISERROR('DATABASE_ERROR_INSUFFICIENT_FREE_BALANCE', 16, 16)

					-- UNA VEZ BLOQUEADO EL REGISTRO DE LA TABLA ACCOUNTVCCBALANCE, REGISTRAMOS LAS SALIDAS Y ENTRADAS EN 
					-- LA TABLA DE MOVIMIENTOS ACCOUNSUBACCOUNTVCCBALANCEMOVEMENT (MOVIMIENTO DE SERVICIO TARGET -> FREE)
					-- DEFINIMOS LA DESCRIPCION DEL MOVIMIENTO PARA ESTE BLOQUE
					SET @MOVEMENT_DESCRIPTION = CONCAT('VCC ', @VCC, ': CHARGE ', @CURRENCY_CODE, ': ', ISNULL(@CURRENCY_SYMBOL, ''), ' ', @Balance, ' FROM FREE WALLET TO ', @Target, ' WALLET FROM BILLING BALANCE ALLOCATION MODULE') 

					-- INSERTAMOS EN LA TABLA ACCOUNSUBACCOUNTVCCBALANCEMOVEMENT EL REGISTRO PARA LA SALIDA DE LA CARTERA FREE
					INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
								([ID] ,[InstallationId] ,[AccountId] ,[SubAccountId] ,[VCC]
								,[Wallet] ,[Type] ,[CurrentStep] ,[TotalSteps] ,[Description]
								,[CurrencyCode] ,[Amount] ,[PrevAvailableBalance] ,[PostAvailableBalance] ,[PrevCountableBalance] 
								,[PostCountableBalance] ,[CreatedDate])
						SELECT @Id ,AVB.[InstallationId] ,AVB.[AccountId] ,AVB.[SubAccountId] ,AVB.[VCC]
								,AVB.[SystemService] ,'CHARGE' ,(@Step * 2) + 1 ,6 ,@MOVEMENT_DESCRIPTION
								,@CURRENCY_CODE ,@Balance ,AVB.[AvailableBalance] ,AVB.[AvailableBalance] - @Balance ,AVB.[CountableBalance] 
								,AVB.[CountableBalance] - @Balance ,GETUTCDATE()
							FROM [dbo].[AccountVCCBalance] AVB (NOLOCK)
							WHERE [InstallationId] = @InstallationId 
								AND [AccountId] = @AccountId
								AND [VCC] = @VCC
								AND [SystemService] = 'FREE'
								AND [AvailableBalance] > @Balance

					-- INSERTAMOS EN LA TABLA ACCOUNSUBACCOUNTVCCBALANCEMOVEMENT EL REGISTRO PARA LA ENTRADA A LA CARTERA TARGET
					INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
								([ID] ,[InstallationId] ,[AccountId] ,[SubAccountId] ,[VCC]
								,[Wallet] ,[Type] ,[CurrentStep] ,[TotalSteps] ,[Description]
								,[CurrencyCode] ,[Amount] ,[PrevAvailableBalance] ,[PostAvailableBalance] ,[PrevCountableBalance] 
								,[PostCountableBalance], [CreatedDate])
						SELECT @Id ,AVB.[InstallationId] ,AVB.[AccountId] ,AVB.[SubAccountId] ,AVB.[VCC]
								,AVB.[SystemService] ,'DISCOUNT' ,(@Step * 2) + 2 ,6 ,@MOVEMENT_DESCRIPTION
								,@CURRENCY_CODE ,@Balance ,AVB.[AvailableBalance] ,AVB.[AvailableBalance] + @Balance ,AVB.[CountableBalance] 
								,AVB.[CountableBalance] + @Balance, GETUTCDATE()
							FROM [dbo].[AccountVCCBalance] AVB (NOLOCK)
							WHERE [InstallationId] = @InstallationId 
								AND [AccountId] = @AccountId
								AND [VCC] = @VCC
								AND [SystemService] = @Target

					-- DESBLOQUEMOS EL REGISTRO DE SALDOS DE LA CARTERA FREE USANDO LOS MISMO PARAMETROS DE BUSQUEDA Y ACTUALIZAMOS EL SALDO DISPONIBLE Y CONTABLE
					UPDATE [dbo].[AccountVCCBalance] 
						SET [IsLocked] = 0, 
							[AvailableBalance] = [AvailableBalance] - @Balance,
							[CountableBalance] = [CountableBalance] - @Balance
						WHERE [InstallationId] = @InstallationId 
							AND [AccountId] = @AccountId
							AND [VCC] = @VCC
							AND [SystemService] = 'FREE'
							AND [AvailableBalance] >= @Balance

					IF @@ROWCOUNT = 0
						RAISERROR('DATABASE_ERROR_INSUFFICIENT_FREE_BALANCE', 16, 16)

					-- ACTUALIZAMOS EL REGISTRO DE SALDO DEL FREE
					UPDATE [dbo].[AccountVCCBalance] 
						SET [AvailableBalance] = [AvailableBalance] + @Balance,
							[CountableBalance] = [CountableBalance] + @Balance
						WHERE [InstallationId] = @InstallationId 
							AND [AccountId] = @AccountId
							AND [VCC] = @VCC
							AND [SystemService] = @Target

				END 
				ELSE
					-- LANZAMOS MESAJE AVISANDO QUE NO HAY SUFICIENTE SALDO DISPONIBLE EN EL SERVICIO DEL SISTEMA
					RAISERROR('DATABASE_ERROR_INSUFFICIENT_SERVICE_AVAILABLE_BALANCE', 16, 16)
			END

			-- PARA EL CASO QUE EL SALDO NO AUMENTE NI DISMINUYE
			ELSE IF @Operation = 'NONE' 
			BEGIN
				-- DEFINIMOS LA DESCRIPCION DEL MOVIMIENTO PARA ESTE BLOQUE
				SET @MOVEMENT_DESCRIPTION = CONCAT('VCC ', @VCC, ': NO OPERATION (', ISNULL(@CURRENCY_SYMBOL, ''), ' ', @Balance, ') ON ', @Target, ' WALLET FROM BILLING BALANCE ALLOCATION MODULE') 

			END
			ELSE 
				-- LANZAMOS MESAJE AVISANDO QUE ES UNA OPERACION NO SOPORTADA
				RAISERROR('DATABASE_ERROR_UNSUPPORTED_OPERATION', 16, 16)
			
		END

		-- RETORNAMOS RESULTADO 1
		SELECT 1 'result'

		-- COMPLETAMOS LA TRANSACCION
		COMMIT TRANSACTION

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH

END
GO
PRINT N'Modificando [dbo].[AccountSubAccountVCCBalanceCreate]...';


GO
-- ACCOUNTSUBACCOUNTVCCBALANCREATE

ALTER  PROCEDURE [dbo].[AccountSubAccountVCCBalanceCreate] (
	@PlatformHash VARCHAR(100) = NULL,
	@InstallationId VARCHAR(50) = NULL,
	@AccountId VARCHAR(50) = NULL,
	@SubAccountId VARCHAR(50) = NULL,
	@VCC VARCHAR(50) = NULL,
	@Balance DECIMAL(18,5) = 0,
	@User VARCHAR(50) = NULL
	)
AS
BEGIN
	SET XACT_ABORT, NOCOUNT ON;

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY
        
		-- INICIAMOS LA TRANSACCION
		BEGIN TRANSACTION

		-- VERIFICAMOS SI ACCOUNTID TIENE VALOR
		IF @AccountId IS NULL
			-- LANZAMOS MENSAJE AVISANDO QUE FALTA ACCOUNTID
			RAISERROR('DATABASE_ERROR_MISSING_ACCOUNT_IDENTIFIER_PARAMETER', 16, 16)
		
		-- DECLARAMOS UNA VARIABLE PARA ALMACENAR EL IDENTIFICADOR UNICO DE LA CUENTA EN LA TABLA ACCOUNTBALANCE
		DECLARE @ACCOUNT_ID VARCHAR(50) = NULL
		DECLARE @SUBACCOUNT_ID VARCHAR(50) = NULL
		DECLARE @MOVEMENT_ID VARCHAR(50) = NEWID()
		DECLARE @MOVEMENT_DESCRIPTION NVARCHAR(250) = NULL

		-- OBTENEMOS EL CURRENCYCODE Y CURRENCYSYMBOL DE LA CUENTA
		DECLARE @CURRENCY_CODE VARCHAR(5) = NULL
		DECLARE @CURRENCY_SYMBOL NVARCHAR(10) = NULL

		SELECT @CURRENCY_CODE = A.[CurrencyCode],
				@CURRENCY_SYMBOL = C.[Symbol]
			FROM [dbo].[Account] A (NOLOCK)
			INNER JOIN [dbo].[Currency] C (NOLOCK)
				ON C.[Code] = A.[CurrencyCode]
			WHERE A.[InstallationId] = @InstallationId 
				AND A.[AccountId] = @AccountId
				AND A.[SubAccountId] IS NULL

		-- CUANDO SUBACCOUNT Y VCC SON NULOS ESTAMOS CREANDO UNA CUENTA
		IF @SubAccountId IS NULL AND @VCC IS NULL 
		BEGIN
			-- INSERTAMOS EL NUEVO REGISTRO EN LA TABLA ACCOUNTBALANCE CON @BALANCE COMO SALDO INICIAL
			INSERT INTO [dbo].[AccountBalance]
						([PlatformHash],[ID],[InstallationId],[AccountId]
						,[TotalBalance],[AvailableBalance],[AssignedBalance],[IsLocked],[CreatedDate]
						,[CreatedByUserId],[TimeStamp])
					VALUES
						(@PlatformHash,NEWID(),@InstallationId,@AccountId
						,@Balance,@Balance,0,0,GETUTCDATE()
						,@User,GETUTCDATE())
			
			-- DEFINIMOS LA DESCRIPCION DEL MOVIMIENTO PARA ESTE BLOQUE
			SET @MOVEMENT_DESCRIPTION = CONCAT('ACCOUNT ', @AccountId, ': FIRST CHARGE ', @CURRENCY_CODE, ': ', ISNULL(@CURRENCY_SYMBOL, ''), @Balance, ' TO FREE WALLET(ACCOUNT ', @AccountId, ') FROM BILLING CREATE ACCOUNT MODULE') 

			-- INSERTAMOS EN LA TABLA ACCOUNSUBACCOUNTVCCBALANCEMOVEMENT EL REGISTRO PARA EL PRIMER INGRESO EN LA CARTERA FREE
			INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
						([ID] ,[InstallationId] ,[AccountId] ,[Wallet] ,[Type]
						,[CurrentStep] ,[TotalSteps] ,[Description] ,[Amount] ,[CurrencyCode]
						,[PostAvailableBalance] ,[PostCountableBalance], [CreatedDate])
					VALUES
						(@MOVEMENT_ID ,@InstallationId ,@AccountId ,'FREE' ,'CHARGE'
						,1 ,1 ,@MOVEMENT_DESCRIPTION ,@Balance ,@CURRENCY_CODE
						,@Balance ,@Balance ,GETUTCDATE())
			
		END
		-- CUANDO SUBACCOUNT ES DIFERENTE DE NULO Y VCC ES NULO ESTAMOS CREANDO UNA SUBCUENTA
		ELSE IF @SubAccountId IS NOT NULL AND @VCC IS NULL 
		BEGIN
			-- BUSCAMOS EL IDENTIFICADOR UNICO DE LA CUENTA EN LA TABLA ACCOUNTBALANCE
			SELECT @ACCOUNT_ID = AB.[ID] 
				FROM [dbo].[AccountBalance] AB (NOLOCK)
				WHERE AB.[PlatformHash] = @PlatformHash
					AND AB.[InstallationId] = @InstallationId 
					AND AB.[AccountId] = @AccountId
					AND AB.[SubAccountId] IS NULL

			-- VALIDAMOS SI EXISTE EL IDENTIFICADOR UNICO
			IF @ACCOUNT_ID IS NOT NULL
			BEGIN
				-- VERIFICAMOS QUE LA CUENTA TIENE SALDO DISPONIBLE MAYOR O IGUAL AL BALANCE QUE ESTAMOS SETTEANDO
				IF 1 = (SELECT 1 
						FROM [dbo].[AccountBalance] AB 
						WHERE AB.[ID] = @ACCOUNT_ID
							AND AB.[AvailableBalance] >= @Balance) 
				BEGIN
					-- BLOQUEMOS TEMPORALMENTE EL REGISTRO DE SALDOS DE LA CUENTA USANDO EL IDENTIFICADOR UNICO Y ASEGURANDONOS QUE 
					-- EL SALDO DISPONIBLE AUN ES MAYOR QUE EL SALDO A SETTEAR
					UPDATE [dbo].[AccountBalance] 
						SET [IsLocked] = 1 
						WHERE [ID] = @ACCOUNT_ID
							AND [AvailableBalance] >= @Balance

					IF @@ROWCOUNT = 0
						RAISERROR('DATABASE_ERROR_INSUFFICIENT_ACCOUNT_BALANCE', 16, 16)

					-- INSERTAMOS EL NUEVO REGISTRO EN LA TABLA ACCOUNTBALANCE CON @BALANCE COMO SALDO INICIAL DE LA SUBCUENTA
					INSERT INTO [dbo].[AccountBalance]
								([PlatformHash],[ID],[InstallationId],[AccountId],[SubAccountId]
								,[TotalBalance],[AvailableBalance],[AssignedBalance],[IsLocked],[CreatedDate]
								,[CreatedByUserId],[TimeStamp])
							VALUES
								(@PlatformHash,NEWID(),@InstallationId,@AccountId,@SubAccountId
								,@Balance,@Balance,0,0,GETUTCDATE()
								,@User,GETUTCDATE())

					-- UNA VEZ BLOQUEADO EL REGISTRO DE LA TABLA ACCOUNTBALANCE, REGISTRAMOS LAS SALIDAS Y ENTRADAS EN 
					-- LA TABLA DE MOVIMIENTOS ACCOUNSUBACCOUNTVCCBALANCEMOVEMENT (MOVIMIENTO DE ACCOUNT FREE -> SUBACCOUNT FREE)

					-- DEFINIMOS LA DESCRIPCION DEL MOVIMIENTO PARA ESTE BLOQUE
					SET @MOVEMENT_DESCRIPTION = CONCAT('SUBACCOUNT ', @SubAccountId, ': FIRST CHARGE ', @CURRENCY_CODE, ': ', ISNULL(@CURRENCY_SYMBOL, ''), ' ', @Balance, ' TO FREE WALLET(SUBACCOUNT ', @SubAccountId, ') FROM FREE WALLET(ACCOUNT ', @AccountId + ')  FROM BILLING CREATE ACCOUNT MODULE') 

					-- EL PRIMER MOVIMIENTO QUE REGISTRAMOS ES LA SALIDA(DISCOUNT) DE LA CARTERA FREE DE LA CUENTA
					INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
								([ID] ,[InstallationId] ,[AccountId] ,[Wallet] ,[Type]
								,[CurrentStep] ,[TotalSteps] ,[Description] ,[Amount] ,[CurrencyCode]
								,[PrevAvailableBalance] ,[PostAvailableBalance] ,[PrevCountableBalance] ,[PostCountableBalance]
								,[CreatedDate])
						SELECT @MOVEMENT_ID ,@InstallationId ,@AccountId ,'FREE' ,'DISCOUNT'
								,1 ,3 ,@MOVEMENT_DESCRIPTION ,@Balance ,@CURRENCY_CODE
								,AB.[AvailableBalance] ,AB.[AvailableBalance] - @Balance ,AB.[AvailableBalance] ,AB.[AvailableBalance] - @Balance
								,GETUTCDATE()
							FROM [dbo].[AccountBalance] AB 
							WHERE [ID] = @ACCOUNT_ID
									
					-- EL SEGUNDO MOVIMIENTO QUE REGISTRAMOS ES LA ENTRADA(CHARGE) A LA CARTERA FREE DE LA SUBCUENTA
					INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
						([ID] ,[InstallationId] ,[AccountId] ,[SubAccountId] ,[Wallet]
						,[Type] ,[CurrentStep] ,[TotalSteps] ,[Description] ,[CurrencyCode]
						,[Amount] ,[PostAvailableBalance] ,[PostCountableBalance] ,[CreatedDate])
					VALUES
						(@MOVEMENT_ID ,@InstallationId ,@AccountId ,@SubAccountId ,'FREE'
						,'CHARGE' ,2 ,3 ,@MOVEMENT_DESCRIPTION ,@CURRENCY_CODE
						,@Balance ,@Balance ,@Balance ,GETUTCDATE())

					-- EL TERCER MOVIMIENTO QUE REGISTRAMOS ES LA ENTRADA(CHARGE) A LA CARTERA ASSIGNED DE LA CUENTA
					INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
								([ID] ,[InstallationId] ,[AccountId] ,[Wallet] ,[Type]
								,[CurrentStep] ,[TotalSteps] ,[Description] ,[Amount] ,[CurrencyCode]
								,[PrevAvailableBalance] ,[PostAvailableBalance] ,[PrevCountableBalance] ,[PostCountableBalance]
								,[CreatedDate])
						SELECT @MOVEMENT_ID ,@InstallationId ,@AccountId ,'ASSIGNED' ,'CHARGE'
								,3 ,3 ,@MOVEMENT_DESCRIPTION ,@Balance ,@CURRENCY_CODE
								,AB.[AssignedBalance] ,AB.[AssignedBalance] + @Balance ,AB.[AssignedBalance] ,AB.[AssignedBalance] + @Balance
								,GETUTCDATE()
							FROM [dbo].[AccountBalance] AB 
							WHERE [ID] = @ACCOUNT_ID
			
					-- DESBLOQUEMOS EL REGISTRO DE SALDOS DE LA CUENTA USANDO EL IDENTIFICADOR UNICO Y ACTUALIZAMOS EL SALDO DISPONIBLE
					UPDATE [dbo].[AccountBalance] 
						SET [IsLocked] = 0, 
							[AvailableBalance] = [AvailableBalance] - @Balance,
							[AssignedBalance] = [AssignedBalance] + @Balance
						WHERE [ID] = @ACCOUNT_ID
							AND [AvailableBalance] >= @Balance

					IF @@ROWCOUNT = 0
						RAISERROR('DATABASE_ERROR_INSUFFICIENT_ACCOUNT_BALANCE', 16, 16)

				END
				ELSE 
					-- LANZAMOS MESAJE AVISANDO QUE NO SE ENCONTRO LA CUENTA
					RAISERROR('DATABASE_ERROR_INSUFFICIENT_ACCOUNT_BALANCE', 16, 16)
			END
			ELSE
			BEGIN
				-- LANZAMOS MESAJE AVISANDO QUE NO SE ENCONTRO LA CUENTA
				RAISERROR('DATABASE_ERROR_NOT_FOUND_ACCOUNT', 16, 16)
			END
		END 
		-- CUANDO SUBACCOUNT ES NULO Y VCC ES DIFERENTE NULO ESTAMOS CREANDO UN VCC QUE PERTENECE A UNA CUENTA
		ELSE IF @SubAccountId IS NULL AND @VCC IS NOT NULL 
		BEGIN
			-- BUSCAMOS EL IDENTIFICADOR UNICO DE LA CUENTA EN LA TABLA ACCOUNTBALANCE
			SELECT @ACCOUNT_ID = AB.[ID] 
				FROM [dbo].[AccountBalance] AB (NOLOCK)
				WHERE AB.[PlatformHash] = @PlatformHash
					AND AB.[InstallationId] = @InstallationId 
					AND AB.[AccountId] = @AccountId
					AND AB.[SubAccountId] IS NULL

			-- VALIDAMOS SI EXISTE EL IDENTIFICADOR UNICO
			IF @ACCOUNT_ID IS NOT NULL
			BEGIN
				-- VERIFICAMOS QUE LA CUENTA TIENE SALDO DISPONIBLE MAYOR O IGUAL AL BALANCE QUE ESTAMOS SETTEANDO
				IF 1 = (SELECT 1 
						FROM [dbo].[AccountBalance] AB (NOLOCK)
						WHERE AB.[ID] = @ACCOUNT_ID
							AND AB.[AvailableBalance] >= @Balance) 
				BEGIN	
					
					-- BLOQUEMOS TEMPORALMENTE EL REGISTRO DE SALDOS DE LA CUENTA USANDO EL IDENTIFICADOR UNICO Y ASEGURANDONOS QUE 
					-- EL SALDO DISPONIBLE AUN ES MAYOR QUE EL SALDO A SETTEAR
					UPDATE [dbo].[AccountBalance] 
						SET [IsLocked] = 1,
							[LastModifiedByUserId] = @User,
							[LastModifiedDate] = GETUTCDATE(),
							[LastModifiedByRemoteUser] = 1
						WHERE [ID] = @ACCOUNT_ID
							AND [AvailableBalance] >= @Balance

					IF @@ROWCOUNT = 0
						RAISERROR('DATABABASE_ERROR_INSUFFICIENT_ACCOUNT_BALANCE', 16, 16)

					-- UNA VEZ BLOQUEADO EL REGISTRO DE LA TABLA ACCOUNTBALANCE, REGISTRAMOS LAS SALIDAS Y ENTRADAS EN LA TABLA DE 
					-- MOVIMIENTOS ACCOUNSUBACCOUNTVCCBALANCEMOVEMENT (MOVIMIENTO DE ACCOUNT FREE -> VCC REE)
					
					-- DEFINIMOS LA DESCRIPCION DEL MOVIMIENTO PARA ESTE BLOQUE
					SET @MOVEMENT_DESCRIPTION = CONCAT('VCC ', @VCC, ': FIRST CHARGE ', @CURRENCY_CODE, ': ', ISNULL(@CURRENCY_SYMBOL, ''), ' ', @Balance, ' TO FREE WALLET(VCC ', @VCC, ') FROM FREE WALLET(ACCOUNT ', @AccountId + ') FROM BILLING CREATE ACCOUNT MODULE') 

					-- EL PRIMER MOVIMIENTO QUE REGISTRAMOS ES LA SALIDA DE LA CUENTA
					INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
								([ID] ,[InstallationId] ,[AccountId] ,[Wallet] ,[Type]
								,[CurrentStep] ,[TotalSteps] ,[Description] ,[Amount] ,[CurrencyCode]
								,[PrevAvailableBalance] ,[PostAvailableBalance] ,[PrevCountableBalance] ,[PostCountableBalance]
								,[CreatedDate])
						SELECT @MOVEMENT_ID ,@InstallationId ,@AccountId ,'FREE' ,'DISCOUNT'
								,1 ,3 ,@MOVEMENT_DESCRIPTION ,@Balance ,@CURRENCY_CODE
								,AB.[AvailableBalance] ,AB.[AvailableBalance] - @Balance ,AB.[AvailableBalance] ,AB.[AvailableBalance] - @Balance
								,GETUTCDATE()
							FROM [dbo].[AccountBalance] AB (NOLOCK)
							WHERE [ID] = @ACCOUNT_ID
									
					-- EL SEGUNDO MOVIMIENTO QUE REGISTRAMOS ES LA ENTRADA AL VCC
					INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
						([ID] ,[InstallationId] ,[AccountId] ,[VCC] ,[Wallet]
						,[Type] ,[CurrentStep] ,[TotalSteps] ,[Description] ,[CurrencyCode]
						,[Amount] ,[PostAvailableBalance] ,[PostCountableBalance] ,[CreatedDate])
					VALUES
						(@MOVEMENT_ID ,@InstallationId ,@AccountId ,@VCC ,'FREE'
						,'CHARGE' ,2 ,3 ,@MOVEMENT_DESCRIPTION ,@CURRENCY_CODE
						,@Balance ,@Balance ,@Balance ,GETUTCDATE())

					-- EL TERCER MOVIMIENTO QUE REGISTRAMOS ES LA ENTRADA(CHARGE) A LA CARTERA ASSIGNED DE LA CUENTA
					INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
								([ID] ,[InstallationId] ,[AccountId] ,[Wallet] ,[Type]
								,[CurrentStep] ,[TotalSteps] ,[Description] ,[Amount] ,[CurrencyCode]
								,[PrevAvailableBalance] ,[PostAvailableBalance] ,[PrevCountableBalance] ,[PostCountableBalance]
								,[CreatedDate])
						SELECT @MOVEMENT_ID ,@InstallationId ,@AccountId ,'ASSIGNED' ,'CHARGE'
								,3 ,3 ,@MOVEMENT_DESCRIPTION ,@Balance ,@CURRENCY_CODE
								,AB.[AssignedBalance] ,AB.[AssignedBalance] + @Balance ,AB.[AssignedBalance] ,AB.[AssignedBalance] + @Balance
								,GETUTCDATE()
							FROM [dbo].[AccountBalance] AB 
							WHERE [ID] = @ACCOUNT_ID
			
					-- DESCBLOQUEMOS EL REGISTRO DE SALDOS DE LA CUENTA USANDO EL IDENTIFICADOR UNICO Y ACTUALIZAMOS EL SALDO DISPONIBLE
					UPDATE [dbo].[AccountBalance] 
						SET [IsLocked] = 0, 
							[AvailableBalance] = [AvailableBalance] - @Balance,
							[AssignedBalance] = [AssignedBalance] + @Balance,
							[LastModifiedByUserId] = @User,
							[LastModifiedDate] = GETUTCDATE(),
							[LastModifiedByRemoteUser] = 1
						WHERE [ID] = @ACCOUNT_ID
							AND [AvailableBalance] >= @Balance

					IF @@ROWCOUNT = 0
						RAISERROR('DATABASE_ERROR_INSUFFICIENT_ACCOUNT_BALANCE', 16, 16)

					-- ACTUALIZAMOS EL REGISTRO DEL WALLET FREE DEL VCC EN LA TABLA ACCOUNTVCCBALANCE
					UPDATE [dbo].[AccountVCCBalance]
						SET [AvailableBalance] = [AvailableBalance] + @Balance,
							[CountableBalance] = [CountableBalance] + @Balance,
							[LastModifiedByUserId] = @User,
							[LastModifiedDate] = GETUTCDATE()
						WHERE [InstallationId] = @InstallationId
							AND [AccountId] = @AccountId
							AND [VCC] = @VCC
							AND [SystemService] = 'FREE'

					IF @@ROWCOUNT = 0
						RAISERROR('DATABASE_ERROR_ACCOUNTVCC_BALANCE_UPDATE_FAILED', 16, 16)
					
				END
				ELSE 
					-- LANZAMOS MESAJE AVISANDO QUE NO SE ENCONTRO LA CUENTA
					RAISERROR('DATABABASE_ERROR_INSUFFICIENT_ACCOUNT_BALANCE', 16, 16)

			END
			ELSE
			BEGIN
				-- LANZAMOS MESAJE AVISANDO QUE NO SE ENCONTRO LA CUENTA
				RAISERROR('DATABASE_ERROR_NOT_FOUND_ACCOUNT', 16, 16)
			END
		END
		-- CUANDO SUBACCOUNT ES DIFERENTE DE NULO Y VCC ES DIFERENTE NULO ESTAMOS CREANDO UN VCC QUE PERTENECE A UNA SUBCUENTA
		ELSE IF @SubAccountId IS NOT NULL AND @VCC IS NOT NULL 
		BEGIN
			-- BUSCAMOS EL IDENTIFICADOR UNICO DE LA CUENTA EN LA TABLA ACCOUNTBALANCE
			SELECT @ACCOUNT_ID = AB.[ID] 
				FROM [dbo].[AccountBalance] AB (NOLOCK)
				WHERE AB.[PlatformHash] = @PlatformHash
					AND AB.[InstallationId] = @InstallationId 
					AND AB.[AccountId] = @AccountId
					AND AB.[SubAccountId] IS NULL

			-- VALIDAMOS SI EXISTE EL IDENTIFICADOR UNICO
			IF @ACCOUNT_ID IS NOT NULL
			BEGIN
				-- BUSCAMOS EL IDENTIFICADOR UNICO DE LA CUENTA EN LA TABLA ACCOUNTBALANCE
				SELECT @SUBACCOUNT_ID = AB.[ID] 
					FROM [dbo].[AccountBalance] AB (NOLOCK)
					WHERE AB.[PlatformHash] = @PlatformHash
						AND AB.[InstallationId] = @InstallationId 
						AND AB.[AccountId] = @AccountId
						AND AB.[SubAccountId] = @SubAccountId

				-- VALIDAMOS SI EXISTE EL IDENTIFICADOR UNICO
				IF @SUBACCOUNT_ID IS NOT NULL
				BEGIN
					-- VERIFICAMOS QUE LA CUENTA TIENE SALDO DISPONIBLE MAYOR O IGUAL AL BALANCE QUE ESTAMOS SETTEANDO
					IF 1 = (SELECT 1 
							FROM [dbo].[AccountBalance] AB 
							WHERE AB.[ID] = @SUBACCOUNT_ID
								AND AB.[AvailableBalance] >= @Balance) 
					BEGIN
						-- BLOQUEMOS TEMPORALMENTE EL REGISTRO DE SALDOS DE LA SUBCUENTA USANDO EL IDENTIFICADOR UNICO Y ASEGURANDONOS QUE 
						-- EL SALDO DISPONIBLE AUN ES MAYOR QUE EL SALDO A SETTEAR
						UPDATE [dbo].[AccountBalance] 
							SET [IsLocked] = 1 
							WHERE [ID] = @SUBACCOUNT_ID
								AND [AvailableBalance] >= @Balance
	
						IF @@ROWCOUNT = 0
							RAISERROR('DATABASE_ERROR_INSUFFICIENT_SUBACCOUNT_BALANCE', 16, 16)

						-- UNA VEZ BLOQUEADO EL REGISTRO DE LA TABLA ACCOUNTBALANCE, REGISTRAMOS LAS SALIDAS Y ENTRADAS EN 
						-- LA TABLA DE MOVIMIENTOS ACCOUNSUBACCOUNTVCCBALANCEMOVEMENT (MOVIMIENTO DE SUBACCOUNT FREE -> VCC FREE)

						-- DEFINIMOS LA DESCRIPCION DEL MOVIMIENTO PARA ESTE BLOQUE
						SET @MOVEMENT_DESCRIPTION = CONCAT('VCC ', @VCC, ': FIRST CHARGE ', @CURRENCY_CODE, ': ', ISNULL(@CURRENCY_SYMBOL, ''), ' ', @Balance, ' TO FREE WALLET(VCC ', @VCC, ') FROM FREE WALLET(SUBACCOUNT ', @SubAccountId, ') FROM BILLING CREATE ACCOUNT MODULE') 

						-- EL PRIMER MOVIMIENTO QUE REGISTRAMOS ES LA SALIDA(DISCOUNT) DE LA CARTERA FREE DE LA SUBCUENTA
						INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
									([ID] ,[InstallationId] ,[AccountId] ,[SubAccountId] ,[Wallet]
									,[Type] ,[CurrentStep] ,[TotalSteps] ,[Description] ,[CurrencyCode]
									,[Amount] ,[PrevAvailableBalance] ,[PostAvailableBalance] ,[PrevCountableBalance] ,[PostCountableBalance]
									,[CreatedDate])
							SELECT @MOVEMENT_ID ,@InstallationId ,@AccountId ,@SubAccountId ,'FREE'
									,'DISCOUNT' ,1 ,3 ,@MOVEMENT_DESCRIPTION ,@CURRENCY_CODE
									,@Balance ,AB.[AvailableBalance] ,AB.[AvailableBalance] - @Balance ,AB.[AvailableBalance] ,AB.[AvailableBalance] - @Balance
									,GETUTCDATE()
								FROM [dbo].[AccountBalance] AB 
								WHERE [ID] = @SUBACCOUNT_ID

						-- EL SEGUNDO MOVIMIENTO QUE REGISTRAMOS ES LA ENTRADA(CHARGE) A LA CARTERA FREE DEL VCC
						INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
							([ID] ,[InstallationId] ,[AccountId] ,[SubAccountId] ,[VCC]
							,[Wallet] ,[Type] ,[CurrentStep] ,[TotalSteps] ,[Description]
							,[CurrencyCode] ,[Amount] ,[PostAvailableBalance] ,[PostCountableBalance] ,[CreatedDate])
						VALUES
							(@MOVEMENT_ID ,@InstallationId ,@AccountId ,@SubAccountId ,@VCC
							,'FREE' ,'CHARGE' ,2 ,3 ,@MOVEMENT_DESCRIPTION
							,@CURRENCY_CODE ,@Balance ,@Balance ,@Balance ,GETUTCDATE())

						-- EL TERCER MOVIMIENTO QUE REGISTRAMOS ES LA ENTRADA(CHARGE) A LA CARTERA ASSIGNED DE LA SUBCUENTA
						INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
									([ID] ,[InstallationId] ,[AccountId] ,[SubAccountId] ,[Wallet]
									,[Type] ,[CurrentStep] ,[TotalSteps] ,[Description] ,[CurrencyCode]
									,[Amount] ,[PrevAvailableBalance] ,[PostAvailableBalance] ,[PrevCountableBalance] ,[PostCountableBalance] 
									,[CreatedDate])
							SELECT @MOVEMENT_ID ,@InstallationId ,@AccountId ,[SubAccountId] ,'ASSIGNED'
									,'CHARGE' ,3 ,3 ,@MOVEMENT_DESCRIPTION, @CURRENCY_CODE 
									,@Balance ,AB.[AssignedBalance] ,AB.[AssignedBalance] + @Balance ,AB.[AssignedBalance] ,AB.[AssignedBalance] + @Balance
									,GETUTCDATE()
								FROM [dbo].[AccountBalance] AB 
								WHERE [ID] = @SUBACCOUNT_ID
			
						-- DESBLOQUEMOS EL REGISTRO DE SALDOS DE LA SUBCUENTA USANDO EL IDENTIFICADOR UNICO Y ACTUALIZAMOS EL SALDO DISPONIBLE
						UPDATE [dbo].[AccountBalance] 
							SET [IsLocked] = 0, 
								[AvailableBalance] = [AvailableBalance] - @Balance,
								[AssignedBalance] = [AssignedBalance] + @Balance
							WHERE [ID] = @SUBACCOUNT_ID
								AND [AvailableBalance] >= @Balance

						IF @@ROWCOUNT = 0
							RAISERROR('DATABASE_ERROR_INSUFFICIENT_SUBACCOUNT_BALANCE', 16, 16)

						-- ACTUALIZAMOS EL REGISTRO DEL WALLET FREE DEL VCC EN LA TABLA ACCOUNTVCCBALANCE
						UPDATE [dbo].[AccountVCCBalance]
							SET [AvailableBalance] = @Balance,
								[CountableBalance] = @Balance,
								[LastModifiedByUserId] = @User,
								[LastModifiedDate] = GETUTCDATE()
							WHERE [InstallationId] = @InstallationId
								AND [AccountId] = @AccountId
								AND [SubAccountId] = @SubAccountId
								AND [VCC] = @VCC
								AND [SystemService] = 'FREE'

						IF @@ROWCOUNT = 0
							RAISERROR('DATABASE_ERROR_ACCOUNTVCC_BALANCE_UPDATE_FAILED', 16, 16)
					END 
					ELSE 
						-- LANZAMOS MESAJE AVISANDO QUE NO HAY SUFICIENTE SALDO EN LA SUBCUENTA
						RAISERROR('DATABASE_ERROR_INSUFFICIENT_SUBACCOUNT_BALANCE', 16, 16)
				END
				ELSE 
					-- LANZAMOS MESAJE AVISANDO QUE NO SE ENCONTRO LA SUBCUENTA
					RAISERROR('DATABASE_ERROR_NOT_FOUND_SUBACCOUNT', 16, 16)
			END
			ELSE 
				-- LANZAMOS MESAJE AVISANDO QUE NO SE ENCONTRO LA CUENTA
				RAISERROR('DATABASE_ERROR_NOT_FOUND_ACCOUNT', 16, 16)
		END

		-- COMPLETAMOS LA TRANSACCION
		COMMIT TRANSACTION;

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION;

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH

END
GO
PRINT N'Modificando [dbo].[AccountSubAccountVCCBalanceRecharge]...';


GO
ALTER PROCEDURE [dbo].[AccountSubAccountVCCBalanceRecharge] (
	@PlatformHash VARCHAR(100) = NULL,
	@InstallationId VARCHAR(50) = NULL,
	@AccountId VARCHAR(50) = NULL,
	@SubAccountId VARCHAR(50) = NULL,
	@VCC VARCHAR(50) = NULL,
	@Balance DECIMAL(18,5) = 0,
	@Target VARCHAR(50) = NULL,
	@User VARCHAR(50) = NULL
	)
AS
BEGIN
	SET XACT_ABORT, NOCOUNT ON;

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY
        
		-- INICIAMOS LA TRANSACCION
		BEGIN TRANSACTION

		IF @User IS NULL OR @User = '' SET @User = 'admin'

		-- VERIFICAMOS SI ACCOUNTID TIENE VALOR
		IF @AccountId IS NULL
			-- LANZAMOS MENSAJE AVISANDO QUE FALTA ACCOUNTID
			RAISERROR('DATABASE_ERROR_MISSING_ACCOUNT_IDENTIFIER_PARAMETER', 16, 16)

		-- DECLARAMOS UNA VARIABLE PARA ALMACENAR EL IDENTIFICADOR UNICO DE LA CUENTA EN LA TABLA ACCOUNTBALANCE
		DECLARE @ACCOUNT_ID VARCHAR(50) = NULL
		DECLARE @SUBACCOUNT_ID VARCHAR(50) = NULL
		DECLARE @MOVEMENT_ID VARCHAR(50) = NEWID()
		DECLARE @MOVEMENT_DESCRIPTION NVARCHAR(250) = NULL
		DECLARE @CURRENCY_CODE VARCHAR(5) = NULL
		DECLARE @CURRENCY_SYMBOL NVARCHAR(10) = NULL

		-- OBTENEMOS EL CURRENCYCODE Y CURRENCYSYMBOL DE LA CUENTA
		SELECT @CURRENCY_CODE = A.[CurrencyCode],
				@CURRENCY_SYMBOL = C.[Symbol]
			FROM [dbo].[Account] A (NOLOCK)
			INNER JOIN [dbo].[Currency] C (NOLOCK)
				ON C.[Code] = A.[CurrencyCode]
			WHERE A.[InstallationId] = @InstallationId 
				AND A.[AccountId] = @AccountId
				AND A.[SubAccountId] IS NULL

		-- USAMOS EL PARAMETRO TARGET PARA IDENTIFICAR A QUE NIVEL SE HARA LA RECARGA
		-- EN EL CASO DE RECARGAR CUENTA NO TENEMOS RESTRICCION SOBRE EL ORIGEN DEL SALDO
		IF @Target = 'ACCOUNT'
		BEGIN
			-- VERIFICAMOS QUE TENEMOS LOS PARAMETROS NECESARIOS
			IF @PlatformHash IS NULL OR @InstallationId IS NULL
				-- LANZAMOS MENSAJE AVISANDO QUE FALTA ACCOUNTID
				RAISERROR('DATABASE_ERROR_MISSING_PARAMETERS', 16, 16)

			-- BUSCAMOS EL IDENTIFICADOR UNICO DE LA CUENTA EN LA TABLA ACCOUNTBALANCE
			SELECT @ACCOUNT_ID = AB.[ID] 
				FROM [dbo].[AccountBalance] AB (NOLOCK)
				WHERE AB.[PlatformHash] = @PlatformHash
					AND AB.[InstallationId] = @InstallationId 
					AND AB.[AccountId] = @AccountId
					AND AB.[SubAccountId] IS NULL

			-- VALIDAMOS SI EXISTE EL IDENTIFICADOR UNICO
			IF @ACCOUNT_ID IS NOT NULL
			BEGIN
				-- DEFINIMOS LA DESCRIPCION DEL MOVIMIENTO PARA ESTE BLOQUE
				SET @MOVEMENT_DESCRIPTION = CONCAT('ACCOUNT ', @AccountId, ': CHARGE ', @CURRENCY_CODE, ': ', ISNULL(@CURRENCY_SYMBOL, ''), ' ', @Balance, ' TO FREE WALLET(ACCOUNT ', @AccountId, ') FROM BILLING RECHARGES MODULE)') 

				IF @@ROWCOUNT = 0
					RAISERROR('DATABASE_ERROR_UPDATE_ACCOUNT_BALANCE_FAILED', 16, 16)

				-- EL UNICO MOVIMIENTO QUE REGISTRAMOS ES LA ENTRADA(CHARGE) A LA CARTERA FREE DE LA CUENTA
				INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
							([ID] ,[InstallationId] ,[AccountId] ,[Wallet] ,[Type]
							,[CurrentStep] ,[TotalSteps] ,[Description] ,[CurrencyCode] ,[Amount] 
							,[PrevAvailableBalance] ,[PostAvailableBalance] ,[PrevCountableBalance] ,[PostCountableBalance] ,[CreatedDate] ,[CreatedByUserId])
					SELECT @MOVEMENT_ID ,@InstallationId ,@AccountId ,'FREE' ,'CHARGE'
							,1 ,1 ,@MOVEMENT_DESCRIPTION ,@CURRENCY_CODE ,@Balance 
							,AB.[AvailableBalance] ,AB.[AvailableBalance] + @Balance ,AB.[AvailableBalance] ,AB.[AvailableBalance] + @Balance ,GETUTCDATE() ,@User
						FROM [dbo].[AccountBalance] AB 
						WHERE [ID] = @ACCOUNT_ID
						
				-- DESBLOQUEMOS EL REGISTRO DE SALDOS DE LA CUENTA USANDO EL IDENTIFICADOR UNICO Y ACTUALIZAMOS EL SALDO DISPONIBLE
				UPDATE [dbo].[AccountBalance] 
					SET [IsLocked] = 0, 
						[TotalBalance] = [TotalBalance] + @Balance, 
						[AvailableBalance] = [AvailableBalance] + @Balance,
						[LastModifiedByUserId] = @User,
						[LastModifiedDate] = GETUTCDATE(),
						[TimeStamp] = GETUTCDATE()
					WHERE [ID] = @ACCOUNT_ID

			END
			ELSE
			BEGIN
				-- LANZAMOS MESAJE AVISANDO QUE NO SE ENCONTRO LA CUENTA
				RAISERROR('DATABASE_ERROR_NOT_FOUND_ACCOUNT', 16, 16)
			END
		END
		-- EN EL CASO DE SUBCUENTA DEBEMOS VALIDAR QUE EL SALDO A INSERTAR SEA MENOR O IGUAL AL DISPONIBLE DE SU 
		-- PADRE INMEDIATO(CUENTA)
		ELSE IF @TARGET = 'SUBACCOUNT'
		BEGIN
			-- BUSCAMOS EL IDENTIFICADOR UNICO DE LA CUENTA EN LA TABLA ACCOUNTBALANCE
			SELECT @ACCOUNT_ID = AB.[ID] 
				FROM [dbo].[AccountBalance] AB (NOLOCK)
				WHERE AB.[PlatformHash] = @PlatformHash
					AND AB.[InstallationId] = @InstallationId 
					AND AB.[AccountId] = @AccountId
					AND AB.[SubAccountId] IS NULL

			-- VALIDAMOS SI EXISTE EL IDENTIFICADOR UNICO
			IF @ACCOUNT_ID IS NOT NULL
			BEGIN
				-- BUSCAMOS EL IDENTIFICADOR UNICO DE LA SUBCUENTA EN LA TABLA ACCOUNTBALANCE
				SELECT @SUBACCOUNT_ID = AB.[ID] 
					FROM [dbo].[AccountBalance] AB (NOLOCK)
					WHERE AB.[PlatformHash] = @PlatformHash
						AND AB.[InstallationId] = @InstallationId 
						AND AB.[AccountId] = @AccountId
						AND AB.[SubAccountId] = @SubAccountId

				-- VALIDAMOS SI EXISTE EL IDENTIFICADOR UNICO
				IF @SUBACCOUNT_ID IS NOT NULL
				BEGIN
					-- VERIFICAMOS QUE LA CUENTA TIENE SALDO DISPONIBLE MAYOR O IGUAL AL BALANCE QUE ESTAMOS SETTEANDO
					IF 1 = (SELECT 1 
							FROM [dbo].[AccountBalance] AB 
							WHERE AB.[ID] = @ACCOUNT_ID
								AND AB.[AvailableBalance] >= @Balance) 
					BEGIN
						-- BLOQUEMOS TEMPORALMENTE EL REGISTRO DE SALDOS DE LA CUENTA USANDO EL IDENTIFICADOR UNICO Y ASEGURANDONOS QUE 
						-- EL SALDO DISPONIBLE AUN ES MAYOR QUE EL SALDO A SETTEAR
						UPDATE [dbo].[AccountBalance] 
							SET [IsLocked] = 1,
								[LastModifiedByUserId] = @User,
								[LastModifiedDate] = GETUTCDATE(),
								[TimeStamp] = GETUTCDATE() 
							WHERE [ID] = @ACCOUNT_ID
								AND [AvailableBalance] >= @Balance
	
						IF @@ROWCOUNT = 0
							RAISERROR('DATABASE_ERROR_INSUFFICIENT_ACCOUNT_BALANCE', 16, 16)

						-- UNA VEZ BLOQUEADO EL REGISTRO DE LA TABLA ACCOUNTBALANCE, REGISTRAMOS LAS SALIDAS Y ENTRADAS EN 
						-- LA TABLA DE MOVIMIENTOS ACCOUNSUBACCOUNTVCCBALANCEMOVEMENT (MOVIMIENTO DE ACCOUNT FREE -> SUBACCOUNT FREE)
						-- DEFINIMOS LA DESCRIPCION DEL MOVIMIENTO PARA ESTE BLOQUE
						SET @MOVEMENT_DESCRIPTION = CONCAT('SUBACCOUNT ', @SubAccountId, ': CHARGE ', @CURRENCY_CODE, ': ', ISNULL(@CURRENCY_SYMBOL, ''), ' ', @Balance, ' TO FREE WALLET(SUBACCOUNT ', @SubAccountId, ') FROM FREE WALLET(ACCOUNT ', @AccountId, ')') 

						-- EL PRIMER MOVIMIENTO QUE REGISTRAMOS ES LA SALIDA(DISCOUNT) DE LA CARTERA FREE DE LA CUENTA
						INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
									([ID] ,[InstallationId] ,[AccountId] ,[Wallet] ,[Type]
									,[CurrentStep] ,[TotalSteps] ,[Description] ,[CurrencyCode] ,[Amount] 
									,[PrevAvailableBalance] ,[PostAvailableBalance] ,[PrevCountableBalance] ,[PostCountableBalance] ,[CreatedDate] ,[CreatedByUserId])
							SELECT @MOVEMENT_ID ,@InstallationId ,@AccountId ,'FREE' ,'DISCOUNT'
									,1 ,3 ,@MOVEMENT_DESCRIPTION ,@CURRENCY_CODE ,@Balance 
									,AB.[AvailableBalance] ,AB.[AvailableBalance] - @Balance ,AB.[AvailableBalance] ,AB.[AvailableBalance] - @Balance, GETUTCDATE() ,@User
								FROM [dbo].[AccountBalance] AB 
								WHERE [ID] = @ACCOUNT_ID
									
						-- EL SEGUNDO MOVIMIENTO QUE REGISTRAMOS ES LA ENTRADA(CHARGE) A LA CARTERA FREE DE LA SUBCUENTA
						INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
							([ID] ,[InstallationId] ,[AccountId] ,[SubAccountId] ,[Wallet]
							,[Type] ,[CurrentStep] ,[TotalSteps] ,[Description] ,[CurrencyCode]
							,[Amount] ,[PrevAvailableBalance] ,[PostAvailableBalance] ,[PrevCountableBalance] ,[PostCountableBalance] 
							,[CreatedDate] ,[CreatedByUserId])
							SELECT @MOVEMENT_ID ,@InstallationId ,@AccountId ,@SubAccountId ,'FREE'
									,'CHARGE' ,2 ,3 ,@MOVEMENT_DESCRIPTION ,@CURRENCY_CODE
									,@Balance ,AB.[AvailableBalance] ,AB.[AvailableBalance] + @Balance ,AB.[AvailableBalance] ,AB.[AvailableBalance] + @Balance 
									,GETUTCDATE() ,@User
									FROM [dbo].[AccountBalance] AB 
									WHERE [ID] = @SUBACCOUNT_ID

						-- EL TERCER MOVIMIENTO QUE REGISTRAMOS ES LA ENTRADA(CHARGE) A LA CARTERA ASSIGNED DE LA CUENTA
						INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
									([ID] ,[InstallationId] ,[AccountId] ,[Wallet] ,[Type]
									,[CurrentStep] ,[TotalSteps] ,[Description] ,[CurrencyCode] ,[Amount] 
									,[PrevAvailableBalance] ,[PostAvailableBalance] ,[PrevCountableBalance] ,[PostCountableBalance] ,[CreatedDate] 
									,[CreatedByUserId])
							SELECT @MOVEMENT_ID ,@InstallationId ,@AccountId ,'ASSIGNED' ,'CHARGE'
									,3 ,3 ,@MOVEMENT_DESCRIPTION ,@CURRENCY_CODE ,@Balance 
									,AB.[AssignedBalance], AB.[AssignedBalance] + @Balance ,AB.[AssignedBalance] ,AB.[AssignedBalance] + @Balance ,GETUTCDATE() 
									,@User
								FROM [dbo].[AccountBalance] AB 
								WHERE [ID] = @ACCOUNT_ID

						-- DESBLOQUEMOS EL REGISTRO DE SALDOS DE LA CUENTA USANDO EL IDENTIFICADOR UNICO Y ACTUALIZAMOS EL SALDO DISPONIBLE
						UPDATE [dbo].[AccountBalance] 
							SET [IsLocked] = 0, 
								[AvailableBalance] = [AvailableBalance] - @Balance,
								[AssignedBalance] = [AssignedBalance] + @Balance,
								[LastModifiedByUserId] = @User,
								[LastModifiedDate] = GETUTCDATE(),
								[TimeStamp] = GETUTCDATE()
							WHERE [ID] = @ACCOUNT_ID
								AND [AvailableBalance] >= @Balance

						IF @@ROWCOUNT = 0
							RAISERROR('DATABASE_ERROR_INSUFFICIENT_ACCOUNT_BALANCE', 16, 16)
			
						-- DESBLOQUEMOS EL REGISTRO DE SALDOS DE LA SUBCUENTA USANDO EL IDENTIFICADOR UNICO Y ACTUALIZAMOS EL SALDO DISPONIBLE
						UPDATE [dbo].[AccountBalance] 
							SET [IsLocked] = 0, 
								[TotalBalance] = [TotalBalance] + @Balance,
								[AvailableBalance] = [AvailableBalance] + @Balance,
								[LastModifiedByUserId] = @User,
								[LastModifiedDate] = GETUTCDATE(),
								[TimeStamp] = GETUTCDATE()
							WHERE [ID] = @SUBACCOUNT_ID

					END 
					ELSE 
						-- LANZAMOS MESAJE AVISANDO QUE NO HAY SUFICIENTE SALDO EN LA SUBCUENTA
						RAISERROR('DATABASE_ERROR_INSUFFICIENT_SUBACCOUNT_BALANCE', 16, 16)
				END
				ELSE 
					-- LANZAMOS MESAJE AVISANDO QUE NO SE ENCONTRO LA SUBCUENTA
					RAISERROR('DATABASE_ERROR_NOT_FOUND_SUBACCOUNT', 16, 16)
			END
			ELSE
			BEGIN
				-- LANZAMOS MESAJE AVISANDO QUE NO SE ENCONTRO LA CUENTA
				RAISERROR('DATABASE_ERROR_NOT_FOUND_ACCOUNT', 16, 16)
			END
		END
		-- EN EL CASO DE VCC DEBEMOS VALIDAR QUE EL SALDO A INSERTAR SEA MENOR O IGUAL AL DISPONIBLE DE SU PADRE 
		-- INMEDIATO(CUENTA O SUBCUENTA)
		ELSE IF @TARGET = 'VCC'
		BEGIN 
			-- BUSCAMOS EL IDENTIFICADOR UNICO DE LA CUENTA EN LA TABLA ACCOUNTBALANCE
			SELECT @ACCOUNT_ID = AB.[ID] 
				FROM [dbo].[AccountBalance] AB (NOLOCK)
				WHERE AB.[PlatformHash] = @PlatformHash
					AND AB.[InstallationId] = @InstallationId 
					AND AB.[AccountId] = @AccountId
					AND AB.[SubAccountId] IS NULL

			-- VALIDAMOS SI EXISTE EL IDENTIFICADOR UNICO
			IF @ACCOUNT_ID IS NOT NULL
			BEGIN
				-- VALIDAMOS SI EXISTE EL VCC
				IF NOT EXISTS (
					SELECT 1 
						FROM [dbo].[AccountVCC] AV (NOLOCK)
						WHERE AV.[InstallationId] = @InstallationId 
							AND AV.[AccountId] = @AccountId
							AND AV.[VCC] = @VCC
				)
					-- LANZAMOS MESAJE AVISANDO QUE NO SE ENCONTRO EL VCC
					RAISERROR('DATABASE_ERROR_NOT_FOUND_VCC', 16, 16)

				-- VALIDAMOS QUE EL VCC TIENE EL WALLET FREE EN ACCOUNTVCCBALANCE
				IF NOT EXISTS (
					SELECT 1 
						FROM [dbo].[AccountVCCBalance] AVB (NOLOCK) 
						WHERE AVB.[InstallationId] = @InstallationId 
							AND AVB.[AccountId] = @AccountId
							AND AVB.[VCC] = @VCC
							AND AVB.[SystemService] = 'FREE'
				)
					-- LANZAMOS MESAJE AVISANDO QUE NO SE ENCONTRO EL VCC
					RAISERROR('DATABASE_ERROR_NOT_FOUND_VCC_WALLET', 16, 16)
				
				-- SI SUBCUENTA ES NULO EL PADRE ES CUENTA
				IF @SubAccountId IS NULL
				BEGIN
					-- VERIFICAMOS QUE LA CUENTA TIENE SALDO DISPONIBLE MAYOR O IGUAL AL BALANCE QUE ESTAMOS SETTEANDO
					IF 1 = (SELECT 1 
							FROM [dbo].[AccountBalance] AB 
							WHERE AB.[ID] = @ACCOUNT_ID
								AND AB.[AvailableBalance] >= @Balance) 
					BEGIN
						-- BLOQUEMOS TEMPORALMENTE EL REGISTRO DE SALDOS DE LA CUENTA USANDO EL IDENTIFICADOR UNICO Y ASEGURANDONOS QUE 
						-- EL SALDO DISPONIBLE AUN ES MAYOR QUE EL SALDO A SETTEAR
						UPDATE [dbo].[AccountBalance] 
							SET [IsLocked] = 1,
								[LastModifiedByUserId] = @User,
								[LastModifiedDate] = GETUTCDATE(),
								[TimeStamp] = GETUTCDATE() 
							WHERE [ID] = @ACCOUNT_ID
								AND [AvailableBalance] >= @Balance

						IF @@ROWCOUNT = 0
							RAISERROR('DATABASE_ERROR_INSUFFICIENT_ACCOUNT_BALANCE', 16, 16)

						-- UNA VEZ BLOQUEADO EL REGISTRO DE LA TABLA ACCOUNTBALANCE, REGISTRAMOS LAS SALIDAS Y ENTRADAS EN LA TABLA DE 
						-- MOVIMIENTOS ACCOUNSUBACCOUNTVCCBALANCEMOVEMENT (MOVIMIENTO DE ACCOUNT FREE -> VCC REE)
					
						-- DEFINIMOS LA DESCRIPCION DEL MOVIMIENTO PARA ESTE BLOQUE
						SET @MOVEMENT_DESCRIPTION = CONCAT('VCC ', @VCC, ': CHARGE ', @CURRENCY_CODE, ': ', ISNULL(@CURRENCY_SYMBOL, ''), ' ', @Balance, ' TO FREE WALLET(VCC ', @VCC, ') FROM FREE WALLET(ACCOUNT ', @AccountId + ')') 

						-- EL PRIMER MOVIMIENTO QUE REGISTRAMOS ES LA SALIDA DE LA CUENTA
						INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
									([ID] ,[InstallationId] ,[AccountId] ,[Wallet] ,[Type]
									,[CurrentStep] ,[TotalSteps] ,[Description] ,[Amount] ,[CurrencyCode]
									,[PrevAvailableBalance] ,[PostAvailableBalance] ,[PrevCountableBalance] ,[PostCountableBalance] ,[CreatedDate] 
									,[CreatedByUserId])
							SELECT @MOVEMENT_ID ,@InstallationId ,@AccountId ,'FREE' ,'DISCOUNT'
									,1 ,3 ,@MOVEMENT_DESCRIPTION ,@Balance ,@CURRENCY_CODE
									,AB.[AvailableBalance] ,AB.[AvailableBalance] - @Balance ,AB.[AvailableBalance] ,AB.[AvailableBalance] - @Balance ,GETUTCDATE() 
									,@User
								FROM [dbo].[AccountBalance] AB (NOLOCK)
								WHERE [ID] = @ACCOUNT_ID
								
						-- EL SEGUNDO MOVIMIENTO QUE REGISTRAMOS ES LA ENTRADA AL VCC
						INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
							([ID] ,[InstallationId] ,[AccountId] ,[VCC] ,[Wallet]
							,[Type] ,[CurrentStep] ,[TotalSteps] ,[Description] ,[CurrencyCode]
							,[Amount],[PrevAvailableBalance] ,[PostAvailableBalance] ,[PrevCountableBalance] ,[PostCountableBalance] 
							,[CreatedDate] ,[CreatedByUserId])
							SELECT @MOVEMENT_ID ,@InstallationId ,@AccountId ,@VCC ,'FREE'
									,'CHARGE' ,2 ,3 ,@MOVEMENT_DESCRIPTION ,@CURRENCY_CODE
									,@Balance ,AVB.[AvailableBalance] ,AVB.[AvailableBalance] +  @Balance ,AVB.[CountableBalance] ,AVB.[CountableBalance] + @Balance 
									,GETUTCDATE() ,@User
								FROM [dbo].[AccountVCCBalance] AVB (NOLOCK) 
								WHERE AVB.[InstallationId] = @InstallationId 
									AND AVB.[AccountId] = @AccountId
									AND AVB.[VCC] = @VCC
									AND AVB.[SystemService] = 'FREE'

						-- EL TERCER MOVIMIENTO QUE REGISTRAMOS ES LA ENTRADA(CHARGE) A LA CARTERA ASSIGNED DE LA CUENTA
						INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
									([ID] ,[InstallationId] ,[AccountId] ,[Wallet] ,[Type]
									,[CurrentStep] ,[TotalSteps] ,[Description] ,[CurrencyCode] ,[Amount] 
									,[PrevAvailableBalance] ,[PostAvailableBalance] ,[PrevCountableBalance] ,[PostCountableBalance] ,[CreatedDate] 
									,[CreatedByUserId])
							SELECT @MOVEMENT_ID ,@InstallationId ,@AccountId ,'ASSIGNED' ,'CHARGE'
									,3 ,3 ,@MOVEMENT_DESCRIPTION ,@CURRENCY_CODE ,@Balance 
									,AB.[AssignedBalance] ,AB.[AssignedBalance] + @Balance ,AB.[AssignedBalance] ,AB.[AssignedBalance] + @Balance ,GETUTCDATE() 
									,@User
								FROM [dbo].[AccountBalance] AB (NOLOCK)
								WHERE [ID] = @ACCOUNT_ID
				
						-- DESBLOQUEMOS EL REGISTRO DE SALDOS DE LA CUENTA USANDO EL IDENTIFICADOR UNICO Y ACTUALIZAMOS EL SALDO DISPONIBLE
						UPDATE [dbo].[AccountBalance] 
							SET [IsLocked] = 0, 
								[AvailableBalance] = [AvailableBalance] - @Balance,
								[AssignedBalance] = [AssignedBalance] + @Balance,
								[LastModifiedByUserId] = @User,
								[LastModifiedDate] = GETUTCDATE(),
								[TimeStamp] = GETUTCDATE()
							WHERE [ID] = @ACCOUNT_ID
								AND [AvailableBalance] >= @Balance

						IF @@ROWCOUNT = 0
							RAISERROR('DATABASE_ERROR_INSUFFICIENT_ACCOUNT_BALANCE', 16, 16)

						-- ACTUALIZAMOS EL SALDO FREE DEL VCC EN LA TABLA ACCOUNTVCCBALANCE
						UPDATE [dbo].[AccountVCCBalance] 
							SET [AvailableBalance] = [AvailableBalance] + @Balance,
								[CountableBalance] = [CountableBalance] + @Balance,
								[LastModifiedByUserId] = @User,
								[LastModifiedDate] = GETUTCDATE(),
								[TimeStamp] = GETUTCDATE() 
							WHERE [InstallationId] = @InstallationId 
								AND [AccountId] = @AccountId
								AND [VCC] = @VCC
								AND [SystemService] = 'FREE'

						IF @@ROWCOUNT = 0
							RAISERROR('DATABASE_ERROR_VCC_BALANCA_UPDATE_FAILED', 16, 16)

					END
					ELSE 
						-- LANZAMOS MESAJE AVISANDO QUE NO SE ENCONTRO LA CUENTA
						RAISERROR('DATABASE_ERROR_INSUFFICIENT_ACCOUNT_BALANCE', 16, 16)

				END 
				-- SI SUBCUENTA ES DIFERENTE DE NULO EL PADRE ES SUBCUENTA
				ELSE
				BEGIN
					-- BUSCAMOS EL IDENTIFICADOR UNICO DE LA SUBCUENTA EN LA TABLA ACCOUNTBALANCE
					SELECT @SUBACCOUNT_ID = AB.[ID] 
						FROM [dbo].[AccountBalance] AB (NOLOCK)
						WHERE AB.[PlatformHash] = @PlatformHash
							AND AB.[InstallationId] = @InstallationId 
							AND AB.[AccountId] = @AccountId
							AND AB.[SubAccountId] = @SubAccountId

					-- VALIDAMOS SI EXISTE EL IDENTIFICADOR UNICO
					IF @SUBACCOUNT_ID IS NOT NULL
					BEGIN
						-- VERIFICAMOS QUE LA SUBCUENTA TIENE SALDO DISPONIBLE MAYOR O IGUAL AL BALANCE QUE ESTAMOS SETTEANDO
						IF 1 = (SELECT 1 
								FROM [dbo].[AccountBalance] AB 
								WHERE AB.[ID] = @SUBACCOUNT_ID
									AND AB.[AvailableBalance] >= @Balance) 
						BEGIN
							-- BLOQUEMOS TEMPORALMENTE EL REGISTRO DE SALDOS DE LA SUBCUENTA USANDO EL IDENTIFICADOR UNICO Y ASEGURANDONOS QUE 
							-- EL SALDO DISPONIBLE AUN ES MAYOR QUE EL SALDO A SETTEAR
							UPDATE [dbo].[AccountBalance] 
								SET [IsLocked] = 1,
									[LastModifiedByUserId] = @User,
									[LastModifiedDate] = GETUTCDATE(),
									[TimeStamp] = GETUTCDATE() 
								WHERE [ID] = @SUBACCOUNT_ID
									AND [AvailableBalance] >= @Balance
	
							IF @@ROWCOUNT = 0
								RAISERROR('DATABASE_ERROR_INSUFFICIENT_ACCOUNT_BALANCE', 16, 16)

							-- UNA VEZ BLOQUEADO EL REGISTRO DE LA TABLA ACCOUNTBALANCE, REGISTRAMOS LAS SALIDAS Y ENTRADAS EN 
							-- LA TABLA DE MOVIMIENTOS ACCOUNSUBACCOUNTVCCBALANCEMOVEMENT (MOVIMIENTO DE SUBACCOUNT FREE -> VCC FREE)

							-- DEFINIMOS LA DESCRIPCION DEL MOVIMIENTO PARA ESTE BLOQUE
							SET @MOVEMENT_DESCRIPTION = CONCAT('VCC ', @VCC, ': CHARGE ', @CURRENCY_CODE, ': ', ISNULL(@CURRENCY_SYMBOL, ''), ' ', @Balance, ' TO FREE WALLET(VCC ', @VCC, ') FROM FREE WALLET(SUBACCOUNT ', @SubAccountId, ')') 

							-- EL PRIMER MOVIMIENTO QUE REGISTRAMOS ES LA SALIDA(DISCOUNT) DE LA CARTERA FREE DE LA SUBCUENTA
							INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
										([ID] ,[InstallationId] ,[AccountId] ,[SubAccountId] ,[Wallet]
										,[Type] ,[CurrentStep] ,[TotalSteps] ,[Description] ,[CurrencyCode]
										,[Amount] ,[PrevAvailableBalance] ,[PostAvailableBalance] ,[PrevCountableBalance] ,[PostCountableBalance] 
										,[CreatedDate] ,[CreatedByUserId])
								SELECT @MOVEMENT_ID, @InstallationId, @AccountId, @SubAccountId, 'FREE'
										,'DISCOUNT' ,1 ,3 ,@MOVEMENT_DESCRIPTION ,@CURRENCY_CODE
										,@Balance ,AB.[AvailableBalance] ,AB.[AvailableBalance] - @Balance ,AB.[AvailableBalance] ,AB.[AvailableBalance] - @Balance 
										,GETUTCDATE() ,@User
									FROM [dbo].[AccountBalance] AB 
									WHERE [ID] = @SUBACCOUNT_ID
									
							-- EL SEGUNDO MOVIMIENTO QUE REGISTRAMOS ES LA ENTRADA(CHARGE) A LA CARTERA FREE DEL VCC
							INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
								([ID] ,[InstallationId] ,[AccountId] ,[SubAccountId] ,[VCC]
								,[Wallet] ,[Type] ,[CurrentStep] ,[TotalSteps] ,[Description]
								,[CurrencyCode] ,[Amount] ,[PostAvailableBalance] ,[PostCountableBalance] ,[CreatedDate])
							VALUES
								(@MOVEMENT_ID ,@InstallationId ,@AccountId ,@SubAccountId ,@VCC
								,'FREE' ,'CHARGE' ,2 ,3 ,@MOVEMENT_DESCRIPTION
								,@CURRENCY_CODE ,@Balance ,@Balance ,@Balance ,GETUTCDATE())

							-- EL TERCER MOVIMIENTO QUE REGISTRAMOS ES LA ENTRADA(CHARGE) A LA CARTERA ASSIGNED DE LA SUBCUENTA
							INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
										([ID] ,[InstallationId] ,[AccountId] ,[SubAccountId] ,[Wallet]
										,[Type] ,[CurrentStep] ,[TotalSteps] ,[Description] ,[CurrencyCode]
										,[Amount] ,[PrevAvailableBalance] ,[PostAvailableBalance] ,[PrevCountableBalance] ,[PostCountableBalance] 
										,[CreatedDate] ,[CreatedByUserId])
								SELECT @MOVEMENT_ID ,@InstallationId ,@AccountId ,[SubAccountId] ,'ASSIGNED'
										,'CHARGE' ,3 ,3 ,@MOVEMENT_DESCRIPTION ,@CURRENCY_CODE
										,@Balance ,AB.[AssignedBalance] ,AB.[AssignedBalance] + @Balance,AB.[AssignedBalance] ,AB.[AssignedBalance] + @Balance 
										,GETUTCDATE() ,@User
									FROM [dbo].[AccountBalance] AB 
									WHERE [ID] = @SUBACCOUNT_ID
			
							-- DESBLOQUEMOS EL REGISTRO DE SALDOS DE LA SUBCUENTA USANDO EL IDENTIFICADOR UNICO Y ACTUALIZAMOS EL SALDO DISPONIBLE
							UPDATE [dbo].[AccountBalance] 
								SET [IsLocked] = 0, 
									[AvailableBalance] = [AvailableBalance] - @Balance,
									[AssignedBalance] = [AssignedBalance] + @Balance
								WHERE [ID] = @SUBACCOUNT_ID
									AND [AvailableBalance] >= @Balance

							IF @@ROWCOUNT = 0
								RAISERROR('DATABASE_ERROR_INSUFFICIENT_SUBACCOUNT_BALANCE', 16, 16)

							-- ACTUALIZAMOS EL SALDO FREE DEL VCC EN LA TABLA ACCOUNTVCCBALANCE
							UPDATE [dbo].[AccountVCCBalance] 
								SET [AvailableBalance] = [AvailableBalance] + @Balance,
									[CountableBalance] = [CountableBalance] + @Balance,
									[LastModifiedByUserId] = @User,
									[LastModifiedDate] = GETUTCDATE(),
									[TimeStamp] = GETUTCDATE() 
								WHERE [InstallationId] = @InstallationId 
									AND [AccountId] = @AccountId
									AND [VCC] = @VCC
									AND [SystemService] = 'FREE'

							IF @@ROWCOUNT = 0
								RAISERROR('DATABASE_ERROR_VCC_BALANCA_UPDATE_FAILED', 16, 16)

						END 
						ELSE 
							-- LANZAMOS MESAJE AVISANDO QUE NO HAY SUFICIENTE SALDO EN LA SUBCUENTA
							RAISERROR('DATABASE_ERROR_INSUFFICIENT_SUBACCOUNT_BALANCE', 16, 16)
					END
					ELSE 
						-- LANZAMOS MESAJE AVISANDO QUE NO SE ENCONTRO LA SUBCUENTA
						RAISERROR('DATABASE_ERROR_NOT_FOUND_SUBACCOUNT', 16, 16)
				END
			END
			ELSE
			BEGIN
				-- LANZAMOS MESAJE AVISANDO QUE NO SE ENCONTRO LA CUENTA
				RAISERROR('DATABASE_ERROR_NOT_FOUND_ACCOUNT', 16, 16)
			END
		END 
		ELSE 
			-- LANZAMOS MENSAJE QUE SE INGRESO UN TARGET NO VALIDO
			RAISERROR('DATABASE_ERROR_INVALID_TARGET', 16, 16)

		-- RETORNAMOS RESULTADO 1
		SELECT 1 'result'

		-- COMPLETAMOS LA TRANSACCION
		COMMIT TRANSACTION

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH

END
GO
PRINT N'Modificando [dbo].[AccountSubAccountVCCBatchSummaryCreate]...';


GO

ALTER PROCEDURE [dbo].[AccountSubAccountVCCBatchSummaryCreate] (
	@PlatformHash VARCHAR(100) = NULL,
	@InstallationId VARCHAR(50) = NULL,
	@AccountId VARCHAR(50) = NULL,
	@SubAccountId VARCHAR(50) = NULL,
	@VCC VARCHAR(50) = NULL,
	@TotalInteraction INT = 0,							-- TOTAL DE INTERACCIONES DE BATCH(SINCRONAS + ASINCRONAS)
	@TotalAsync INT = 0,								-- TOTAL DE INTERACCIONES ASINCRONAS(OTROS CANALES)
	@TotalAsyncDetail INT = 0,							-- TOTAL DE DETALLES DE INTERACCIONES ASINCRONAS
	@EstimatedNewTextPhrases DECIMAL(18, 5) = 0,		-- TOTAL DE FRASES TEXT ESTIMADAS PARA EL BATCH ACTUAL(SU FINALIDAD ES LA RETENCION)
	@AlreadyTextPhrases INT = 0,						-- TOTAL DE FRASES TEXT YA GENERADAS EN OTRO PROCESS
	@RawChar INT = 0,									-- TOTAL BRUTO DE CARACTERES DEL PRESENTE BATCH(TODOS)
	@InProcessAsyncDetail INT = 0,						-- CANTIDAD DE ASYNCDETAIL YA PRESENTES EN OTROS BATCH
	@InProcessRawChar INT = 0,							-- TOTAL BRUTO DE CARACTERES YA PRESENTES EN OTROS BATCH EN CURSO
	@AlreadyTranscribedAsyncDetail INT = 0,				-- CANTIDAD DE ASYNCDETAIL YA TRANSCRITOS EN OTROS BATCH EN CURSO
	@AlreadyTranscribedRawChar INT = 0,					-- TOTAL BRUTO DE CARACTERES PRESENTES EN OTROS BATCH YA TRANSCRITOS
	@AlreadyTranscribedValidChar INT = 0,				-- TOTAL CARACTERES VALIDOS PRESENTES EN OTROS BATCH YA TRANSCRITOS
	@TotalSync INT = 0,									-- TOTAL DE INTERACCIONES SINCRONAS(CALL)
	@TotalAudio int = 0,								-- CANTIDAD DE AUDIOS DEL PRESENTE BATCH
	@EstimatedNewCallPhrases DECIMAL(18, 5) = 0,		-- TOTAL DE FRASES CALL ESTIMADAS PARA EL BATCH ACTUAL(SU FINALIDAD ES LA RETENCION)
	@AlreadyCallPhrases INT = 0,						-- TOTAL DE FRASES CALL YA GENERADAS EN OTRO PROCESS
	@DurationTime int = 0,								-- TIEMPO BRUTO DE LOS AUDIOS DEL PRESENTE BATCH
	@RoundedDurationTime int = 0,						-- TIEMPO REDONDEADO BRUTO DE LOS AUDIOS DEL PRESENTE BATCH
	@InProcessAudio int = 0,							-- CANTIDAD DE AUDIOS YA PRESENTES EN OTROS BATCH
	@InProcessDurationTime int = 0,						-- TIEMPO BRUTO DE LOS AUDIOS YA PRESENTES EN OTROS BATCH
	@AlreadyTranscribedAudio int = 0,					-- CANTIDAD DE AUDIOS YA TRANSCRITOS EN OTROS BATCH
	@AlreadyDurationTime int = 0,						-- TIEMPO BRUTO YA TRANSCRITO
	@AlreadyRoundedDurationTime int = 0,				-- TIEMPO REDODNEADO BRUTO YA TRANSCRITO
	@AlreadyTranscribedDurationTime int = 0,			-- TIEMPO TRANCRITO
	@AlreadyTranscribedRoundedDurationTime int = 0,		-- TIEMPO REDONDEADO YA TRANSCRITO
	@TotalQueries int = 0,
	@TotalModels DECIMAL(18, 5) = 0,
	@ModelRequests DECIMAL(18, 5) = 0,					-- CANTIDAD ESTIMADA DE REQUESTS POR MODELO(INCLUYE CALL Y TEXT + MARGEN DE SEGURIDAD)
	@BatchTypeName VARCHAR(50) = NULL,
	@BatchTypeId VARCHAR(50) = NULL,
	@ProcessId VARCHAR(100) = NULL,
	@TemplateId VARCHAR(100) = NULL,
	@UserApi VARCHAR(50) = NULL,
	@StartDate DateTime = NULL
) AS
BEGIN
	-- ESTE SP TIENE LA FINALIDAD DE ACTUALIZAR LOS SALDOS DE LAS CARTERAS DEL VCC
	SET XACT_ABORT, NOCOUNT ON

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY

		-- INICIAMOS LA TRANSACCION
		BEGIN TRANSACTION

		-- VERIFICAMOS QUE TENEMOS LOS PARAMETROS NECESARIOS
		IF @PlatformHash IS NULL 
			OR @InstallationId IS NULL 
			OR @AccountId IS NULL 
			OR @VCC IS NULL
			OR @UserApi IS NULL
			-- LANZAMOS MENSAJE AVISANDO QUE FALTA ACCOUNTID
			RAISERROR('DATABASE_ERROR_MISSING_PARAMETERS', 16, 16)

		-- TOCA VALIDAR QUE EXISTE EL PLATFORMHASH Y ESTA ACTIVO
		-- TOCA VALIDAR QUE EXISTE EL ACCOUNT
		-- TOCA VALIDAR QUE EXISTE EL SUBACCOUNT
		-- TOCA VALIDAR QUE EXISTE EL VCC
		-- TOCA VALIDAD QUE EXISTE EL USERAPI

		-- VERIFICAMOS QUE EL BATCH TYPE SEA UN TIPO CONOCIDO DEL SISTEMA
		IF @BatchTypeName <> 'EVALUATION_PLAN' AND @BatchTypeName <> 'TRANSCRIPTION' AND @BatchTypeName <> 'STUDY'
			-- LANZAMOS MENSAJE AVISANDO QUE HAN SOLICITADO UNA OPERACION DESCONOCIDA
			RAISERROR('DATABASE_ERROR_UNKNOWN_BATCH_TYPE', 16, 16)

		-- VERIFICAMOS QUE EL BATCH RECIBIDO NO EXISTA YA EN LA BASE DE DATOS
		IF EXISTS(
			SELECT ASAVST.[BatchTypeId] 
				FROM [dbo].[AccountSubAccountVCCSummaryTransaction] ASAVST (NOLOCK)
				WHERE ASAVST.[InstallationId] = @InstallationId
					AND ASAVST.[VCC] =  @VCC
					AND ASAVST.[BatchTypeName] = @BatchTypeName
					AND ASAVST.[BatchTypeId] =  @BatchTypeId
		)
			-- LANZAMOS MENSAJE AVISANDO QUE FALTA ACCOUNTID
			RAISERROR('DATABASE_ERROR_DUPLICATE_BATCHTYPEID', 16, 16)
		
		-- DECLARAMOS VARIABLES TIPO BIT QUE NOS INDIQUEN LOS SERVICIOS DEL SISTEMA QUE REGISTRAREMOS
		DECLARE @TRANSCRIPTION BIT = 0
		DECLARE @SEMANTIC_ANALYSIS BIT = 0
		DECLARE @COGNITIVE_ANALYSIS BIT = 0
		DECLARE @VALID_DURATION_TIME INT = 0
		DECLARE @ROUNDED_VALID_DURATION_TIME INT = 0
		DECLARE @VALID_CHAR INT = 0

		-- SEGUN LOS DATOS RECIBIDOS DETERMINAMOS QUE SERVICIO DEL SISTEMA ESTA INCLUIDO EN EL BATCH
		IF @DurationTime > 0 OR @TotalAsyncDetail > 0 SET @TRANSCRIPTION = 1
		IF @TotalQueries > 0 SET @SEMANTIC_ANALYSIS = 1
		IF @TotalModels > 0 SET @COGNITIVE_ANALYSIS = 1

		-- VALIDAMOS QUE AL MENOS UNO DE LOS SERVICIOS DEL SISTEMA SEA CONSUMIDO PRO EL BATCH
		IF @TRANSCRIPTION = 0 AND @SEMANTIC_ANALYSIS = 0 AND @COGNITIVE_ANALYSIS = 0
			-- LANZAMOS MENSAJE AVISANDO QUE HAN SOLICITADO UNA OPERACION DESCONOCIDA
			RAISERROR('DATABASE_ERROR_NO_SYSTEM_SERVICE', 16, 16)

		-- OBTENEMOS EL CURRENCYCODE Y CURRENCYSYMBOL DE LA CUENTA
		DECLARE @CURRENCY_CODE VARCHAR(5) = NULL
		DECLARE @CURRENCY_SYMBOL NVARCHAR(10) = NULL

		SELECT @CURRENCY_CODE = A.[CurrencyCode],
				@CURRENCY_SYMBOL = C.[Symbol]
			FROM [dbo].[Account] A (NOLOCK)
			INNER JOIN [dbo].[Currency] C (NOLOCK)
				ON C.[Code] = A.[CurrencyCode]
			WHERE A.[InstallationId] = @InstallationId 
				AND A.[AccountId] = @AccountId
				AND A.[SubAccountId] IS NULL

		-- DECLARAMOS VARIABLES TIPO DECIMAL PARA ALMACENAR LAS TARIFAS DE SERVICIOS SETTEADAS PARA EL VCC
		DECLARE @TRANSCRIPTION_RATE_PER_SECOND DECIMAL(18,5) = 0
		DECLARE @SEMANTIC_ANALYSIS_RATE_PER_SECOND DECIMAL(18,5) = 0
		DECLARE @COGNITIVE_ANALYSIS_RATE_PER_SECOND DECIMAL(18,5) = 0

		-- BUSCAMOS LA TARIFAS DE LOS SERVICIOS DEL SISTEMA PARA EL VCC INGRESADO
		SELECT @TRANSCRIPTION_RATE_PER_SECOND = AV.[TranscriptionRatePerSecond],
				@SEMANTIC_ANALYSIS_RATE_PER_SECOND = AV.[SemanticAnalysisRatePerTransaction],
				@COGNITIVE_ANALYSIS_RATE_PER_SECOND = AV.[CognitiveAnalysisRatePerTransaction]
			FROM [dbo].[AccountVCC] AV (NOLOCK)
			WHERE AV.[InstallationId] = @InstallationId
				AND AV.[VCC] = @VCC 

		-- DECLARAMOS UNA VARIABLE PARA EL IDENTIFICADOR UNICO DEL BATCH
		DECLARE @ID VARCHAR(100) = NEWID()

		-- DECLARAMOS UNA VARIABLE INT PARA CONTABILIZAR LOS PASOS PARA EL REGISTRO DE MOVIMIENTOS
		DECLARE @BATCH_TOTAL_STEPS INT = 0

		-- DECLARAMOS UNA VARIABLE PARA INDICAR EL PASO ACTUAL EN QUE NOS ENCONTRAMOS
		DECLARE @BATCH_CURRENT_STEP INT = 0 

		-- DECLARAMOS UNA VARIABLE VARCHAR PARA CONCATENAR LOS SERVICIOS DEL SISTEMA INCLUIDOS EN EL BATCH
		DECLARE @BATCH_SYSTEM_SERVICES VARCHAR(100) = ''
		
		-- CONCATENAMOS LOS SERVICIOS DEL SISTEMA Y ACUMULMOS EL TOTAL DE PASOS PARA EL REGISTRO DE MOVIMIENTOS (SERVICIOS ASOCIADOS * 3 <- RETAIN, DEBT, RELEASE)
		IF @TRANSCRIPTION = 1
		BEGIN
			SET @BATCH_TOTAL_STEPS = @BATCH_TOTAL_STEPS + (1 * 3)
			SET @BATCH_SYSTEM_SERVICES = CONCAT(@BATCH_SYSTEM_SERVICES, 'TRANSCRIPTION,')
		END

		IF @SEMANTIC_ANALYSIS = 1 
		BEGIN 
			SET @BATCH_TOTAL_STEPS = @BATCH_TOTAL_STEPS + (1 * 3)
			SET @BATCH_SYSTEM_SERVICES = CONCAT(@BATCH_SYSTEM_SERVICES, 'SEMANTIC_ANALYSIS,')
		END
		
		IF @COGNITIVE_ANALYSIS = 1 
		BEGIN 
			SET @BATCH_TOTAL_STEPS = @BATCH_TOTAL_STEPS + (1 * 3)
			SET @BATCH_SYSTEM_SERVICES = CONCAT(@BATCH_SYSTEM_SERVICES, 'COGNITIVE_ANALYSIS,')
		END

		-- DECLARAMOS LA VARIABLE EN LA QUE ESCRIBIREMOS LA DESCRIPCION DEL MOVIMIENTO
		DECLARE @MOVEMENT_DESCRIPTION NVARCHAR(250) = NULL

		-- DECLARAMOS UNA VARIABLE VARCHAR PARA ALMACENAR EL SERVICIO DEL SISTEMA RECORRIDO
		DECLARE @CURRENT_SYSTEM_SERVICE VARCHAR(50) = ''

		-- DECLARAMOS UNA VARIABLE DECIMAL PARA ALMACENAR EL SALDO RETENIDO RECORRIDO
		DECLARE @CURRENT_RETAINED_AMOUNT DECIMAL(18,5) = 0

		-- DECLARAMOS UNA VARIABLE DECIMAL PARA ALMACENAR EL TOTAL DE REQUEST ESTIMADOS CON % DE SEGURIDAD INCLUIDO
		DECLARE @CURRENT_TOTAL_ESTIMATED_MODEL_REQUESTS DECIMAL(18,5) = 0
		
		-- DECLARAMOS UNA VARIABLE PARA ALMACENAR EL SPLIT DE SERVICIOS
		DECLARE MY_CURSOR CURSOR 
		LOCAL STATIC READ_ONLY FORWARD_ONLY
			FOR 
			SELECT Value FROM [dbo].[split](@BATCH_SYSTEM_SERVICES, ',')

		-- ABRIMOS Y RECORREMOS EL ARRAY DE PRODUCTOS
		OPEN MY_CURSOR
		FETCH NEXT FROM MY_CURSOR INTO @CURRENT_SYSTEM_SERVICE
		WHILE @@FETCH_STATUS = 0
		BEGIN 
			-- VALIDAMOS QUE NO SEA UNA VARIABLE VACIA
			IF @CURRENT_SYSTEM_SERVICE <> ''
			BEGIN
				-- SETTEAMOS EL MONTO RETENIDO ACTUAL
				IF @CURRENT_SYSTEM_SERVICE = 'TRANSCRIPTION' 
				BEGIN
					-- EN CASO DURATIONTIME SEA NULL
					IF @DurationTime IS NULL SET @DurationTime = 0
					IF @InProcessDurationTime IS NULL SET @InProcessDurationTime = 0
					IF @AlreadyDurationTime IS NULL SET @AlreadyDurationTime = 0

					SET @CURRENT_RETAINED_AMOUNT = @TRANSCRIPTION_RATE_PER_SECOND * (@DurationTime - @InProcessDurationTime - @AlreadyDurationTime)
					
					-- EN CASO EL RETENIDO SEA MENOR A 0, LO IGUALAMOS A 0
					IF @CURRENT_RETAINED_AMOUNT < 0 
						SET @CURRENT_RETAINED_AMOUNT = 0
				END

				IF @CURRENT_SYSTEM_SERVICE IN ('SEMANTIC_ANALYSIS', 'COGNITIVE_ANALYSIS')
				BEGIN
					-- DEPENDIENDO DEL TIPO DE BATCH TOMAMOS EL TIEMPO DE DIFERENTE FORMA
					IF @BatchTypeName = 'STUDY'
					BEGIN
						-- RECUPERAMOS EL DURATIONTIME USANDO EL PROCESSID
						SELECT @TotalAsyncDetail = (ASAVST.[ValidTotalAsyncDetail] + ASAVST.[AlreadyTranscribedAsyncDetail]),
								@RawChar = (ASAVST.[RawChar] + ASAVST.[AlreadyTranscribedRawChar]),
								@VALID_CHAR = (ASAVST.[ValidChar] + ASAVST.[AlreadyTranscribedValidChar]),
								@TotalAudio = (ASAVST.[ValidTotalAudio] + ASAVST.[AlreadyTranscribedAudio]),
								@DurationTime = (ASAVST.[DurationTime] + ASAVST.[AlreadyDurationTime]),
								@VALID_DURATION_TIME = (ASAVST.[ValidDurationTime] + ASAVST.[AlreadyTranscribedDurationTime]),
								@ROUNDED_VALID_DURATION_TIME = (ASAVST.[RoundedDurationTime] + ASAVST.[AlreadyTranscribedRoundedDurationTime])
							FROM [dbo].[AccountSubAccountVCCSummaryTransaction] ASAVST (NOLOCK) 
							WHERE ASAVST.[InstallationId] = @InstallationId
								AND ASAVST.[VCC] = @Vcc
								AND ASAVST.[BatchTypeName] = 'TRANSCRIPTION'
								AND ASAVST.[BatchTypeID] = @ProcessId

						-- SI ESTAMOS CREANDO ESTUDIO, NO NECESITAMOS EL DURATIONTIME PORQUE YA TENEMOS EL VALIDDURATIONTIME
					END
					ELSE IF @BatchTypeName = 'EVALUATION_PLAN'
					BEGIN 
						SET @DurationTime = @DurationTime
						SET @ROUNDED_VALID_DURATION_TIME = @RoundedDurationTime
						SET @AlreadyDurationTime = @AlreadyDurationTime
						SET @AlreadyRoundedDurationTime = @AlreadyRoundedDurationTime
					END

					IF @CURRENT_SYSTEM_SERVICE = 'SEMANTIC_ANALYSIS' SET @CURRENT_RETAINED_AMOUNT = @SEMANTIC_ANALYSIS_RATE_PER_SECOND * @TotalQueries

					IF @CURRENT_SYSTEM_SERVICE = 'COGNITIVE_ANALYSIS'
					BEGIN
						-- DECLARAMOS UN FLAG PARA INDICAR SI ES UN PROCESS PREVIO A LA IMPLEMENTACION DE CANALES
						DECLARE @IS_POST_CHANNEL BIT = 0

						IF EXISTS (
							SELECT TOP 1 1 
								FROM [dbo].[AccountSubAccountVCCSummaryTransaction] ASAVST (NOLOCK), [dbo].[GeneralSecureRate] GSR (NOLOCK)
								WHERE ASAVST.[BatchTypeName] = 'TRANSCRIPTION'
									AND ASAVST.[EndDate] > GSR.[CreatedDate]
									AND ASAVST.[BatchTypeId] = @ProcessId
						)
							SET @IS_POST_CHANNEL = 1

						-- *TASA DE SEGURO DE FACTURACION: SI SE CAMBIA ESTE VALOR, SE DEBERA CAMBIAR TAMBIEN EN EL SP [dbo].[AccountSubAccountVCCBatchSummaryCreate]
						DECLARE @COGNITIVE_ANALYSIS_BILLING_SECURE_RATE DECIMAL(5, 2) = 0 
						DECLARE @COGNITIVE_ANALYSIS_TEXT_BILLING_SECURE_RATE DECIMAL(5, 2) = 0 

						-- RECUPERAMOS LAS TASAS DE SEGURIDAD DE LA TABLA CORRESPONDIENTE(ESTE BLOQUE SE BORRARA, LA FUNCIONALIDAD ACTUALIZADA HACE LA DISCRIMINACION AUTOMATICAMENTE)
						SELECT TOP 1 @COGNITIVE_ANALYSIS_BILLING_SECURE_RATE  = GSR.[QualityCallCognitiveAnalysisSecureRate],
										@COGNITIVE_ANALYSIS_TEXT_BILLING_SECURE_RATE = GSR.[QualityTextCognitiveAnalysisSecureRate]
							FROM [dbo].[GeneralSecureRate] GSR (NOLOCK)

						-- SEGUIMOS SEGUN EL PROCESS ES PREVIO O NO A LA IMPLMENTACION DE CANALES(2020-07)
						-- IF @IS_POST_CHANNEL = 1
						IF @BatchTypeName = 'STUDY'
						BEGIN
							SET @CURRENT_RETAINED_AMOUNT = CEILING(@TotalModels * @COGNITIVE_ANALYSIS_RATE_PER_SECOND * @ModelRequests) 
							/*
							SET @CURRENT_RETAINED_AMOUNT = CEILING(
																	@TotalModels 
																	*
																	@COGNITIVE_ANALYSIS_RATE_PER_SECOND
																	*
																	(
																		((@EstimatedNewCallPhrases + @AlreadyCallPhrases) * (1 + @COGNITIVE_ANALYSIS_BILLING_SECURE_RATE))
																		+
																		((@EstimatedNewTextPhrases + @AlreadyTextPhrases) * (1 + @COGNITIVE_ANALYSIS_TEXT_BILLING_SECURE_RATE))
																	)
															)
							*/

							SET @CURRENT_TOTAL_ESTIMATED_MODEL_REQUESTS = CEILING(@TotalModels * @ModelRequests)
						END
						ELSE
						BEGIN 
							-- DECLARAMOS UNA TABLE PARA ALMACENAR EL RESULTADO DEL SP
							DECLARE @SP_RESULT AS TABLE ([CAASPR] DECIMAL(5, 2), [CAARPM] DECIMAL(5, 2), [CABSR] DECIMAL(5, 2))

							-- EJECUTAMOS EL SP QUE GENERA EL NUMERO DE REQUEST PROMEDIO POR MINUTO
							INSERT @SP_RESULT EXEC [dbo].[GetCognitiveAnalysisAverageRequestPerMinute] @InstallationId, @Vcc 
							--EXEC [dbo].[GetCognitiveAnalysisAverageRequestPerMinute] @InstallationId, @Vcc 

							DECLARE @AVERAGE_COGNITIVE_ANALYSIS_SECONDS_PER_REQUEST DECIMAL(5, 2) = 0
							DECLARE @AVERAGE_COGNITIVE_ANALYSIS_REQUEST_PER_MINUTE DECIMAL(5, 2) = 0
							DECLARE @DUMMY_VAR DECIMAL(5, 2) = 0
							
							-- RECUPERAMOS LOS DATOS DE REQUEST PROMEDIO Y TASA DE SEGURIDAD RECUPERADAS EN EL SP
							SELECT TOP 1 @AVERAGE_COGNITIVE_ANALYSIS_SECONDS_PER_REQUEST = SPR.[CAASPR],
									@AVERAGE_COGNITIVE_ANALYSIS_REQUEST_PER_MINUTE = SPR.[CAARPM],
									@DUMMY_VAR = SPR.[CABSR] -- <- ESTO SE BORRARA
								FROM @SP_RESULT SPR 

							-- CALCULAMOS EL MONTO PARA ANALISIS COGNITIVO DE LA SIGUIENTE FORMA:
							-- TOTAL MODELS * TARIFA POR TRANSACCION * (TIEMPO TRANSCRITO O TIEMPO TOTAL * PROMEDIO DE REQUEST POR MINUTO / 60) * (1 + TASA DE SEGURO DE FACTURACION)
							-- *TASA DE SEGURO DE FACTURACION: SI SE CAMBIA ESTE VALOR, SE DEBERA CAMBIAR TAMBIEN EN EL SP [dbo].[GetAccountVccRates]
							-- SET @CURRENT_RETAINED_AMOUNT = (@TotalModels * @COGNITIVE_ANALYSIS_RATE_PER_SECOND * @DurationTime * @AVERAGE_COGNITIVE_ANALYSIS_REQUEST_PER_MINUTE * (1 + @COGNITIVE_ANALYSIS_BILLING_SECURE_RATE)) / 60
							-- SET @CURRENT_RETAINED_AMOUNT = CEILING((@TotalModels * @COGNITIVE_ANALYSIS_RATE_PER_SECOND * @ROUNDED_VALID_DURATION_TIME * @AVERAGE_COGNITIVE_ANALYSIS_REQUEST_PER_MINUTE * (1 + @COGNITIVE_ANALYSIS_BILLING_SECURE_RATE)) / 60)
							-- SET @CURRENT_RETAINED_AMOUNT = @TotalModels * CEILING(@COGNITIVE_ANALYSIS_RATE_PER_SECOND * @DurationTime * (1 + @COGNITIVE_ANALYSIS_BILLING_SECURE_RATE) / @AVERAGE_COGNITIVE_ANALYSIS_SECONDS_PER_REQUEST) 
							-- SET @CURRENT_RETAINED_AMOUNT = CEILING(@TotalModels * @COGNITIVE_ANALYSIS_RATE_PER_SECOND * @ModelRequests * (1 + @COGNITIVE_ANALYSIS_BILLING_SECURE_RATE)) 
							SET @CURRENT_RETAINED_AMOUNT = CEILING(
																	@TotalModels 
																	*
																	@COGNITIVE_ANALYSIS_RATE_PER_SECOND
																	*
																	(
																		((@EstimatedNewCallPhrases + @AlreadyCallPhrases) * (1 + @COGNITIVE_ANALYSIS_BILLING_SECURE_RATE))
																		+
																		((@EstimatedNewTextPhrases + @AlreadyTextPhrases) * (1 + @COGNITIVE_ANALYSIS_TEXT_BILLING_SECURE_RATE))
																	)
															)
							SET @CURRENT_TOTAL_ESTIMATED_MODEL_REQUESTS = CEILING(@TotalModels * @ModelRequests)
						END					
					END 
				END
				
				--VERIFICAMOS QUE TENEMOS SALDO SUFICIENTE EN LA CARTERA DE TRANSCRICIOPNES DEL VCC
				IF NOT EXISTS (SELECT AVB.[AccountId] 
							FROM [dbo].[AccountVccBalance] AVB (NOLOCK)
							WHERE AVB.[InstallationId] = @InstallationId
								AND AVB.[AccountId] = @AccountID
								AND AVB.[VCC] = @VCC
								AND AVB.[SystemService] = @CURRENT_SYSTEM_SERVICE
								AND AVB.[AvailableBalance] >= @CURRENT_RETAINED_AMOUNT)
					-- SI NO ENCUENTRA RESULTADO, LANZAMOS MENSAJE AVISANDO NO HAY SUFICIENTE SALDO PARA TRANSCRIPCIONES
					RAISERROR('DATABASE_ERROR_INSUFFICIENT_SYSTEM_SERVICE_BALANCE', 16, 16)

				-- BLOQUEMOS TEMPORALMENTE EL REGISTRO DE SALDOS DEL SERVICO USANDO LOS MISMO PARAMETROS DE BUSQUEDA Y 
				-- ASEGURANDONOS QUE EL SALDO DISPONIBLE AUN ES MAYOR QUE EL SALDO A SETTEAR
				UPDATE [dbo].[AccountVccBalance] 
					SET [IsLocked] = 1,
						[LastModifiedByUserId] = @UserApi,
						[LastModifiedDate] = GETUTCDATE(),
						[TimeStamp] = GETUTCDATE()
					WHERE [InstallationId] = @InstallationId
						AND [AccountId] = @AccountID
						AND [VCC] = @VCC
						AND [SystemService] = @CURRENT_SYSTEM_SERVICE
						AND [AvailableBalance] >= @CURRENT_RETAINED_AMOUNT
			
				IF @@ROWCOUNT = 0
					RAISERROR('DATABASE_ERROR_INSUFFICIENT_SYSTEM_SERVICE_BALANCE', 16, 16)

				-- UNA VEZ BLOQUEADO EL REGISTRO DE LA TABLA ACCOUNTVCCBALANCE, REGISTRAMOS LAS SALIDAS Y ENTRADAS EN 
				-- LA TABLA DE MOVIMIENTOS ACCOUNSUBACCOUNTVCCBALANCEMOVEMENT (MOVIMIENTO DE SERVICIO TARGET -> FREE)
				-- DEFINIMOS LA DESCRIPCION DEL MOVIMIENTO PARA ESTE BLOQUE
				SET @MOVEMENT_DESCRIPTION = CONCAT('VCC ', @VCC, ': RETAIN ', @CURRENCY_CODE, ': ', ISNULL(@CURRENCY_SYMBOL, ''), ' ', @CURRENT_RETAINED_AMOUNT, ' FROM ', @CURRENT_SYSTEM_SERVICE, ' WALLET ON CREATING BATCH') 

				-- INCREMENTAMOS EN 1 EL PASO EN QUE NOS ENCONTRAMOS
				SET @BATCH_CURRENT_STEP = @BATCH_CURRENT_STEP + 1

				-- INSERTAMOS EN LA TABLA ACCOUNSUBACCOUNTVCCBALANCEMOVEMENT EL REGISTRO PARA LA RETENCION DE LA CARTERA TRANSCRIPTION
				INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
							([ID] ,[InstallationId] ,[AccountId] ,[SubAccountId] ,[VCC]
							,[Wallet] ,[Type] ,[CurrentStep] ,[TotalSteps] ,[Description]
							,[BatchId] 
							,[CurrencyCode] ,[Amount]
							,[PrevAvailableBalance] ,[PostAvailableBalance] 
							,[PrevCountableBalance] ,[PostCountableBalance] 
							,[CreatedDate] ,[TimeStamp])
					SELECT @ID ,AVB.[InstallationId] ,AVB.[AccountId] ,AVB.[SubAccountId] ,AVB.[VCC]
							,AVB.[SystemService] ,'RETAIN' ,@BATCH_CURRENT_STEP ,@BATCH_TOTAL_STEPS ,@MOVEMENT_DESCRIPTION
							,@ID 
							,@CURRENCY_CODE ,@CURRENT_RETAINED_AMOUNT
							,AVB.[AvailableBalance] ,AVB.[AvailableBalance] - @CURRENT_RETAINED_AMOUNT 
							,AVB.[CountableBalance] ,AVB.[CountableBalance]
							,GETUTCDATE() ,GETUTCDATE()
						FROM [dbo].[AccountVCCBalance] AVB (NOLOCK)
						WHERE [InstallationId] = @InstallationId 
							AND [AccountId] = @AccountId
							AND [VCC] = @VCC
							AND [SystemService] = @CURRENT_SYSTEM_SERVICE
							AND [AvailableBalance] >= @CURRENT_RETAINED_AMOUNT

				-- VARIAMOS EL INSERT SEGUN EL SERVICIO DEL SISTEMA
				IF @CURRENT_SYSTEM_SERVICE = 'TRANSCRIPTION'
				BEGIN
					-- INSERTAMOS EL PAQUETE DE TRANSCRIPCION EN LA TABLA GENERAL DE TRANSACCION
					INSERT INTO [dbo].[AccountSubAccountVCCSummaryTransaction] (
						[BatchID] ,[InstallationId] ,[VCC] ,[SystemService] ,[CurrencyCode]
						,[TotalInteraction] ,[TotalAsync] ,[TotalAsyncDetail] ,[AlreadyTextPhrases] ,[TotalSync] 
						,[TotalAudio] ,[AlreadyCallPhrases] ,[DurationTime] ,[AlreadyTranscribedAudio] ,[AlreadyDurationTime] 
						,[AlreadyRoundedDurationTime] ,[AlreadyTranscribedDurationTime] ,[AlreadyTranscribedRoundedDurationTime] ,[TranscriptionRatePerSecond] ,[BatchTypeName] 
						,[BatchTypeId] ,[ProcessId] ,[UserApi] ,[StartDate] ,[CreatedDate] 
						,[CreatedByuserId] ,[TimeStamp]
					)
					VALUES (
						@ID ,@InstallationId ,@VCC ,@CURRENT_SYSTEM_SERVICE ,@CURRENCY_CODE
						,@TotalInteraction ,@TotalAsync ,@TotalAsyncDetail ,@AlreadyTextPhrases ,@TotalSync 
						,@TotalAudio ,@AlreadyCallPhrases ,@DurationTime ,@AlreadyTranscribedAudio ,@AlreadyDurationTime 
						,@AlreadyRoundedDurationTime ,@AlreadyTranscribedDurationTime ,@AlreadyTranscribedRoundedDurationTime ,@TRANSCRIPTION_RATE_PER_SECOND ,@BatchTypeName 
						,@BatchTypeId ,@ProcessId ,@UserApi ,@StartDate ,GETUTCDATE() 
						,@UserApi ,GETUTCDATE()
					)
			
					-- LANZAMOS MENSAJE DE ERROR SI HUBO FALLA AL INSERTAR EL PAQUETE DE TRANSCRIPCION
					IF @@ROWCOUNT = 0
						RAISERROR('DATABASE_ERROR_TRANSCRIPTION_SERVICE_PACKAGE_SAVE_FAILED', 16, 16)
				END 

				IF @CURRENT_SYSTEM_SERVICE = 'SEMANTIC_ANALYSIS' 
				BEGIN
					-- INSERTAMOS EL PAQUETE DE TRANSCRIPCION EN LA TABLA GENERAL DE TRANSACCION
					INSERT INTO [dbo].[AccountSubAccountVCCSummaryTransaction] (
						[BatchID] ,[InstallationId] ,[VCC] ,[SystemService] ,[CurrencyCode]
						,[TotalAudio] ,[ValidDurationTime] ,[DurationTime] ,[RoundedDurationTime] ,[TotalQueries] 
						,[SemanticAnalysisRatePerTransaction] ,[BatchTypeName] ,[BatchTypeId] ,[ProcessId] ,[TemplateId] 
						,[UserApi] ,[StartDate] ,[CreatedDate] ,[CreatedByuserId] ,[TimeStamp]
					)
					VALUES (
						@ID ,@InstallationId ,@VCC ,@CURRENT_SYSTEM_SERVICE ,@CURRENCY_CODE
						,@TotalAudio ,@VALID_DURATION_TIME ,@DurationTime, @ROUNDED_VALID_DURATION_TIME ,@TotalQueries 
						,@SEMANTIC_ANALYSIS_RATE_PER_SECOND ,@BatchTypeName ,@BatchTypeId ,@ProcessId ,@TemplateId 
						,@UserApi ,@StartDate ,GETUTCDATE() ,@UserApi ,GETUTCDATE()
					)
			
					-- LANZAMOS MENSAJE DE ERROR SI HUBO FALLA AL INSERTAR EL PAQUETE DE TRANSCRIPCION
					IF @@ROWCOUNT = 0
						RAISERROR('DATABASE_ERROR_SEMANTIC_ANALYSIS_SERVICE_PACKAGE_SAVE_FAILED', 16, 16)
				END
				
				IF @CURRENT_SYSTEM_SERVICE = 'COGNITIVE_ANALYSIS' 
				BEGIN
					-- INSERTAMOS EL PAQUETE DE TRANSCRIPCION EN LA TABLA GENERAL DE TRANSACCION
					INSERT INTO [dbo].[AccountSubAccountVCCSummaryTransaction] (
						[BatchID] ,[InstallationId] ,[VCC] ,[SystemService] ,[CurrencyCode]
						,[TotalAudio] ,[ValidDurationTime] ,[DurationTime] ,[RoundedDurationTime] ,[TotalModels] 
						,[EstimatedRequestsPerModel] ,[EstimatedModelRequests] ,[CognitiveAnalysisRatePerTransaction] ,[BatchTypeName] ,[BatchTypeId] 
						,[ProcessId] ,[TemplateId] ,[UserApi] ,[StartDate] ,[CreatedDate] 
						,[CreatedByuserId] ,[TimeStamp]
					)
					VALUES (
						@ID ,@InstallationId ,@VCC ,@CURRENT_SYSTEM_SERVICE ,@CURRENCY_CODE
						,@TotalAudio ,@VALID_DURATION_TIME ,@DurationTime ,@ROUNDED_VALID_DURATION_TIME ,@TotalModels 
						,@ModelRequests ,@CURRENT_TOTAL_ESTIMATED_MODEL_REQUESTS ,@COGNITIVE_ANALYSIS_RATE_PER_SECOND ,@BatchTypeName ,@BatchTypeId 
						,@ProcessId ,@TemplateId ,@UserApi ,@StartDate ,GETUTCDATE() 
						,@UserApi ,GETUTCDATE()
					)
			
					-- LANZAMOS MENSAJE DE ERROR SI HUBO FALLA AL INSERTAR EL PAQUETE DE TRANSCRIPCION
					IF @@ROWCOUNT = 0
						RAISERROR('DATABASE_ERROR_COGNITIVE_ANALYSIS_SERVICE_PACKAGE_SAVE_FAILED', 16, 16)
				END

				-- DESBLOQUEMOS EL REGISTRO DE SALDOS DEL SERVICO USANDO LOS MISMOS PARAMETROS DE BUSQUEDA Y ACTUALIZAMOS EL SALDO DISPONIBLE
				UPDATE [dbo].[AccountVccBalance] 
					SET [IsLocked] = 1,
						[LastModifiedByUserId] = @UserApi,
						[LastModifiedDate] = GETUTCDATE(),
						[TimeStamp] = GETUTCDATE()
					WHERE [InstallationId] = @InstallationId
						AND [AccountId] = @AccountID
						AND [VCC] = @VCC
						AND [SystemService] = @CURRENT_SYSTEM_SERVICE
						AND [AvailableBalance] >= @CURRENT_RETAINED_AMOUNT

				-- DESBLOQUEMOS EL REGISTRO DE SALDOS DEL SERVICO USANDO LOS MISMO PARAMETROS DE BUSQUEDA Y ACTUALIZAMOS EL SALDO DISPONIBLE Y CONTABLE
				UPDATE [dbo].[AccountVCCBalance] 
					SET [IsLocked] = 0, 
						[AvailableBalance] = [AvailableBalance] - @CURRENT_RETAINED_AMOUNT,
						[RetainedBalance] = [RetainedBalance] + @CURRENT_RETAINED_AMOUNT,
						[LastModifiedByUserId] = @UserApi,
						[LastModifiedDate] = GETUTCDATE(),
						[TimeStamp] = GETUTCDATE() 
					WHERE [InstallationId] = @InstallationId 
						AND [AccountId] = @AccountId
						AND [VCC] = @VCC
						AND [SystemService] = @CURRENT_SYSTEM_SERVICE
						AND [AvailableBalance] >= @CURRENT_RETAINED_AMOUNT

				IF @@ROWCOUNT = 0
					RAISERROR('DATABASE_ERROR_INSUFFICIENT_SYSTEM_SERVICE_BALANCE', 16, 16)

			END 
			FETCH NEXT FROM MY_CURSOR INTO @CURRENT_SYSTEM_SERVICE
		END
		CLOSE MY_CURSOR
		DEALLOCATE MY_CURSOR

		-- RETORNAMOS RESULTADO 1
		SELECT 1 'result'

		-- COMPLETAMOS LA TRANSACCION
		COMMIT TRANSACTION

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH

END
GO
PRINT N'Modificando [dbo].[CreateAccountVCC]...';


GO
ALTER PROCEDURE [dbo].[CreateAccountVCC] (
	@PlatformHash Varchar(100) = NULL,
	@InstallationId Varchar(50) = NULL,
	@AccountId Varchar(50) = NULL,
	@SubAccountId Varchar(50) = NULL,
	@AccountVCC Varchar(50) = NULL,
	@TranscriptionRatePerHour Decimal(18, 5) = 0, 
	@TranscriptionRatePerSecond Decimal(18, 5) = 0, 
	@SemanticAnalysisRatePerTransaction Decimal(18, 5) = 0,  
	@CognitiveAnalysisRatePerTransaction Decimal(18, 5) = 0, 
	@VCCBalance Decimal(18, 5) = 0, 
	@Product Varchar(250) = NULL,
	@User Varchar(50) = NULL
	)
AS
BEGIN

	SET XACT_ABORT, NOCOUNT ON

    BEGIN TRY
        BEGIN TRANSACTION

		-- VALIDAMOS QUE EL COSTO DE TRANSCRIPCION POR HORA SEA MINIMO DE 0.036
		IF 0 < @TranscriptionRatePerHour AND @TranscriptionRatePerHour < 0.036
			RAISERROR('DATABASE_ERROR_TRANSCRIPTION_RATE_PER_HOUR_MINIMUN_EXCEPTION', 16, 16)

       -- DECLARAMOS UNA VARIABLE PARA ALMACENAR EL PRODUCTO RECORRIDO
		DECLARE @VCC_PRODUCT VARCHAR(50)

		-- DECLARAMOS UNA VARIABLE PARA ALMACENAR EL SPLIT DE PRODUCTOS
		DECLARE MY_CURSOR CURSOR 
		LOCAL STATIC READ_ONLY FORWARD_ONLY
			FOR 
			SELECT Value FROM [dbo].[split](@Product, ',')

		-- ABRIMOS Y RECORREMOS EL ARRAY DE PRODUCTOS
		OPEN MY_CURSOR
		FETCH NEXT FROM MY_CURSOR INTO @VCC_PRODUCT
		WHILE @@FETCH_STATUS = 0
		BEGIN 

			-- EJECUTAMOS EL PROCEDURE QUE INSERTA VCCPRODUCT
			EXEC AddAccountVCCProduct @InstallationId=@InstallationId, @AccountId=@AccountId, @SubAccountId=@SubAccountId, @VCC = @AccountVCC, @Product = @VCC_PRODUCT

			--Do something with Id here
			--PRINT @VCC_PRODUCT
			FETCH NEXT FROM MY_CURSOR INTO @VCC_PRODUCT
		END
		CLOSE MY_CURSOR
		DEALLOCATE MY_CURSOR

		-- DECLARAMOS UNA VARIABLE PARA PARA ALMACENAR EL USUARIO DE LA CUENTA
		DECLARE @ACCOUNT_USER VARCHAR(50)
		DECLARE @ACCOUNT_CURRENCY_CODE VARCHAR(10)

		-- RECUPERAMOS LA FECHA DE EXPIRACION DE LA CUENTA
		-- SI LA SUBCUENTA ES DIFERENTE DE NULO
		IF (@SubAccountId IS NOT NULL)
		BEGIN
			SELECT @ACCOUNT_USER = A.[User],
					@ACCOUNT_CURRENCY_CODE = A.[CurrencyCode]
				FROM dbo.[Account] A (NOLOCK) 
				WHERE A.[InstallationId] = @InstallationId 
					AND A.[AccountId] = @AccountId
					AND A.[SubAccountId] = @SubAccountId
					AND A.[Enabled] = 1
		END

		-- SI LA SUBCUENTA ES NULO
		ELSE
		BEGIN
			SELECT @ACCOUNT_USER = A.[User],
					@ACCOUNT_CURRENCY_CODE = A.[CurrencyCode]
				FROM dbo.[Account] A (NOLOCK) 
				WHERE A.[InstallationId] = @InstallationId 
					AND A.[AccountId] = @AccountId
					AND A.[SubAccountId] IS NULL
					AND A.[Enabled] = 1
		END

		-- INSERTAMOS EL REGISTRO EN VCC-ACCOUNT
		INSERT INTO [dbo].[AccountVCC]
				   ([InstallationId]
				   ,[AccountId]
				   ,[SubAccountId]
				   ,[VCC]
				   ,[CurrencyCode]
				   ,[TranscriptionRatePerHour]
				   ,[TranscriptionRatePerSecond]
				   ,[SemanticAnalysisRatePerTransaction]
				   ,[CognitiveAnalysisRatePerTransaction]
				   ,[CreatedDate]
				   ,[CreatedByUserId]
				   ,[TimeStamp])
			 VALUES
				   (@InstallationId
				   ,@AccountId
				   ,@SubAccountId
				   ,@AccountVCC
				   ,@ACCOUNT_CURRENCY_CODE
				   ,@TranscriptionRatePerHour
				   ,@TranscriptionRatePerSecond
				   ,@SemanticAnalysisRatePerTransaction
				   ,@CognitiveAnalysisRatePerTransaction 
				   ,GETUTCDATE()
				   ,@ACCOUNT_USER
				   ,GETUTCDATE())
			
		-- LANZAMOS MENSAJE DE ERROR SI EL INSERT FALLO
		IF @@ROWCOUNT = 0
			RAISERROR('DATABASE_ERROR_CREATE_ACCOUNTVCC_FAILED', 16, 16)

		-- EJECUTAMOS EL SP QUE INSERTAR/COMPLETA LOS SERVICIOS DEL SISTEMA PARA EL VCC
		EXEC [dbo].[CreateAccountVCCSystemService] @InstallationId, @AccountId, @SubAccountId, @AccountVCC, @User

		-- EJECUTAMOS EL SP QUE INSERTA EL PRIMER SALDO AL VCC EN LA TABLA ACCOUNTVCCBALANCE Y AccountSubAccountVCCBalanceMovement
		EXEC [dbo].[AccountSubAccountVCCBalanceCreate] @PlatformHash, @InstallationId, @AccountId, @SubAccountId, @AccountVCC, @VCCBalance, @User
		
		SELECT 1 'result'
        /* Other good code omitted*/

        COMMIT TRANSACTION;
    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION;

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE();
        DECLARE @Severity integer = ERROR_SEVERITY();
        DECLARE @State integer = ERROR_STATE();

        RAISERROR(@Message, @Severity, @State);
        RETURN -1;
    END CATCH;

END
GO
PRINT N'Modificando [dbo].[CreateMainAccount]...';


GO
ALTER PROCEDURE [dbo].[CreateMainAccount] (
	@PlatformHash VARCHAR(100) = NULL,
	@AccountId VARCHAR(50) = NULL,
	@User VARCHAR(50) = NULL,
	@Token VARCHAR(50) = NULL,
	@CountryCode VARCHAR(10) = NULL,
	@CurrencyCode VARCHAR(10) = NULL,
	@TranscriptionRatePerHour DECIMAL(18,5) = 0,
	@TranscriptionRatePerSecond DECIMAL(18,5) = 0,
	@SemanticAnalysisRatePerTransaction DECIMAL(18,5) = 0,
	@CognitiveAnalysisRatePerTransaction DECIMAL(18,5) = 0,
	@CreatedByUserId VARCHAR(50) = NULL
	)
AS
BEGIN
	SET XACT_ABORT, NOCOUNT ON

	-- ABRIMOS EL TRY PARA LA TRANSACCION
    BEGIN TRY
		BEGIN TRANSACTION
			-- VALIDAMOS QUE EL COSTO DE TRANSCRIPCION POR HORA SEA MINIMO DE 0.036
			IF 0 < @TranscriptionRatePerHour  AND @TranscriptionRatePerHour < 0.036
				RAISERROR('DATABASE_ERROR_TRANSCRIPTION_RATE_PER_HOUR_MINIMUN_EXCEPTION', 16, 16)

			-- DECLARAMOS UNA VARIABLE PARA DETERMINAR SI YA EXISTE UNA CUENTA PRINCIPAL
			DECLARE @MAIN_ACCOUNT_EXISTS BIT

			-- SETTEAMOS FALSE POR DEFECTO
			SET @MAIN_ACCOUNT_EXISTS = 0

			-- DECLARAMOS UNA VARIABLE PARA GUARDAR EL NIVEL DE LA CUENTA
			DECLARE @ACCOUNT_LEVEL INT

			-- SETTEAMOS NIVEL 2 POR DEFECTO(CUENTAS COMUNES)
			SET @ACCOUNT_LEVEL = 2

			-- CONSULTAMOS SI YA EXISTE UNA CUENTA LEVEL 1 PARA ESE PLATFORMHASH
			SELECT @MAIN_ACCOUNT_EXISTS = 1
				FROM [dbo].[Account] A 
				WHERE A.[PlatformHash] = @PlatformHash
					AND A.[AccountLevel] = 1

			-- VERIFICAMOS SI LA CUENTA PRINCIPAL NO EXISTE
			IF @MAIN_ACCOUNT_EXISTS = 0 
			BEGIN	
				--SETEAMOS NIVEL PARA LA CUENTA
				SET @ACCOUNT_LEVEL = 1
			END

			-- INSERTAMOS EL NUEVO REGISTRO EN LA TABLA PLATFORMDATA
			INSERT INTO [dbo].[Account]
					   ([ID]
					   ,[PlatformHash]
					   ,[AccountId]
					   ,[User]
					   ,[Token]
					   ,[AccountLevel]
					   ,[CountryCode]
					   ,[CurrencyCode]
					   ,[TranscriptionRatePerHour]
					   ,[TranscriptionRatePerSecond]
					   ,[SemanticAnalysisRatePerTransaction]
					   ,[CognitiveAnalysisRatePerTransaction]
					   ,[CreatedDate]
					   ,[CreatedByUserId]
					   ,[TimeStamp])
				 VALUES
					   (NEWID()
					   ,@PlatformHash
					   ,@AccountId
					   ,@User
					   ,@Token
					   ,@ACCOUNT_LEVEL
					   ,@CountryCode
					   ,@CurrencyCode
					   ,@TranscriptionRatePerHour
					   ,@TranscriptionRatePerSecond
					   ,@SemanticAnalysisRatePerTransaction
					   ,@CognitiveAnalysisRatePerTransaction
					   ,GETUTCDATE()
					   ,@CreatedByUserId
					   ,GETUTCDATE())

			-- RETORNAMOS UN VALOR DE RESPUESTA
			SELECT 1 AS 'result'

		-- CONSOLIDAMOS LA TRANSACCION
		COMMIT TRANSACTION

	 -- TERMINAMOS EL TRY PARA LA TRANSACCION
    END TRY
	BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH

    SET XACT_ABORT, NOCOUNT OFF
END
GO
PRINT N'Modificando [dbo].[GetAccount]...';


GO
ALTER PROCEDURE [dbo].[GetAccount]
(
	@PlatformHash VARCHAR(100) = NULL,
	@InstallationId VARCHAR(50) = NULL,
	@Id VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	-- VERIFICAMOS SI EXISTE UNA PLATAFORMA CON EL PLATFORMHASH INGRESADO
	IF NOT EXISTS(
		SELECT P.[PlatformHash] 
			FROM dbo.[Platform] P 
			WHERE P.[PlatformHash] = @PlatformHash
				AND P.[InstallationId] = @InstallationId
	)
	BEGIN
		-- SI NO EXISTE LA PLATAFORMA ENVIAMOS UN MENSAJE DE ERROR
		RAISERROR('DATABASE_ERROR_NOT_FOUND_PLATAFORM', 16, 16)
	END

	-- HACEMOS LA BUSQUEDA DE LA CUENTA POR MEDIO DE LA ID
	SELECT A.[AccountId]											'accountId',
			A.[SubAccountId]										'subAccountId',
			A.[CountryCode]										'countryCode',
			A.[CurrencyCode]										'currencyCode',
			A.[TranscriptionRatePerHour]							'transcriptionRatePerHour',
			A.[TranscriptionRatePerSecond]							'transcriptionRatePerSecond',
			A.[SemanticAnalysisRatePerTransaction]					'semanticAnalysisRatePerTransaction',
			A.[CognitiveAnalysisRatePerTransaction]					'cognitiveAnalysisRatePerTransaction',
			A.[User]												'user',
			A.[Token]												'token'
		FROM dbo.[Account] A (NOLOCK)
		WHERE A.[PlatformHash] = @PlatformHash
			AND A.[InstallationId] = @InstallationId
			AND A.[ID] = @Id

	SET NOCOUNT OFF
END
GO
PRINT N'Modificando [dbo].[GetAccounts]...';


GO

ALTER PROCEDURE [dbo].[GetAccounts]
(
	@SearchCriteria				VARCHAR(50) = NULL,
	@SearchExpression			VARCHAR(50) = NULL
)
AS
BEGIN
	-- ESTE SP RECUPERA TODAS CUENTAS PRESENTES EN EL SERVIDOR CENTRALIZADO
	-- TIENE TRES CRITERIOS DE BUSQUEDA POR EL COMBO EN LA VISTA (INSTALLATIONID, CLIENTNAME, ACCOUNTID)
	SET NOCOUNT ON

	-- EJECUTAMOS LA FUNCION QUE RECUPERA LAS CUENTAS
	SELECT A.[PlatformHash]					'platformHash',
			A.[ID]							'id',
			A.[InstallationId]				'installationId',
			A.[AccountId]					'accountId',
			A.[SubAccountId]				'subAccountId',
			A.[CountryCode]					'countryCode',
			A.[CurrencyCode]				'currencyCode',
			C.[LargeName]					'currencyLargeName',
			C.[Symbol]						'currencySymbol',
			PD.[ClientName]					'clientName',
			PD.[ClientContactName]			'clientContactName',
			PD.[ClientContactEmail]			'clientContactEmail',
			A.[Enabled]						'enabled',
			PD.[CreatedDate]				'createdDate'
	FROM [dbo].[Account] A (NOLOCK)
	INNER JOIN [dbo].[PlatformData] PD (NOLOCK) 
		ON PD.[PlatformHash] = A.[PlatformHash]
	INNER JOIN [dbo].[Currency] C (NOLOCK)
		ON C.Code = A.[CurrencyCode]
	/*
	INNER JOIN [dbo].[AccountVCC] AV (NOLOCK) 
		ON AV.[AccountId] = A.[AccountId]
			AND AV.[SubAccountId] = A.[SubAccountId]
	*/
	WHERE A.[Enabled] = 1
		AND ISNULL(PD.[InstallationId], '') LIKE CASE WHEN @SearchCriteria = 'installationId' THEN '%' + @SearchExpression + '%' ELSE '%' END
		AND ISNULL(PD.[ClientName], '') LIKE CASE WHEN @SearchCriteria = 'clientName' THEN '%' + @SearchExpression + '%' ELSE '%' END
		AND ISNULL(A.[AccountId], '') LIKE CASE WHEN @SearchCriteria = 'accountId' THEN '%' + @SearchExpression + '%' ELSE '%' END

	SET NOCOUNT OFF
END
GO
PRINT N'Modificando [dbo].[GetPlatform]...';


GO
ALTER PROCEDURE [dbo].[GetPlatform]
(
	@PlatformHash						VARCHAR(100)
)
AS
BEGIN
	SET NOCOUNT ON

	-- OBTENEMOS LOS DATOS DE LA SECCION DE PLATFORMA. SOLO RETORNA UNA FILA.
	BEGIN
		SELECT
			P.[PlatformHash]														'platformHash',
			P.[Type]																'type',
			P.[EnvironmentId]														'environmentId',
			PD.[Name]																'name',
			PD.[Description]														'description',
			PD.[CountryCode]														'countryCode',
			PD.[ClientName]															'clientName',
			P.[PublicIP]															'publicIP',
			PD.[ClientContactName]													'clientContactName',
			PD.[ClientContactEmail]													'clientContactEmail',
			PD.[InconcertContactName]												'inconcertContactName',
			PD.[CommercialContactName]												'commercialContactName',
			PD.[TechnicalContactName]												'technicalContactName',
			CAST(PD.[TranscriptionRatePerHour] AS DECIMAL(15, 8))					'transcriptionRatePerHour',
			CAST(PD.[SemanticAnalysisRatePerTransaction] AS DECIMAL(15, 8))			'semanticAnalysisRatePerTransaction',
			CAST(PD.[CognitiveAnalysisRatePerTransaction] AS DECIMAL(15, 8))		'cognitiveAnalysisRatePerTransaction'
		FROM [dbo].[Platform] P (NOLOCK)
		INNER JOIN [dbo].[PlatformData] PD (NOLOCK) 
			ON P.[PlatformHash] = PD.[PlatformHash]
		WHERE
			P.[PlatformHash]  = @PlatformHash
	END

	-- OBTENEMOS LOS DATOS DE USUARIO DE ACTIVACION. SOLO RETORNA UNA FILA.
	BEGIN
		SELECT TOP 1
			P.[PlatformHash]														'platformHash',
			A.[User]																'user',
			A.[Token]																'password',
			A.[AccountId]															'name',
			A.[CountryCode]															'countryCode',
			A.[CurrencyCode]														'currencyCode',
			CAST(AB.[AvailableBalance] AS DECIMAL(18, 5))							'balance',
			CAST(A.[TranscriptionRatePerHour] AS DECIMAL(18, 5))					'transcriptionRatePerHour',
			CAST(A.[SemanticAnalysisRatePerTransaction] AS DECIMAL(18, 5))			'semanticAnalysisRatePerTransaction',
			CAST(A.[CognitiveAnalysisRatePerTransaction] AS DECIMAL(18, 5))			'cognitiveAnalysisRatePerTransaction'
		FROM [dbo].[Platform] P (NOLOCK)
		INNER JOIN [dbo].[Account] A (NOLOCK)
			ON P.PlatformHash = A.PlatformHash
				AND A.[AccountLevel] = 1 -- EL VALOR INDICA CUENTA PRINCIPAL DE PLATAFORMA
		INNER JOIN [dbo].[AccountBalance] AB (NOLOCK)
			ON A.[AccountId] = AB.[AccountId]
		WHERE P.[PlatformHash]  = @PlatformHash
		ORDER BY A.[TimeStamp] DESC
	END

	-- OBTENEMOS DATOS DE LICENCIAS.
	BEGIN
		SELECT
			P.[PlatformHash]														'platformHash',
			PL.[Product]															'product',
			PL.[TotalLicense]														'totalLicense',
			PL.[ExpirationDate]														'expirationDate'
		FROM [dbo].[Platform] P (NOLOCK)
		INNER JOIN [dbo].[PlatformLicense] PL (NOLOCK) 
			ON P.[PlatformHash] = PL.[PlatformHash]
		WHERE P.[PlatformHash]  = @PlatformHash
	END

	SET NOCOUNT OFF
END
GO
PRINT N'Modificando [dbo].[SaveAccount]...';


GO
ALTER PROCEDURE [dbo].[SaveAccount] (
	@PlatformHash VARCHAR(100) = NULL,
	@Id VARCHAR(100) = NULL,
	@InstallationId VARCHAR(50) = NULL,
	@AccountId VARCHAR(50) = NULL,
	@SubAccountId VARCHAR(50) = NULL,
	@CountryCode VARCHAR(10) = NULL,
	@CurrencyCode VARCHAR(10) = NULL,
	@User VARCHAR(50) = NULL,
	@Token VARCHAR(50) = NULL,
	@TranscriptionRatePerHour DECIMAL(18,5) = 0,
	@TranscriptionRatePerSecond DECIMAL(18,5) = 0,
	@SemanticAnalysisRatePerTransaction DECIMAL(18,5) = 0,
	@CognitiveAnalysisRatePerTransaction DECIMAL(18,5) = 0,
	@CreatedByUserId VARCHAR(50) = NULL,
	@Balance  DECIMAL(18,5) = 0
	)
AS
BEGIN
	SET XACT_ABORT, NOCOUNT ON

	-- ABRIMOS EL TRY PARA LA TRANSACCION
    BEGIN TRY
        BEGIN TRANSACTION
			-- HACEMOS UNA BUSQUEDA DE LA PLATFORM INGRESADA
			IF NOT EXISTS (SELECT P.[PlatformHash]
							FROM dbo.[Platform] P (NOLOCK)
							WHERE P.[PlatformHash] = @PlatformHash)
			BEGIN
				RAISERROR('DATABASE_ERROR_NOT_FOUND_PLATAFORM', 16, 16)
			END

			-- VALIDAMOS QUE CUANDO EL COSTO DE TRANSCRIPCION POR HORA ES MAYOR A 0, SEA MINIMO DE 0.036 (ESTO SE DEBE CUMPLIR EN TODAS LA MONEDAS)
			IF 0 < @TranscriptionRatePerHour AND @TranscriptionRatePerHour < 0.036
				RAISERROR('DATABASE_ERROR_TRANSCRIPTION_RATE_PER_HOUR_MINIMUN_EXCEPTION', 16, 16)

			-- DECLARAMOS UNA VARIABLE PARA DETERMINAR SI YA EXISTE UNA CUENTA PRINCIPAL
			DECLARE @MAIN_ACCOUNT_EXISTS BIT

			-- SETTEAMOS FALSE POR DEFECTO
			SET @MAIN_ACCOUNT_EXISTS = 0

			-- DECLARAMOS UNA VARIABLE PARA GUARDAR EL NIVEL DE LA CUENTA
			DECLARE @ACCOUNT_LEVEL INT

			-- SETTEAMOS NIVEL 2 POR DEFECTO(CUENTAS COMUNES)
			SET @ACCOUNT_LEVEL = 2

			-- CONSULTAMOS SI YA EXISTE UNA CUENTA LEVEL 1 PARA ESE PLATFORMHASH
			SELECT @MAIN_ACCOUNT_EXISTS = 1
				FROM [dbo].[Account] A (NOLOCK) 
				WHERE A.[PlatformHash] = @PlatformHash
					AND A.[AccountLevel] = 1

			-- VERIFICAMOS SI LA CUENTA PRINCIPAL NO EXISTE
			IF @MAIN_ACCOUNT_EXISTS = 0 
			BEGIN	
				-- SETEAMOS NIVEL PARA LA CUENTA
				SET @ACCOUNT_LEVEL = 1
			END
			ELSE 
			BEGIN
				-- SI LA SUBCUENTA ES DIFERENTE DE NULO
				IF @SubAccountId IS NULL
				BEGIN
					-- HACEMOS UNA BUSQUEDA DE LA ACCOUNTID INGRESADA
					IF EXISTS (SELECT A.[AccountId]
								FROM dbo.[Account] A (NOLOCK)
								WHERE A.[PlatformHash] = @PlatformHash 
									AND A.[InstallationId] = @InstallationId
									AND A.[AccountId] =  @AccountId) AND @Id IS NULL
					BEGIN
						-- SI NO SE ENCONTRO LA CUENTA LANZAMOS UN MENSAJE DE ERROR
						RAISERROR('DATABASE_ERROR_ACCOUNT_ALREADY_EXISTS', 16, 16)
					END
					ELSE 
						-- SETEAMOS NIVEL PARA LA SUBCUENTA
						SET @ACCOUNT_LEVEL = 2
				END
				ELSE 
				BEGIN
					-- HACEMOS UNA BUSQUEDA DE LA ACCOUNTID INGRESADA
					IF EXISTS (SELECT A.[AccountId]
								FROM dbo.[Account] A (NOLOCK)
								WHERE A.[PlatformHash] = @PlatformHash 
									AND A.[InstallationId] = @InstallationId
									AND A.[AccountId] =  @AccountId)
					BEGIN
						-- SI ENCONTRO LA CUENTA, VALIDAMOS QUE NO EXISTE LA SUBCUENTA
						IF EXISTS (SELECT A.[AccountId]
									FROM dbo.[Account] A (NOLOCK)
									WHERE A.[PlatformHash] = @PlatformHash 
										AND A.[InstallationId] = @InstallationId
										AND A.[AccountId] =  @AccountId
										AND A.[SubAccountId] = @SubAccountId) AND @Id IS NULL
						BEGIN
							-- SI SE ENCONTRO LA SUBCUENTA LANZAMOS UN MENSAJE DE ERROR
							RAISERROR('DATABASE_ERROR_SUBACCOUNT_ALREADY_EXISTS', 16, 16)
						END
						ELSE 
							-- SETTEAMOS NIVEL 3(PARA SUBCUENTAS)
							SET @ACCOUNT_LEVEL = 3
					END
					ELSE 
						-- SI NO SE ENCONTRO LA CUENTA LANZAMOS UN MENSAJE DE ERROR
						RAISERROR('DATABASE_ERROR_NOT_FOUND_ACCOUNT', 16, 16)
					
				END
			END

			-- RAISERROR('DATABABASE_ERROR_DEMOSTRATION', 16, 16)

			-- SI EL ID ES NULL SE TRATA DE UNA NUEVA CUENTA/SUBCUENTA
			IF @Id IS NULL
			BEGIN
				-- INSERTAMOS EL NUEVO REGISTRO EN LA TABLA PLATFORMDATA
				INSERT INTO [dbo].[Account]
						   ([ID]
						   ,[PlatformHash]
						   ,[InstallationId]
						   ,[AccountId]
						   ,[SubAccountId]
						   ,[CountryCode]
						   ,[CurrencyCode]
						   ,[User]
						   ,[Token]
						   ,[AccountLevel]
						   ,[TranscriptionRatePerHour]
						   ,[TranscriptionRatePerSecond]
						   ,[SemanticAnalysisRatePerTransaction]
						   ,[CognitiveAnalysisRatePerTransaction]
						   ,[CreatedDate]
						   ,[CreatedByUserId]
						   ,[TimeStamp])
					 VALUES
						   (NEWID()
						   ,@PlatformHash
						   ,@InstallationId
						   ,@AccountId
						   ,@SubAccountId
						   ,@CountryCode
						   ,@CurrencyCode
						   ,@User
						   ,@Token
						   ,@ACCOUNT_LEVEL
						   ,@TranscriptionRatePerHour
						   ,@TranscriptionRatePerSecond
						   ,@SemanticAnalysisRatePerTransaction
						   ,@CognitiveAnalysisRatePerTransaction
						   ,GETUTCDATE()
						   ,@CreatedByUserId
						   ,GETUTCDATE())

				-- EJECUTAMOS EL SP QUE CREA EL REGISTRO EN ACCOUNTBALANCE Y LOS RESPECTIVOS REGISTROS EN LA TABLA DE MOVIMIENTOS
				EXEC [dbo].[AccountSubAccountVCCBalanceCreate] @PlatformHash, @InstallationId, @AccountId, @SubAccountId, NULL, @Balance = @Balance, @User = @User

			END
			-- SI EL ID ES DIFERENTE DE NULL ESTAMOS EDITANDO UN REGISTRO
			ELSE
			BEGIN
				-- ACTUALIZAMOS EL REGISTRO EN LA TABLA ACCOUNT
				UPDATE [dbo].[Account]
					SET [User] = @User
						   ,[Token] = @Token
						   -- ,[CountryCode] = @CountryCode
						   -- ,[CurrencyCode] = @CurrencyCode
						   ,[TranscriptionRatePerHour] = @TranscriptionRatePerHour
						   ,[TranscriptionRatePerSecond] = @TranscriptionRatePerSecond
						   ,[SemanticAnalysisRatePerTransaction] = @SemanticAnalysisRatePerTransaction
						   ,[CognitiveAnalysisRatePerTransaction] = @CognitiveAnalysisRatePerTransaction
						   ,[LastModifiedDate] = GETUTCDATE()
						   ,[LastModifiedByUserId] = @CreatedByUserId
						   ,[TimeStamp] = GETUTCDATE()
					WHERE [ID] = @Id
				
				-- SI HUBO CERO ACTUALIZACIONES, LANZAMOS EL MENSAJE DE ERROR
				IF @@ROWCOUNT = 0 
				BEGIN
					IF @ACCOUNT_LEVEL = 1 OR @ACCOUNT_LEVEL = 2 RAISERROR('DATABASE_ERROR_ACCOUNT_UPDATE_FAILED', 16, 16)
					IF @ACCOUNT_LEVEL = 3 RAISERROR('DATABASE_ERROR_SUBACCOUNT_UPDATE_FAILED', 16, 16)
				END 

				-- DECLARAMOS UNA VARIABLE PARA ALMACENAR LA CANTIDAD DE VCC ASOCIADOS
				DECLARE @ACCOUNT_VCCS INT = 0

				-- ACTUALIZAMOS LOS VCC QUE ESTAN ASOCIADOS A LA CUENTA Y/O SUBCUENTA ACTUALIZADA
				IF @ACCOUNT_LEVEL = 1 OR @ACCOUNT_LEVEL = 2
				BEGIN
					-- HACEMOS UN COUNT DE TODOS LOS VCC QUE TIENEN CUENTA CON EL VALOR RECIBIDO Y SUBCUENTA NULL
					SELECT @ACCOUNT_VCCS = COUNT(AV.[VCC])
						FROM [dbo].[AccountVCC] AV (NOLOCK) 
						WHERE AV.[InstallationId] = @InstallationId
							AND AV.[AccountId] = @AccountId
							AND AV.[SubAccountId] IS NULL

					-- SI ELRESULTADO ES MAYOR A CERO ACTUALIZAMOS LOS VCC
					IF @ACCOUNT_VCCS > 0
						UPDATE [dbo].[AccountVCC] 
							SET [TranscriptionRatePerHour] = @TranscriptionRatePerHour
								,[TranscriptionRatePerSecond] = @TranscriptionRatePerSecond
								,[SemanticAnalysisRatePerTransaction] = @SemanticAnalysisRatePerTransaction
								,[CognitiveAnalysisRatePerTransaction] = @CognitiveAnalysisRatePerTransaction
								,[LastModifiedDate] = GETUTCDATE()
								,[LastModifiedByUserId] = @CreatedByUserId
								,[TimeStamp] = GETUTCDATE()
							WHERE [InstallationId] = @InstallationId
							AND [AccountId] = @AccountId
							AND [SubAccountId] IS NULL

				END
				ELSE IF @ACCOUNT_LEVEL = 3 
				BEGIN
					-- HACEMOS UN COUNT DE TODOS LOS VCC QUE TIENEN CUENTA Y SUBCUENTA CON EL VALOR RECIBIDO 
					SELECT @ACCOUNT_VCCS = COUNT(AV.[VCC])
						FROM [dbo].[AccountVCC] AV (NOLOCK) 
						WHERE AV.[InstallationId] = @InstallationId
							AND AV.[AccountId] = @AccountId
							AND AV.[SubAccountId] = @SubAccountId

					-- SI ELRESULTADO ES MAYOR A CERO ACTUALIZAMOS LOS VCC
					IF @ACCOUNT_VCCS > 0
						UPDATE [dbo].[AccountVCC] 
							SET [TranscriptionRatePerHour] = @TranscriptionRatePerHour
								,[TranscriptionRatePerSecond] = @TranscriptionRatePerSecond
								,[SemanticAnalysisRatePerTransaction] = @SemanticAnalysisRatePerTransaction
								,[CognitiveAnalysisRatePerTransaction] = @CognitiveAnalysisRatePerTransaction
								,[LastModifiedDate] = GETUTCDATE()
								,[LastModifiedByUserId] = @CreatedByUserId
								,[TimeStamp] = GETUTCDATE()
							WHERE [InstallationId] = @InstallationId
							AND [AccountId] = @AccountId
							AND [SubAccountId] = @SubAccountId

				END

			END

			-- RETORNAMOS UN VALOR DE RESPUESTA
			SELECT 1 AS 'result'
			--SELECT SCOPE_IDENTITY()

		-- CONSOLIDAMOS LA TRANSACCION
		COMMIT TRANSACTION

    -- TERMINAMOS EL TRY PARA LA TRANSACCION
    END TRY
	BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH

    SET NOCOUNT OFF
END
GO
PRINT N'Modificando [dbo].[SaveAccountCurrency]...';


GO
ALTER PROCEDURE [dbo].[SaveAccountCurrency] (
	@InstallationId VARCHAR(50) = NULL,
	@AccountId VARCHAR(100) = NULL,
	@NewCountryCode VARCHAR(10) = NULL,
	@NewCurrencyCode VARCHAR(10) = NULL,
	@ConvertAccountRates BIT = 0,
	@convertSubAccountsRates BIT = 0,
	@NewTranscriptionRatePerHour DECIMAL(18,5) = 0,
	@NewTranscriptionRatePerSecond DECIMAL(18,5) = 0,
	@NewSemanticAnalysisRatePerTransaction DECIMAL(18,5) = 0,
	@NewCognitiveAnalysisRatePerTransaction DECIMAL(18,5) = 0,
	@ChangeValue DECIMAL(18, 5) = 0,
	@UserId VARCHAR(50) = NULL
) AS
BEGIN
	SET XACT_ABORT, NOCOUNT ON

	-- ABRIMOS EL TRY PARA LA TRANSACCION
    BEGIN TRY
		-- INICIAMOS LA TRANSACCION
        BEGIN TRANSACTION

			-- HACEMOS UNA BUSQUEDA DE LA PLATFORM INGRESADA
			IF NOT EXISTS (
				SELECT P.[PlatformHash]
					FROM dbo.[Platform] P (NOLOCK)
					WHERE P.[InstallationId] = @InstallationId
			)
			BEGIN
				RAISERROR('DATABASE_ERROR_NOT_FOUND_PLATAFORM', 16, 16)
			END

			-- VALIDAMOS QUE CUANDO EL COSTO DE TRANSCRIPCION POR HORA ES MAYOR A 0, SEA MINIMO DE 0.036 (ESTO SE DEBE CUMPLIR EN TODAS LA MONEDAS)
			IF 0 < @NewTranscriptionRatePerHour AND @NewTranscriptionRatePerHour < 0.036
				RAISERROR('DATABASE_ERROR_TRANSCRIPTION_RATE_PER_HOUR_MINIMUN_EXCEPTION', 16, 16)

			-- DECLARAMOS VARIABLES AUXILIARES PARA LA CONVERSION
			DECLARE @OLD_CURRENCY_CODE VARCHAR(5) = NULL
			DECLARE @OLD_CURRENCY_SYMBOL VARCHAR(5) = NULL
			DECLARE @NEW_CURRENCY_SYMBOL VARCHAR(5) = NULL
			DECLARE @ACCOUNT_VCC_TOTAL_AVAILABLE_BALANCE DECIMAL(18, 5) = 0
			DECLARE @ACCOUNT_VCC_TOTAL_RETAINED_BALANCE DECIMAL(18, 5) = 0
			DECLARE @ACCOUNT_SUBACCOUNT_TOTAL_AVAILABLE_BALANCE DECIMAL(18, 5) = 0
			DECLARE @ACCOUNT_SUBACCOUNT_TOTAL_ASSIGNED_BALANCE DECIMAL(18, 5) = 0
			DECLARE @ACCOUNT_SUBACCOUNT_VCC_TOTAL_AVAILABLE_BALANCE DECIMAL(18, 5) = 0
			DECLARE @ACCOUNT_SUBACCOUNT_VCC_TOTAL_RETAINED_BALANCE DECIMAL(18, 5) = 0

			-- DECLARARAMOS UNA VARIABLE PARA INDICAR LA SUBCUENTA Y VCC ACTUAL QUE ESTAMOS ACTUALIZANDO
			DECLARE @CURRENT_SUBACCOUNT VARCHAR(100) = NULL
			DECLARE @CURRENT_ACCOUNT_SUBACCOUNT_TOTAL_BALANCE DECIMAL(18, 5) = 0
			DECLARE @CURRENT_ACCOUNT_SUBACCOUNT_AVAILABLE_BALANCE DECIMAL(18, 5) = 0
			DECLARE @CURRENT_ACCOUNT_SUBACCOUNT_ASSIGNED_BALANCE DECIMAL(18, 5) = 0
			DECLARE @CURRENT_VCC VARCHAR(100) = NULL
			DECLARE @CURRENT_VCC_SYSTEM_SERVICE VARCHAR(100) = NULL
			-- DECLARAMOS LAS VARIABLES PARA LOS SALDOS DEL SERVICIO DEL SISTEMA
			DECLARE @CURRENT_VCC_SYSTEM_SERVICE_COUNTABLE_BALANCE DECIMAL(18, 5) = 0
			DECLARE @CURRENT_VCC_SYSTEM_SERVICE_AVAILABLE_BALANCE DECIMAL(18, 5) = 0
			DECLARE @CURRENT_VCC_SYSTEM_SERVICE_RETAINED_BALANCE DECIMAL(18, 5) = 0
			-- DECLARAMOS LA VARIABLE EN LA QUE ESCRIBIREMOS LA DESCRIPCION DEL MOVIMIENTO
			DECLARE @MOVEMENT_ID VARCHAR(50) = NULL
			DECLARE @MOVEMENT_DESCRIPTION VARCHAR(250) = NULL

			-- OBTENEMOS EL ACTUAL CURRENCY DE LA CUENTA
			SELECT @OLD_CURRENCY_CODE = A.[CurrencyCode],
					@OLD_CURRENCY_SYMBOL = C.[Symbol]
				FROM [dbo].[Account] A (NOLOCK) 
				INNER JOIN [dbo].[Currency] C (NOLOCK) 
					ON C.[Code] = A.[CurrencyCode]
				WHERE A.[InstallationId] = @InstallationId 
					AND A.[AccountId] = @AccountId 
					AND A.[SubAccountId] IS NULL
			
			-- OBTENEMOS EL SIMBOLO DEL NUEVO CURRENCY DE LA CUENTA
			SELECT @NEW_CURRENCY_SYMBOL = [Symbol] FROM [dbo].[Currency] WHERE [Code] = @NewCurrencyCode

			-- VALIDAMOS QUE EL CURRENCY ANTIGUO Y EL NUEVO SEAN DIFERENTES
			IF @OLD_CURRENCY_CODE = @NewCurrencyCode 
				RAISERROR('DATABASE_ERROR_NEW_CURRENCY_AND_OLD_CURRENCY_ARE_SAME', 16, 16)

			

			-- ACTUALIZAMOS EL CURRENCY EN LA TABLA ACCOUNT PARA TODOS
			UPDATE [dbo].[Account]
				SET [CountryCode] = @NewCountryCode,
					[CurrencyCode] = @NewCurrencyCode,
					[LastModifiedByUserId] = @UserId,
					[LastModifiedDate] = GETUTCDATE(),
					[TimeStamp] = GETUTCDATE()
				WHERE [InstallationId] = @InstallationId 
					AND [AccountId] = @AccountId



			-- DECLARAMOS UN CURSOR PARA RECORRER LOS VCC RELACIONADOS A LA CUENTA
			DECLARE RECURRENT_CURSOR_00 CURSOR 
			LOCAL STATIC READ_ONLY FORWARD_ONLY
			FOR 
			-- SELECCIONAMOS LOS VCC EXISTENTES
			SELECT [SubAccountId]
				FROM [AccountBalance] (NOLOCK) 
				WHERE [InstallationId] = @InstallationId 
					AND [AccountId] = @AccountId
					AND [TotalBalance] > 0

			-- ABRIMOS Y RECORREMOS EL ARRAY DE SERVICIOS DE SISTEMA DE VCC A ACTUALIZAR
			OPEN RECURRENT_CURSOR_00
			FETCH NEXT FROM RECURRENT_CURSOR_00 INTO @CURRENT_SUBACCOUNT
			WHILE @@FETCH_STATUS = 0
			BEGIN 
				-- OBTENEMOS LOS SALDOS DE LA CUENTA/SUBCUENTA ACTUAL
				SELECT @CURRENT_ACCOUNT_SUBACCOUNT_TOTAL_BALANCE = [TotalBalance],
						@CURRENT_ACCOUNT_SUBACCOUNT_AVAILABLE_BALANCE = [AvailableBalance],
						@CURRENT_ACCOUNT_SUBACCOUNT_ASSIGNED_BALANCE = [AssignedBalance]
					FROM [AccountBalance] (NOLOCK) 
					WHERE [InstallationId] = @InstallationId 
						AND [AccountId] = @AccountId
						AND (
							([SubAccountId] = @CURRENT_SUBACCOUNT)
							OR
							([SubAccountId] IS NULL AND @CURRENT_SUBACCOUNT IS NULL)
						)

				-- SETTEAMOS LOS NUEVOS SALDOS PARA EL VCC
				UPDATE [dbo].[AccountBalance]
					SET [TotalBalance] = @CURRENT_ACCOUNT_SUBACCOUNT_TOTAL_BALANCE * @ChangeValue
						,[AvailableBalance] = @CURRENT_ACCOUNT_SUBACCOUNT_AVAILABLE_BALANCE * @ChangeValue
						,[AssignedBalance] = @CURRENT_ACCOUNT_SUBACCOUNT_ASSIGNED_BALANCE * @ChangeValue
					WHERE [InstallationId] = @InstallationId 
						AND [AccountId] = @AccountId
						AND (
							([SubAccountId] = @CURRENT_SUBACCOUNT)
							OR
							([SubAccountId] IS NULL AND @CURRENT_SUBACCOUNT IS NULL)
						)

				-- GENERAMOS UN ID PARA EL ACTUAL MOVIMIENTO
				SET @MOVEMENT_ID = NEWID()

				-- PASO 1 DE 3 INICIO (REGISTRAR LA CONVERSION DEL SALDO TOTAL DE LA CUENTA) --

				-- SETTEAMOS EL DESCRIPTION PARA EL PRIMER PASO 
				SET @MOVEMENT_DESCRIPTION = CASE WHEN @CURRENT_SUBACCOUNT IS NULL THEN CONCAT('ACCOUNT ', @AccountId) ELSE CONCAT('SUBACCOUNT ', @CURRENT_SUBACCOUNT) END
				SET @MOVEMENT_DESCRIPTION += CONCAT(': ADDING ', @NewCurrencyCode, ': ', ISNULL(@NEW_CURRENCY_SYMBOL, ''), @CURRENT_ACCOUNT_SUBACCOUNT_TOTAL_BALANCE * @ChangeValue, ' TO FREE WALLET (TOTAL_BALLANCE) (CONVERTED FROM ', @OLD_CURRENCY_CODE, ': ', ISNULL(@OLD_CURRENCY_SYMBOL, ''), @CURRENT_ACCOUNT_SUBACCOUNT_TOTAL_BALANCE, ' TO ', @NewCurrencyCode, ' WITH EXCHANGE RATE ', @ChangeValue, ')')
		
				-- CONVERTIRMOS EL SALDO CONTABLE
				INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] (
							[ID] ,[InstallationId] ,[AccountId] ,[SubAccountId] 
							,[Wallet] ,[Type] ,[CurrentStep] ,[TotalSteps] ,[Description]
							,[CurrencyCode] ,[Amount] ,[PrevAvailableBalance] ,[PostAvailableBalance] ,[PrevCountableBalance] 
							,[PostCountableBalance] ,[CreatedDate]
				)
					SELECT @MOVEMENT_ID ,AB.[InstallationId] ,AB.[AccountId] ,AB.[SubAccountId] 
							,'FREE' ,'CURRENCY_CHANGE' ,1 ,3 ,@MOVEMENT_DESCRIPTION
							,@NewCurrencyCode ,@CURRENT_ACCOUNT_SUBACCOUNT_TOTAL_BALANCE * @ChangeValue  ,@CURRENT_ACCOUNT_SUBACCOUNT_AVAILABLE_BALANCE ,@CURRENT_ACCOUNT_SUBACCOUNT_TOTAL_BALANCE ,@CURRENT_ACCOUNT_SUBACCOUNT_TOTAL_BALANCE * @ChangeValue
							,@CURRENT_ACCOUNT_SUBACCOUNT_TOTAL_BALANCE * @ChangeValue ,GETUTCDATE()
						FROM [dbo].[AccountBalance] AB (NOLOCK)
						WHERE [InstallationId] = @InstallationId 
							AND [AccountId] = @AccountId
							AND (
								([SubAccountId] = @CURRENT_SUBACCOUNT)
								OR
								([SubAccountId] IS NULL AND @CURRENT_SUBACCOUNT IS NULL)
							)

				-- PASO 1 DE 3 FIN (REGISTRAR LA CONVERSION DEL SALDO TOTAL DE LA CUENTA) --



				-- PASO 2 DE 3 INICIO (REGISTRAR LA CONVERSION DEL SALDO DISPONIBLE DE LA CUENTA) --

				-- SETTEAMOS EL DESCRIPTION PARA EL PRIMER PASO 
				SET @MOVEMENT_DESCRIPTION = CASE WHEN @CURRENT_SUBACCOUNT IS NULL THEN CONCAT('ACCOUNT ', @AccountId) ELSE CONCAT('SUBACCOUNT ', @CURRENT_SUBACCOUNT) END
				SET @MOVEMENT_DESCRIPTION += CONCAT(': ADDING ', @NewCurrencyCode, ': ', ISNULL(@NEW_CURRENCY_SYMBOL, ''), @CURRENT_ACCOUNT_SUBACCOUNT_AVAILABLE_BALANCE * @ChangeValue, ' TO FREE WALLET (AVAILABLE_BALLANCE) (CONVERTED FROM ', @OLD_CURRENCY_CODE, ': ', ISNULL(@OLD_CURRENCY_SYMBOL, ''), @CURRENT_ACCOUNT_SUBACCOUNT_AVAILABLE_BALANCE, ' TO ', @NewCurrencyCode, ' WITH EXCHANGE RATE ', @ChangeValue, ')')
		
				-- CONVERTIRMOS EL SALDO DISPONIBLE
				INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] (
							[ID] ,[InstallationId] ,[AccountId] ,[SubAccountId] 
							,[Wallet] ,[Type] ,[CurrentStep] ,[TotalSteps] ,[Description]
							,[CurrencyCode] ,[Amount] ,[PrevAvailableBalance] ,[PostAvailableBalance] ,[PrevCountableBalance] 
							,[PostCountableBalance] ,[CreatedDate]
				)
					SELECT @MOVEMENT_ID ,AB.[InstallationId] ,AB.[AccountId] ,AB.[SubAccountId] 
							,'FREE' ,'CURRENCY_CHANGE' ,2 ,3 ,@MOVEMENT_DESCRIPTION
							,@NewCurrencyCode ,@CURRENT_ACCOUNT_SUBACCOUNT_AVAILABLE_BALANCE * @ChangeValue ,@CURRENT_ACCOUNT_SUBACCOUNT_AVAILABLE_BALANCE ,@CURRENT_ACCOUNT_SUBACCOUNT_AVAILABLE_BALANCE * @ChangeValue ,AB.[TotalBalance] 
							,AB.[TotalBalance] * @ChangeValue ,GETUTCDATE()
						FROM [dbo].[AccountBalance] AB (NOLOCK)
						WHERE [InstallationId] = @InstallationId 
							AND [AccountId] = @AccountId
							AND (
								([SubAccountId] = @CURRENT_SUBACCOUNT)
								OR
								([SubAccountId] IS NULL AND @CURRENT_SUBACCOUNT IS NULL)
							)

				-- PASO 2 DE 3 FIN (REGISTRAR LA CONVERSION DEL SALDO DISPONIBLE DE LA CUENTA) --



				-- PASO 3 DE 3 INICIO (REGISTRAR LA CONVERSION DEL SALDO ASIGNADO DE LA CUENTA) --

				-- SETTEAMOS EL DESCRIPTION PARA EL PRIMER PASO 
				SET @MOVEMENT_DESCRIPTION = CASE WHEN @CURRENT_SUBACCOUNT IS NULL THEN CONCAT('ACCOUNT ', @AccountId) ELSE CONCAT('SUBACCOUNT ', @CURRENT_SUBACCOUNT) END
				SET @MOVEMENT_DESCRIPTION += CONCAT(': ADDING ', @NewCurrencyCode, ': ', ISNULL(@NEW_CURRENCY_SYMBOL, ''), @CURRENT_ACCOUNT_SUBACCOUNT_ASSIGNED_BALANCE * @ChangeValue, ' TO FREE WALLET (ASSIGNED_BALLANCE) (CONVERTED FROM ', @OLD_CURRENCY_CODE, ': ', ISNULL(@OLD_CURRENCY_SYMBOL, ''), @CURRENT_ACCOUNT_SUBACCOUNT_ASSIGNED_BALANCE, ' TO ', @NewCurrencyCode, ' WITH EXCHANGE RATE ', @ChangeValue, ')')
		
				-- CONVERTIRMOS EL SALDO ASIGNADO
				INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] (
							[ID] ,[InstallationId] ,[AccountId] ,[SubAccountId] 
							,[Wallet] ,[Type] ,[CurrentStep] ,[TotalSteps] ,[Description]
							,[CurrencyCode] ,[Amount] ,[PrevAvailableBalance] ,[PostAvailableBalance] ,[PrevCountableBalance] 
							,[PostCountableBalance] ,[CreatedDate]
				)
					SELECT @MOVEMENT_ID ,AB.[InstallationId] ,AB.[AccountId] ,AB.[SubAccountId] 
							,'FREE' ,'CURRENCY_CHANGE' ,3 ,3 ,@MOVEMENT_DESCRIPTION
							,@NewCurrencyCode ,@CURRENT_ACCOUNT_SUBACCOUNT_ASSIGNED_BALANCE * @ChangeValue ,AB.[AvailableBalance] ,AB.[AvailableBalance] ,AB.[TotalBalance] 
							,AB.[TotalBalance]  ,GETUTCDATE()
						FROM [dbo].[AccountBalance] AB (NOLOCK)
						WHERE [InstallationId] = @InstallationId 
							AND [AccountId] = @AccountId
							AND (
								([SubAccountId] = @CURRENT_SUBACCOUNT)
								OR
								([SubAccountId] IS NULL AND @CURRENT_SUBACCOUNT IS NULL)
							)

				-- PASO 3 DE 3 FIN (REGISTRAR LA CONVERSION DEL SALDO ASIGNADO DE LA CUENTA) --
				
				FETCH NEXT FROM RECURRENT_CURSOR_00 INTO @CURRENT_SUBACCOUNT
			END
			CLOSE RECURRENT_CURSOR_00
			DEALLOCATE RECURRENT_CURSOR_00


			-- ACTUALIZAMOS EL CURRENCY EN LA TABLA ACCOUNTVCC PARA TODOS
			UPDATE [dbo].[AccountVCC]
				SET [CurrencyCode] = @NewCurrencyCode,
					[LastModifiedByUserId] = @UserId,
					[LastModifiedDate] = GETUTCDATE(),
					[TimeStamp] = GETUTCDATE()
				WHERE [InstallationId] = @InstallationId 
					AND [AccountId] = @AccountId

			-- SI EL FLAG CONVERTIR TARIFAS DE CUENTAS LLEGA 1, ACTUALIZAMOS LAS TARIFAS CON EL VALOR RECIBIDO
			IF @ConvertAccountRates = 1
			BEGIN
				-- ACTUALIZAMOS LAS TARIFAS CONVERTIDAS PARA LA CUENTA
				UPDATE [dbo].[Account]
					SET [TranscriptionRatePerHour] = @NewTranscriptionRatePerHour,
						[TranscriptionRatePerSecond] = @NewTranscriptionRatePerSecond,
						[SemanticAnalysisRatePerTransaction] = @NewSemanticAnalysisRatePerTransaction,
						[CognitiveAnalysisRatePerTransaction] = @NewCognitiveAnalysisRatePerTransaction,
						[LastModifiedByUserId] = @UserId,
						[LastModifiedDate] = GETUTCDATE(),
						[TimeStamp] = GETUTCDATE()
					WHERE [InstallationId] = @InstallationId 
						AND [AccountId] = @AccountId
						AND (
							[AccountLevel] = 1
							OR 
							[AccountLevel] = 2
						)
						AND [SubAccountId] IS NULL

				-- ACTUALIZAMOS LAS TARIFAS DE LOS VCCS QUE DESCIENDEN DIRECTAMENTE DE LA CUENTA
				UPDATE AV 
					SET AV.[TranscriptionRatePerHour] = A.[TranscriptionRatePerHour],
						AV.[TranscriptionRatePerSecond] = A.[TranscriptionRatePerSecond],
						AV.[SemanticAnalysisRatePerTransaction] = A.[SemanticAnalysisRatePerTransaction],
						AV.[CognitiveAnalysisRatePerTransaction] = A.[CognitiveAnalysisRatePerTransaction],
						AV.[LastModifiedByUserId] = @UserId,
						AV.[LastModifiedDate] = GETUTCDATE(),
						AV.[TimeStamp] = GETUTCDATE()
					FROM [dbo].[AccountVCC] AV
					INNER JOIN [dbo].[Account] A
						ON A.[InstallationId] = AV.[InstallationId]
							AND A.[AccountId] = AV.[AccountId]
							AND A.[SubAccountId] IS NULL
					WHERE AV.[InstallationId] = @InstallationId 
						AND AV.[AccountId] = @AccountId
						AND AV.[SubAccountId] IS NULL

				-- SI EL FLAG CONVERTIR TARIFAS DE SUBCUENTAS LLEGA 1, ACTUALIZAMOS LAS TARIFAS CON EL VALOR PRODUCTO
				IF @convertSubAccountsRates = 1
				BEGIN
					-- ACTUALIZAMOS EL CURRENCY EN LA TABLA ACCOUNT PARA TODOS LOS SUBACCOUNTS DE LA CUENTA
					UPDATE [dbo].[Account]
						SET [TranscriptionRatePerHour] = CASE 
								WHEN [TranscriptionRatePerHour] * @ChangeValue < 0.036 
									AND [TranscriptionRatePerHour] * @ChangeValue > 0
								THEN 0.036 ELSE [TranscriptionRatePerHour] * @ChangeValue END,
							[TranscriptionRatePerSecond] = CASE
								WHEN [TranscriptionRatePerHour] * @ChangeValue < 0.036 
									AND [TranscriptionRatePerHour] * @ChangeValue > 0
								THEN 0.00001 ELSE [TranscriptionRatePerSecond] * @ChangeValue / 3600 END,
							[SemanticAnalysisRatePerTransaction] = CASE
								WHEN [SemanticAnalysisRatePerTransaction] * @ChangeValue < 0.00001
									AND [SemanticAnalysisRatePerTransaction] * @ChangeValue > 0
								THEN 0.00001 ELSE [SemanticAnalysisRatePerTransaction] * @ChangeValue END,
							[CognitiveAnalysisRatePerTransaction] = CASE
								WHEN [CognitiveAnalysisRatePerTransaction] * @ChangeValue < 0.00001
									AND [CognitiveAnalysisRatePerTransaction] * @ChangeValue > 0
								THEN 0.00001 ELSE [CognitiveAnalysisRatePerTransaction] * @ChangeValue END,
							[LastModifiedByUserId] = @UserId,
							[LastModifiedDate] = GETUTCDATE(),
							[TimeStamp] = GETUTCDATE()
						WHERE [InstallationId] = @InstallationId 
							AND [AccountId] = @AccountId
							AND [AccountLevel] = 3
							AND [SubAccountId] IS NOT NULL

					-- ACTUALIZAMOS LAS TARIFAS DE LOS VCCS QUE HEREDAN DE SUBCUENTA
					UPDATE AV 
						SET AV.[TranscriptionRatePerHour] = A.[TranscriptionRatePerHour],
							AV.[TranscriptionRatePerSecond] = A.[TranscriptionRatePerSecond],
							AV.[SemanticAnalysisRatePerTransaction] = A.[SemanticAnalysisRatePerTransaction],
							AV.[CognitiveAnalysisRatePerTransaction] = A.[CognitiveAnalysisRatePerTransaction],
							AV.[LastModifiedByUserId] = @UserId,
							AV.[LastModifiedDate] = GETUTCDATE(),
							AV.[TimeStamp] = GETUTCDATE()
						FROM [dbo].[AccountVCC] AV
						INNER JOIN [dbo].[Account] A
							ON A.[InstallationId] = AV.[InstallationId]
								AND A.[AccountId] = AV.[AccountId]
								AND A.[SubAccountId] = AV.[SubAccountId]
						WHERE AV.[InstallationId] = @InstallationId 
							AND AV.[AccountId] = @AccountId
							AND AV.[SubAccountId] IS NOT NULL
				END
			END
			
			-- DECLARAMOS UN CURSOR PARA RECORRER LOS VCC RELACIONADOS A LA CUENTA
			DECLARE RECURRENT_CURSOR_01 CURSOR 
			LOCAL STATIC READ_ONLY FORWARD_ONLY
			FOR 
			-- SELECCIONAMOS LOS VCC EXISTENTES
			SELECT [SubAccountId],
					[VCC]
				FROM [AccountVCC] (NOLOCK) 
				WHERE [InstallationId] = @InstallationId 
					AND [AccountId] = @AccountId

			-- ABRIMOS Y RECORREMOS EL ARRAY DE SERVICIOS DE SISTEMA DE VCC A ACTUALIZAR
			OPEN RECURRENT_CURSOR_01
			FETCH NEXT FROM RECURRENT_CURSOR_01 INTO @CURRENT_SUBACCOUNT, @CURRENT_VCC
			WHILE @@FETCH_STATUS = 0
			BEGIN 
				-- DECLARAMOS OTRO CURSOS PARA RECORRER LOS WALLET DEL VCC RECORRIDO
				DECLARE RECURRENT_CURSOR_02 CURSOR 
				LOCAL STATIC READ_ONLY FORWARD_ONLY
				FOR 
				-- SELECCIONAMOS LOS WALLETS CON SALDO DISPONIBLE DEL VCC ACTUAL
				SELECT [SystemService]
					FROM [AccountVCCBalance] (NOLOCK) 
					WHERE [InstallationId] = @InstallationId 
						AND [AccountId] = @AccountId
						AND (
							([SubAccountId] = @CURRENT_SUBACCOUNT)
							OR
							([SubAccountId] IS NULL AND @CURRENT_SUBACCOUNT IS NULL)
						)
						AND [VCC] = @CURRENT_VCC
						AND [AvailableBalance] > 0

				-- ABRIMOS Y RECORREMOS EL ARRAY DE CAMPOS A ACTUALIZAR
				OPEN RECURRENT_CURSOR_02
				FETCH NEXT FROM RECURRENT_CURSOR_02 INTO @CURRENT_VCC_SYSTEM_SERVICE
				WHILE @@FETCH_STATUS = 0
				BEGIN 
					-- OBTENEMOS LOS SALDOS DE SERVICIO DEL SISTEMA ACTUAL
					SELECT @CURRENT_VCC_SYSTEM_SERVICE_COUNTABLE_BALANCE = [CountableBalance],
							@CURRENT_VCC_SYSTEM_SERVICE_AVAILABLE_BALANCE = [AvailableBalance],
							@CURRENT_VCC_SYSTEM_SERVICE_RETAINED_BALANCE = [RetainedBalance]
						FROM [AccountVCCBalance] (NOLOCK) 
						WHERE [InstallationId] = @InstallationId 
							AND [AccountId] = @AccountId
							AND (
								([SubAccountId] = @CURRENT_SUBACCOUNT)
								OR
								([SubAccountId] IS NULL AND @CURRENT_SUBACCOUNT IS NULL)
							)
							AND [VCC] = @CURRENT_VCC
							AND [SystemService] = @CURRENT_VCC_SYSTEM_SERVICE

					-- GENERAMOS UN ID PARA EL ACTUAL MOVIMIENTO
					SET @MOVEMENT_ID = NEWID()


		
				
		
					-- PASO 1 DE 3 INICIO (ACTUALIZAR EL SALDO CONTABLE DE VCC PARA EL SERVCIO DEL SISTEMA ACTUAL) --

					-- SETTEAMOS EL DESCRIPTION PARA EL PRIMER PASO 
					SET @MOVEMENT_DESCRIPTION = CONCAT('VCC ', @CURRENT_VCC, ': ADDING ', @NewCurrencyCode, ': ', ISNULL(@NEW_CURRENCY_SYMBOL, ''), @CURRENT_VCC_SYSTEM_SERVICE_COUNTABLE_BALANCE * @ChangeValue, ' TO ', @CURRENT_VCC_SYSTEM_SERVICE, '-COUNTABLE_BALANCE WALLET DUE TO CURRENCY CHANGE (CONVERTED FROM ', @OLD_CURRENCY_CODE, ': ', ISNULL(@OLD_CURRENCY_SYMBOL, ''), @CURRENT_VCC_SYSTEM_SERVICE_COUNTABLE_BALANCE, ' TO ', @NewCurrencyCode, ' WITH EXCHANGE RATE ', @ChangeValue, ')')
		
					-- CONVERTIRMOS EL SALDO CONTABLE
					INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] (
								[ID] ,[InstallationId] ,[AccountId] ,[SubAccountId] ,[VCC]
								,[Wallet] ,[Type] ,[CurrentStep] ,[TotalSteps] ,[Description]
								,[CurrencyCode] ,[Amount] ,[PrevAvailableBalance] ,[PostAvailableBalance] ,[PrevCountableBalance] 
								,[PostCountableBalance] ,[CreatedDate]
					)
						SELECT @MOVEMENT_ID ,AVB.[InstallationId] ,AVB.[AccountId] ,AVB.[SubAccountId] ,AVB.[VCC]
								,AVB.[SystemService] ,'CURRENCY_CHANGE' ,1 ,3 ,@MOVEMENT_DESCRIPTION
								,@NewCurrencyCode ,@CURRENT_VCC_SYSTEM_SERVICE_COUNTABLE_BALANCE * @ChangeValue ,AVB.[AvailableBalance] ,AVB.[AvailableBalance] ,@CURRENT_VCC_SYSTEM_SERVICE_COUNTABLE_BALANCE * @ChangeValue 
								,@CURRENT_VCC_SYSTEM_SERVICE_COUNTABLE_BALANCE * @ChangeValue ,GETUTCDATE()
							FROM [dbo].[AccountVCCBalance] AVB (NOLOCK)
							WHERE [InstallationId] = @InstallationId 
								AND [AccountId] = @AccountId
								AND (
									([SubAccountId] = @CURRENT_SUBACCOUNT)
									OR
									([SubAccountId] IS NULL AND @CURRENT_SUBACCOUNT IS NULL)
								)
								AND [VCC] = @CURRENT_VCC
								AND [SystemService] = @CURRENT_VCC_SYSTEM_SERVICE

		
					-- SETTEAMOS LOS NUEVOS SALDOS PARA EL VCC
					UPDATE [dbo].[AccountVCCBalance]
						SET [CountableBalance] = @CURRENT_VCC_SYSTEM_SERVICE_COUNTABLE_BALANCE * @ChangeValue
						WHERE [InstallationId] = @InstallationId 
								AND [AccountId] = @AccountId
								AND (
									([SubAccountId] = @CURRENT_SUBACCOUNT)
									OR
									([SubAccountId] IS NULL AND @CURRENT_SUBACCOUNT IS NULL)
								)
								AND [VCC] = @CURRENT_VCC
								AND [SystemService] = @CURRENT_VCC_SYSTEM_SERVICE

					-- PASO 1 DE 3 FIN (ACTUALIZAR EL SALDO CONTABLE DE VCC PARA EL SERVCIO DEL SISTEMA ACTUAL) --





					-- PASO 2 DE 3 INICIO (ACTUALIZAR EL SALDO DISPONIBLE DE VCC PARA EL SERVCIO DEL SISTEMA ACTUAL) --
		
					-- SETTEAMOS EL DESCRIPTION PARA EL SEGUNDO PASO 
					SET @MOVEMENT_DESCRIPTION = CONCAT('VCC ', @CURRENT_VCC, ': ADDING ', @NewCurrencyCode, ': ', ISNULL(@NEW_CURRENCY_SYMBOL, ''), @CURRENT_VCC_SYSTEM_SERVICE_AVAILABLE_BALANCE * @ChangeValue, ' TO ', @CURRENT_VCC_SYSTEM_SERVICE, '-AVAILABLE_BALANCE WALLET DUE TO CURRENCY CHANGE (CONVERTED FROM ', @OLD_CURRENCY_CODE, ': ', ISNULL(@OLD_CURRENCY_SYMBOL, ''), @CURRENT_VCC_SYSTEM_SERVICE_AVAILABLE_BALANCE, ' TO ', @NewCurrencyCode, ' WITH EXCHANGE RATE ', @ChangeValue, ')')
		
					-- CONVERTIMOS EL SALDO DISPONIBLE
					INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] (
								[ID] ,[InstallationId] ,[AccountId] ,[SubAccountId] ,[VCC]
								,[Wallet] ,[Type] ,[CurrentStep] ,[TotalSteps] ,[Description]
								,[CurrencyCode] ,[Amount] ,[PrevAvailableBalance] ,[PostAvailableBalance] ,[PrevCountableBalance] 
								,[PostCountableBalance] ,[CreatedDate]
					)
						SELECT @MOVEMENT_ID ,AVB.[InstallationId] ,AVB.[AccountId] ,AVB.[SubAccountId] ,AVB.[VCC]
								,AVB.[SystemService] ,'CURRENCY_CHANGE' ,2 ,3 ,@MOVEMENT_DESCRIPTION
								,@NewCurrencyCode ,@CURRENT_VCC_SYSTEM_SERVICE_AVAILABLE_BALANCE * @ChangeValue ,AVB.[AvailableBalance] ,@CURRENT_VCC_SYSTEM_SERVICE_AVAILABLE_BALANCE * @ChangeValue ,AVB.[CountableBalance] 
								,@CURRENT_VCC_SYSTEM_SERVICE_AVAILABLE_BALANCE * @ChangeValue ,GETUTCDATE()
							FROM [dbo].[AccountVCCBalance] AVB (NOLOCK)
							WHERE [InstallationId] = @InstallationId 
								AND [AccountId] = @AccountId
								AND (
									([SubAccountId] = @CURRENT_SUBACCOUNT)
									OR
									([SubAccountId] IS NULL AND @CURRENT_SUBACCOUNT IS NULL)
								)
								AND [VCC] = @CURRENT_VCC
								AND [SystemService] = @CURRENT_VCC_SYSTEM_SERVICE

		
					-- SETTEAMOS LOS NUEVOS SALDOS PARA EL VCC
					UPDATE [dbo].[AccountVCCBalance]
						SET [AvailableBalance] = @CURRENT_VCC_SYSTEM_SERVICE_AVAILABLE_BALANCE * @ChangeValue
						WHERE [InstallationId] = @InstallationId 
								AND [AccountId] = @AccountId
								AND (
									([SubAccountId] = @CURRENT_SUBACCOUNT)
									OR
									([SubAccountId] IS NULL AND @CURRENT_SUBACCOUNT IS NULL)
								)
								AND [VCC] = @CURRENT_VCC
								AND [SystemService] = @CURRENT_VCC_SYSTEM_SERVICE
					
					-- PASO 2 DE 3 FIN (ACTUALIZAR EL SALDO DISPONIBLE DE VCC PARA EL SERVCIO DEL SISTEMA ACTUAL) --





					-- PASO 3 DE 3 INICIO (ACTUALIZAR EL SALDO RETENIDO DE VCC PARA EL SERVCIO DEL SISTEMA ACTUAL) --
		
					-- SETTEAMOS EL DESCRIPTION PARA EL TERCER PASO 
					SET @MOVEMENT_DESCRIPTION = CONCAT('VCC ', @CURRENT_VCC, ': ADDING ', @NewCurrencyCode, ': ', ISNULL(@NEW_CURRENCY_SYMBOL, ''), @CURRENT_VCC_SYSTEM_SERVICE_RETAINED_BALANCE * @ChangeValue, ' TO ', @CURRENT_VCC_SYSTEM_SERVICE, '-RETAINED_BALANCE WALLET DUE TO CURRENCY CHANGE (CONVERTED FROM ', @OLD_CURRENCY_CODE, ': ', ISNULL(@OLD_CURRENCY_SYMBOL, ''), @CURRENT_VCC_SYSTEM_SERVICE_RETAINED_BALANCE, ' TO ', @NewCurrencyCode, ' WITH EXCHANGE RATE ', @ChangeValue, ')')
		
					-- CONVERSION DE SALDOS SE HARA EN SEIS PASOS PARA LA TABLA DE MOVIMIENTOS, PASO 3 AGREGAR EL SALDO RETENIDO NUEVO
					INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] (
								[ID] ,[InstallationId] ,[AccountId] ,[SubAccountId] ,[VCC]
								,[Wallet] ,[Type] ,[CurrentStep] ,[TotalSteps] ,[Description]
								,[CurrencyCode] ,[Amount] ,[PrevAvailableBalance] ,[PostAvailableBalance] ,[PrevCountableBalance] 
								,[PostCountableBalance] ,[CreatedDate]
					)
						SELECT @MOVEMENT_ID ,AVB.[InstallationId] ,AVB.[AccountId] ,AVB.[SubAccountId] ,AVB.[VCC]
								,AVB.[SystemService] ,'CURRENCY_CHANGE' ,3 ,3 ,@MOVEMENT_DESCRIPTION
								,@NewCurrencyCode ,@CURRENT_VCC_SYSTEM_SERVICE_RETAINED_BALANCE * @ChangeValue ,AVB.[AvailableBalance] ,AVB.[AvailableBalance] ,AVB.[CountableBalance] 
								,AVB.[CountableBalance] ,GETUTCDATE()
							FROM [dbo].[AccountVCCBalance] AVB (NOLOCK)
							WHERE [InstallationId] = @InstallationId 
								AND [AccountId] = @AccountId
								AND (
									([SubAccountId] = @CURRENT_SUBACCOUNT)
									OR
									([SubAccountId] IS NULL AND @CURRENT_SUBACCOUNT IS NULL)
								)
								AND [VCC] = @CURRENT_VCC
								AND [SystemService] = @CURRENT_VCC_SYSTEM_SERVICE

		
					-- SETTEAMOS LOS NUEVOS SALDOS PARA EL VCC
					UPDATE [dbo].[AccountVCCBalance]
						SET [RetainedBalance] = @CURRENT_VCC_SYSTEM_SERVICE_RETAINED_BALANCE * @ChangeValue
						WHERE [InstallationId] = @InstallationId 
								AND [AccountId] = @AccountId
								AND (
									([SubAccountId] = @CURRENT_SUBACCOUNT)
									OR
									([SubAccountId] IS NULL AND @CURRENT_SUBACCOUNT IS NULL)
								)
								AND [VCC] = @CURRENT_VCC
								AND [SystemService] = @CURRENT_VCC_SYSTEM_SERVICE
		
					-- PASO 3 DE 3 FIN (ACTUALIZAR EL SALDO RETENIDO DE VCC PARA EL SERVCIO DEL SISTEMA ACTUAL) --

		



					FETCH NEXT FROM RECURRENT_CURSOR_02 INTO @CURRENT_VCC_SYSTEM_SERVICE
				END
				CLOSE RECURRENT_CURSOR_02
				DEALLOCATE RECURRENT_CURSOR_02

				FETCH NEXT FROM RECURRENT_CURSOR_01 INTO @CURRENT_SUBACCOUNT, @CURRENT_VCC
			END
			CLOSE RECURRENT_CURSOR_01
			DEALLOCATE RECURRENT_CURSOR_01

			-- RETORNAMOS RESULTADO 1
			SELECT 1 'result'


			

			
		-- CONSOLIDAMOS LA TRANSACCION
		COMMIT TRANSACTION

    -- TERMINAMOS EL TRY PARA LA TRANSACCION
    END TRY
	BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH
	
    SET NOCOUNT OFF
END
GO
PRINT N'Modificando [dbo].[AccountSubAccountVCCBatchSummaryClose]...';


GO
ALTER PROCEDURE [dbo].[AccountSubAccountVCCBatchSummaryClose](
	@BatchId VARCHAR(50) = NULL, 
	@ClosedDate DATETIME = NULL
)
AS
BEGIN
	-- ESTE SP TIENE LA FINALIDAD DE ACTUALIZAR LOS SALDOS DE LAS CARTERAS DEL VCC
	SET XACT_ABORT, NOCOUNT ON

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY

		-- INICIAMOS LA TRANSACCION
		BEGIN TRANSACTION

		-- VERIFICAMOS QUE TENEMOS LOS PARAMETROS NECESARIOS
		IF @BatchId IS NULL 
			-- LANZAMOS MENSAJE AVISANDO QUE FALTA ACCOUNTID
			RAISERROR('DATABASE_ERROR_MISSING_PARAMETERS', 16, 16)

		-- DECLARAMOS UNA VARIABLE PARA ALMACENAR LOS SYSTEMSERVICE RELACIONADOS AL BATCH
		DECLARE MY_CURSOR CURSOR 
		LOCAL STATIC READ_ONLY FORWARD_ONLY
			FOR 
			-- CONSULTAMOS LOS SYSTEMSERVICE DEL BATCH PARA LA ACTUALIZACION DE SALDOS
			SELECT ASAVST.[InstallationId],
					ASAVST.[VCC], 
					ASAVST.[SystemService], 
					ASAVST.[Rate]
				FROM [dbo].[AccountSubAccountVCCSummaryTransaction] ASAVST (NOLOCK)
				WHERE ASAVST.[BatchID] = @BatchId

		-- DECLARARAMOS UNA VARIABLE PARA INDICAR EL BACTH ACTUAL QUE ESTAMOS RECORRIENDO
		DECLARE @CURRENT_INSTALLATION_ID VARCHAR(50) = ''
		DECLARE @CURRENT_VCC VARCHAR(50) = ''
		DECLARE @CURRENT_SYSTEM_SERVICE VARCHAR(50) = ''
		DECLARE @CURRENT_RATE DECIMAL(18, 5) = 0

		-- DECLARAMOS VARIABLES DE AYUDA QUE CAMBIARAN EN EL BUCLE
		DECLARE @MOVEMENT_DESCRIPTION NVARCHAR(250) = NULL
		DECLARE @BATCH_TOTAL_STEPS INT = 0
		DECLARE @BATCH_CURRENT_STEP INT = 0
		DECLARE @INITIAL_CALCULATED_AMOUNT DECIMAL(18,5) = 0
		DECLARE @FINAL_CALCULATED_AMOUNT DECIMAL(18,5) = 0
		DECLARE @CURRENCY_CODE VARCHAR(5) = NULL
		DECLARE @CURRENCY_SYMBOL NVARCHAR(10) = NULL

		-- ABRIMOS Y RECORREMOS EL ARRAY DE CAMPOS A ACTUALIZAR
		OPEN MY_CURSOR
		FETCH NEXT FROM MY_CURSOR INTO @CURRENT_INSTALLATION_ID, @CURRENT_VCC, @CURRENT_SYSTEM_SERVICE, @CURRENT_RATE
		WHILE @@FETCH_STATUS = 0
		BEGIN 
			-- EN CASO EL CURRENCYCODE SEA NULO LO BUSCAMOS
			IF @CURRENCY_CODE IS NULL
			BEGIN
				-- OBTENEMOS EL CURRENCYCODE Y CURRENCYSYMBOL CON QUE SE REGISTR EL BATCH
				SELECT @CURRENCY_CODE = ASAVBM.[CurrencyCode],
						@CURRENCY_SYMBOL = C.[Symbol]
					FROM [dbo].[AccountSubAccountVCCBalanceMovement] ASAVBM (NOLOCK)
					INNER JOIN [dbo].[Currency] C (NOLOCK)
						ON C.[Code] = ASAVBM.[CurrencyCode]
					WHERE ASAVBM.[InstallationId] = @CURRENT_INSTALLATION_ID 
						AND ASAVBM.[BatchID] = @BatchId
						AND ASAVBM.[CurrentStep] = 1
			END

			-- OBTENEMOS LOS DATOS GENERALES DEL BATCH EN LA TABLA DE MOVIMIENTOS
			SELECT @BATCH_CURRENT_STEP = MAX(ASAVBM.[CurrentStep]),
					@BATCH_TOTAL_STEPS = ASAVBM.[TotalSteps]
				FROM [dbo].[AccountSubAccountVCCBalanceMovement] ASAVBM (NOLOCK)
				WHERE ASAVBM.[BatchID] = @BatchId
				GROUP BY ASAVBM.[InstallationId], ASAVBM.[VCC], ASAVBM.[TotalSteps]

			-- OBTENEMOS LOS DATOS ESPECIFICOS DEL BATCH-SYSTEMSERVICE EN LA TABLA DE MOVIMIENTOS
			SELECT @INITIAL_CALCULATED_AMOUNT = ASAVBM.[Amount]
				FROM [dbo].[AccountSubAccountVCCBalanceMovement] ASAVBM (NOLOCK)
				WHERE ASAVBM.[BatchID] = @BatchId
					AND ASAVBM.[InstallationId] = @CURRENT_INSTALLATION_ID
					AND ASAVBM.[Wallet] = @CURRENT_SYSTEM_SERVICE

			-- PRINT 'GOT DATA FROM MOVEMENTS TABLE'
			-- PRINT CONCAT('TOTAL STEPS ', @BATCH_TOTAL_STEPS)
			-- PRINT CONCAT('CURRENT STEP ', @BATCH_CURRENT_STEP)

			-- VERIFICAMOS QUE LOS PASO AUN NO HAYAN SIDO COMPLETADOS
			IF @BATCH_TOTAL_STEPS > @BATCH_CURRENT_STEP
			BEGIN
				PRINT 'BEGINNING DEBT'

				-- PARA REGISTRAR LOS MOVIMIENTOS AUMENTAMOS EL CURRENT_STEP PARA REGISTRAR EL SIGUIENTE MOVIMIENTO
				SET @BATCH_CURRENT_STEP = @BATCH_CURRENT_STEP + 1

				-- DEFINIMOS LA DESCRIPCION DEL MOVIMIENTO DEBITAJE
				SET @MOVEMENT_DESCRIPTION = CONCAT('VCC ', @CURRENT_VCC, ': DEBT ', @CURRENCY_CODE, ': ', ISNULL(@CURRENCY_SYMBOL, ''), ' ', @CURRENT_RATE, ' FROM ', @CURRENT_SYSTEM_SERVICE, ' WALLET ON CLOSING BATCH')

				-- INSERTAMOS EN LA TABLA ACCOUNSUBACCOUNTVCCBALANCEMOVEMENT EL REGISTRO PARA EL COBRO A LA CARTERA RECORRIDA
				INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
							([ID] ,[InstallationId] ,[AccountId] ,[SubAccountId] ,[VCC]
							,[Wallet] ,[Type] ,[CurrentStep] ,[TotalSteps] ,[Description]
							,[BatchId] 
							,[CurrencyCode] ,[Amount]
							,[PrevAvailableBalance] ,[PostAvailableBalance] 
							,[PrevCountableBalance] ,[PostCountableBalance] 
							,[CreatedDate] ,[TimeStamp])
					SELECT @BatchId ,AVB.[InstallationId] ,AVB.[AccountId] ,AVB.[SubAccountId] ,AVB.[VCC]
							,AVB.[SystemService] ,'DEBT' ,@BATCH_CURRENT_STEP ,@BATCH_TOTAL_STEPS ,@MOVEMENT_DESCRIPTION
							,@BatchId 
							,@CURRENCY_CODE ,@CURRENT_RATE
							,AVB.[AvailableBalance] ,AVB.[AvailableBalance]
							,AVB.[CountableBalance] ,AVB.[CountableBalance] - @CURRENT_RATE 
							,GETUTCDATE() ,GETUTCDATE()
						FROM [dbo].[AccountVCCBalance] AVB (NOLOCK)
						WHERE [InstallationId] = @CURRENT_INSTALLATION_ID 
							AND [VCC] = @CURRENT_VCC
							AND [SystemService] = @CURRENT_SYSTEM_SERVICE
							--AND [CountableBalance] > @FINAL_CALCULATED_AMOUNT

				-- DESBLOQUEMOS EL REGISTRO DE SALDOS DEL SERVICO USANDO LOS MISMO PARAMETROS DE BUSQUEDA Y ACTUALIZAMOS EL SALDO DISPONIBLE Y CONTABLE
				UPDATE [dbo].[AccountVCCBalance] 
					SET [IsLocked] = 0, 
						[AvailableBalance] = [AvailableBalance] + (@INITIAL_CALCULATED_AMOUNT - @CURRENT_RATE),
						[RetainedBalance] = [RetainedBalance] - @INITIAL_CALCULATED_AMOUNT,
						[CountableBalance] = [CountableBalance] - @CURRENT_RATE,
						--[LastModifiedByUserId] = @UserApi,
						[LastModifiedDate] = GETUTCDATE(),
						[TimeStamp] = GETUTCDATE() 
					WHERE [InstallationId] = @CURRENT_INSTALLATION_ID 
						AND [VCC] = @CURRENT_VCC
						AND [SystemService] = @CURRENT_SYSTEM_SERVICE

				-- AUMENTAMOS EL CURRENT_STEP PARA REGISTRAR EL SIGUIENTE MOVIMIENTO
				SET @BATCH_CURRENT_STEP = @BATCH_CURRENT_STEP + 1

				-- DEFINIMOS LA DESCRIPCION DEL MOVIMIENTO LIBERAR
				SET @MOVEMENT_DESCRIPTION = CONCAT('VCC ', @CURRENT_VCC, ': RELEASE ', @CURRENCY_CODE, ': ', ISNULL(@CURRENCY_SYMBOL, ''), ' ', (@INITIAL_CALCULATED_AMOUNT - @CURRENT_RATE), ' FROM ', @CURRENT_SYSTEM_SERVICE, ' WALLET ON CLOSING BATCH')
			
				-- INSERTAMOS EN LA TABLA ACCOUNSUBACCOUNTVCCBALANCEMOVEMENT EL REGISTRO PARA LA LIBERACION DE SALDO RETENIDO A LA CARTERA RECORRIDA
				INSERT INTO [dbo].[AccountSubAccountVCCBalanceMovement] 
							([ID] ,[InstallationId] ,[AccountId] ,[SubAccountId] ,[VCC]
							,[Wallet] ,[Type] ,[CurrentStep] ,[TotalSteps] ,[Description]
							,[BatchId] 
							,[CurrencyCode] ,[Amount]
							,[PrevAvailableBalance] ,[PostAvailableBalance] 
							,[PrevCountableBalance] ,[PostCountableBalance] 
							,[CreatedDate] ,[TimeStamp])
					SELECT @BatchId ,AVB.[InstallationId] ,AVB.[AccountId] ,AVB.[SubAccountId] ,AVB.[VCC]
							,AVB.[SystemService] ,'RELEASE' ,@BATCH_CURRENT_STEP ,@BATCH_TOTAL_STEPS ,@MOVEMENT_DESCRIPTION
							,@BatchId 
							,@CURRENCY_CODE ,@INITIAL_CALCULATED_AMOUNT - @CURRENT_RATE
							,AVB.[AvailableBalance] ,AVB.[AvailableBalance] + (@INITIAL_CALCULATED_AMOUNT - @CURRENT_RATE)
							,AVB.[CountableBalance] ,AVB.[CountableBalance] 
							,GETUTCDATE() ,GETUTCDATE()
						FROM [dbo].[AccountVCCBalance] AVB (NOLOCK)
						WHERE [InstallationId] = @CURRENT_INSTALLATION_ID 
							AND [VCC] = @CURRENT_VCC
							AND [SystemService] = @CURRENT_SYSTEM_SERVICE
			END

			-- ACTUALIZAR EL BATCH COMO ISFINISHED TRUE EN CASO NO HAYA SIDO MARCADO TODAVIA
			UPDATE [dbo].[AccountSubAccountVCCSummaryTransaction]
				SET [IsFinished] = 1,
					[EndDate] = CASE WHEN @ClosedDate IS NULL THEN [EndDate] ELSE @ClosedDate END,
					[Status] = CASE 
						-- WHEN [Status] LIKE 'CANCELLED%' THEN 'FINISHED_BY_CANCELLED' 
						WHEN [Status] = 'CANCELLED_BY_PAUSED_TIMEOUT' THEN 'FINISHED_BY_CANCELLED_BY_PAUSED_TIMEOUT' 
						WHEN [Status] = 'CANCELLED_BY_RUNNING_TIMEOUT' THEN 'FINISHED_BY_CANCELLED_BY_RUNNING_TIMEOUT' 
						WHEN [Status] = 'CANCELLED_BY_USER' THEN 'FINISHED_BY_CANCELLED_BY_USER' 
						ELSE 'FINISHED' END, 
					--[Status] = 'FINISHED', -- ESTE ESTADO SE CAMBIARA PARA DAR OPCION A FINISHED COMO COMPLETADO SIN INTERVENSION O FINISHED_BY_USER CUANDO EL USUARIO DECIDE TERMINARLO
					[LasModifiedDate] = GETUTCDATE(),
					[TimeStamp] = GETUTCDATE()
				WHERE [InstallationId] = @CURRENT_INSTALLATION_ID 
					AND [BatchID] = @BatchId
					AND [VCC] = @CURRENT_VCC
					AND [SystemService] = @CURRENT_SYSTEM_SERVICE
					--AND [IsFinished] = 0
					AND [Status] NOT IN ('FINISHED', 'FINISHED_BY_CANCELLED_PAUSED_TIMEOUT', 'FINISHED_BY_CANCELLED_RUNNING_TIMEOUT', 'FINISHED_BY_CANCELLED_BY_USER', 'FINISHED_BY_CANCELLED')

			FETCH NEXT FROM MY_CURSOR INTO @CURRENT_INSTALLATION_ID, @CURRENT_VCC, @CURRENT_SYSTEM_SERVICE, @CURRENT_RATE
		END
		CLOSE MY_CURSOR
		DEALLOCATE MY_CURSOR

		-- RETORNAMOS RESULTADO 1
		SELECT 1 'result'

		-- COMPLETAMOS LA TRANSACCION
		COMMIT TRANSACTION

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH

	SET XACT_ABORT, NOCOUNT OFF
END
GO
PRINT N'Modificando [dbo].[SavePlatformData]...';


GO
ALTER PROCEDURE [dbo].[SavePlatformData] (
	@PlatformHash VARCHAR(100) = NULL,
	@Name VARCHAR(100) = NULL,
	@Description VARCHAR(100) = NULL,
	@CountryCode VARCHAR(10) = NULL,
	@ClientName VARCHAR(100) = NULL,
	@ClientContactName VARCHAR(100) = NULL,
	@ClientContactEmail VARCHAR(50) = NULL,
	@InconcertContactName VARCHAR(100) = NULL,
	@CommercialContactName VARCHAR(100) = NULL,
	@TechnicalContactName VARCHAR(100) = NULL,
	@TranscriptionRatePerHour DECIMAL(18,5) = 0,
	@TranscriptionRatePerSecond DECIMAL(18,5) = 0,
	@SemanticAnalysisRatePerTransaction DECIMAL(18,5) = 0,
	@CognitiveAnalysisRatePerTransaction DECIMAL(18,5) = 0,
	@User VARCHAR(50) = NULL
)
AS
BEGIN
	SET XACT_ABORT, NOCOUNT ON

	-- ABRIMOS EL TRY PARA LA TRANSACCION
    BEGIN TRY
		BEGIN TRANSACTION
			-- VALIDAMOS QUE EL COSTO DE TRANSCRIPCION POR HORA SEA MINIMO DE 0.036
			IF 0 < @TranscriptionRatePerHour AND @TranscriptionRatePerHour < 0.036
				RAISERROR('DATABASE_ERROR_TRANSCRIPTION_RATE_PER_HOUR_MINIMUN_EXCEPTION', 16, 16)

			-- ACTUALIZAMOS LOS CAMPOS CLIENTCONTACTEMAIL, TRANSCRIPTIONRATEPERHOUR, SEMANTICANALYSISRATEPERTRANSACTION, COGNITIVEANALYSISRATEPERTRANSACTION, LASTMODIFIEDDATE, LASTMODIFIEDBYUSERID y TIMESTAMP DE ACUERDO EL PLATFORMHASH.
			UPDATE [dbo].[PlatformData] SET
				[Name] = @Name,
				[Description] = @Description,
				[CountryCode] = @CountryCode,
				[ClientName] = @ClientName,
				[ClientContactEmail] = @ClientContactEmail,
				[InconcertContactName] = @InconcertContactName,
				[CommercialContactName] = @CommercialContactName,
				[TechnicalContactName] = @TechnicalContactName,
				[TranscriptionRatePerHour] = @TranscriptionRatePerHour,
				[SemanticAnalysisRatePerTransaction] = @SemanticAnalysisRatePerTransaction,
				[CognitiveAnalysisRatePerTransaction] = @CognitiveAnalysisRatePerTransaction,
				[LastModifiedDate] = GETUTCDATE(),
				[TimeStamp] = GETUTCDATE()
			WHERE PlatformHash = @PlatformHash

			IF(@@ROWCOUNT = 0)
			BEGIN
				-- INSERTAMOS EL NUEVO REGISTRO EN LA TABLA PLATFORMDATA.
				INSERT INTO [dbo].[PlatformData]
					   ([PlatformHash]
					   ,[Name]
					   ,[Description]
					   ,[CountryCode]
					   ,[ClientName]
					   ,[ClientContactName]
					   ,[ClientContactEmail]
					   ,[InconcertContactName]
					   ,[CommercialContactName]
					   ,[TechnicalContactName]
					   ,[TranscriptionRatePerHour]
					   ,[TranscriptionRatePerSecond]
					   ,[SemanticAnalysisRatePerTransaction]
					   ,[CognitiveAnalysisRatePerTransaction]
					   ,[CreatedDate]
					   ,[CreatedByUserId]
					   ,[TimeStamp])
				 VALUES
					   (@PlatformHash
					   ,@Name
					   ,@Description
					   ,@CountryCode
					   ,@ClientName
					   ,@ClientContactName
					   ,@ClientContactEmail
					   ,@InconcertContactName
					   ,@CommercialContactName
					   ,@TechnicalContactName
					   ,@TranscriptionRatePerHour
					   ,@TranscriptionRatePerSecond
					   ,@SemanticAnalysisRatePerTransaction
					   ,@CognitiveAnalysisRatePerTransaction
					   ,GETUTCDATE()
					   ,@User
					   ,GETUTCDATE())

			END

			-- INSERTAMOS EL CLIENTNAME EN CASO NO EXISTA
			EXEC [dbo].[SaveClient] @ClientName
	
			-- RETORNAMOS UN VALOR DE RESPUESTA
			SELECT 1 AS 'result'

		-- CONSOLIDAMOS LA TRANSACCION
		COMMIT TRANSACTION

	 -- TERMINAMOS EL TRY PARA LA TRANSACCION
    END TRY
	BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH

    SET XACT_ABORT, NOCOUNT OFF
END
GO
PRINT N'Modificando [dbo].[GetAccountSubAccountBalance]...';


GO
-- ESTA FUNCION RETORNA LOS SALDO DE UNA CUENTA O SUBCUENTA DE TODOS LOS PRODUCTOS O UNO ESPECIFICO
ALTER  PROCEDURE [dbo].[GetAccountSubAccountBalance] (
	@InstallationId Varchar(100) = NULL,
	@AccountId Varchar(50) = NULL,
	@SubAccountId Varchar(50) = NULL,
	@SystemService Varchar(50) = NULL,
	@SearchType Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
	-- SI @AccountId LLEGA COMO NULL SE BUSCA SOBRE TODAS LAS CUENTAS, CASO CONTRARIO UNICAMENTE SOBRE EL VALOR RECIBIDO
	-- SI @SubAccountId LLEGA COMO NULL SE BUSCA SOBRE TODAS LAS SUBCUENTAS, CASO CONTRARIO UNICAMENTE SOBRE EL VALOR RECIBIDO
	-- SI @SystemService LLEGA COMO NULL SE BUSCA SOBRE TODOS LOS PRODUCTOS, CASO CONTRARIO UNICAMENTE SOBRE EL VALOR RECIBIDO
	-- LA VARIABLE @SearchType NOS INDICA SI VAMOS A BUSCAR CUENTA(S) O SUBCUENTA(S)

	IF @SearchType = 'CTA'
	BEGIN 
		-- DECLARAMOS LAS CTE QUE JUNTARAN LA DATA REQUERIDA PARA EL RESULT
		;WITH CTE_SUBACCOUNS_TOTAL_AVAILABLE_BALANCE AS (
			-- SELECCIONAMOS LA SUMA TOTAL DEL DISPONIBLE DE LAS SUBCUENTAS
			SELECT INNER_AB.[InstallationId]								'InstalationId',
					INNER_AB.[AccountId]									'AccountId',
					SUM(INNER_AB.[AvailableBalance])						'SubAccountsTotalAvailableBalance'
				FROM [dbo].[AccountBalance] INNER_AB (NOLOCK) 
				WHERE INNER_AB.[InstallationId] = @InstallationId 
					AND INNER_AB.[AccountId] = CASE WHEN @AccountId IS NULL THEN INNER_AB.[AccountId] ELSE @AccountId END
					AND INNER_AB.[SubAccountId] IS NOT NULL
				GROUP BY INNER_AB.[InstallationId], INNER_AB.[AccountId]

		),
		CTE_VCCS_TOTAL_BALANCE AS (
			-- SELECCIONAMOS LA SUMA TOTAL DEL DISPONIBLE Y RETENIDO DE LOS VCCS (YA SEAN DIRECTOS O DE UNA SUBCUENTA DE LA CUENTA)
			SELECT INNER_AVB.[InstallationId]								'InstalationId',
					INNER_AVB.[AccountId]									'AccountId',
					CTE_SATAB.[SubAccountsTotalAvailableBalance]			'SubAccountsTotalAvailableBalance',
					SUM(INNER_AVB.[AvailableBalance])						'VCCsTotalAvailableBalance', 
					SUM(INNER_AVB.[RetainedBalance])						'VCCsTotalRetainedBalance' 
				FROM [dbo].[AccountVCCBalance] INNER_AVB (NOLOCK) 
				LEFT JOIN CTE_SUBACCOUNS_TOTAL_AVAILABLE_BALANCE CTE_SATAB 
					ON CTE_SATAB.[InstalationId] = INNER_AVB.[InstallationId]
						AND CTE_SATAB.[AccountId] = INNER_AVB.[AccountId]
				WHERE INNER_AVB.[InstallationId] = @InstallationId 
					AND INNER_AVB.[AccountId] = CASE WHEN @AccountId IS NULL THEN INNER_AVB.[AccountId] ELSE @AccountId END
				GROUP BY INNER_AVB.[InstallationId], INNER_AVB.[AccountId], CTE_SATAB.[SubAccountsTotalAvailableBalance]
		)
			
		-- HACEMOS LA BUSQUEDA DE LA CUENTA DONDE SUBCUENTA ES NULL
		SELECT AB.[AccountId]												'accountId', 
				AB.[SubAccountId]											'subaccountId',
				AB.[TotalBalance]											'totalBalance',
				AB.[AvailableBalance]										'availableBalance',
				AB.[AssignedBalance]										'assignedBalance',
				ISNULL(CTE_VTB.[SubAccountsTotalAvailableBalance], 0)		'subAccountsTotalAvailableBalance',
				ISNULL(CTE_VTB.[VCCsTotalAvailableBalance], 0)				'vccsTotalAvailableBalance',
				ISNULL(CTE_VTB.[VCCsTotalRetainedBalance], 0)				'vccsTotalRetainedBalance'
			FROM dbo.[AccountBalance] AB (NOLOCK) 
			LEFT JOIN CTE_VCCS_TOTAL_BALANCE CTE_VTB 
				ON CTE_VTB.[InstalationId] = AB.[InstallationId]
					AND CTE_VTB.[AccountId] = AB.[AccountId]
			WHERE AB.[InstallationId] = @InstallationId 
				AND AB.[AccountId] = CASE WHEN @AccountId IS NULL THEN AB.[AccountId] ELSE @AccountId END
				AND AB.[SubAccountId] IS NULL
	END
	ELSE 
	BEGIN
		-- DECLARAMOS LAS CTE QUE JUNTARAN LA DATA REQUERIDA PARA EL RESULT
		;WITH CTE_VCCS_TOTAL_BALANCE AS (
			-- SELECCIONAMOS LA SUMA TOTAL DEL DISPONIBLE Y RETENIDO DE LOS VCCS DE LA SUBCUENTA
			SELECT INNER_AVB.[InstallationId]								'InstalationId',
					INNER_AVB.[AccountId]									'AccountId',
					INNER_AVB.[SubAccountId]								'SubAccountId',
					SUM(INNER_AVB.[AvailableBalance])						'VCCsTotalAvailableBalance', 
					SUM(INNER_AVB.[RetainedBalance])						'VCCsTotalRetainedBalance' 
				FROM [dbo].[AccountVCCBalance] INNER_AVB (NOLOCK) 
				WHERE INNER_AVB.[InstallationId] = @InstallationId 
					AND INNER_AVB.[AccountId] = CASE WHEN @AccountId IS NULL THEN INNER_AVB.[AccountId] ELSE @AccountId END
					AND INNER_AVB.[SubAccountId] = CASE WHEN @SubAccountId IS NULL THEN INNER_AVB.[SubAccountId] ELSE @SubAccountId END
				GROUP BY INNER_AVB.[InstallationId], INNER_AVB.[AccountId], INNER_AVB.[SubAccountId]
		)

		-- HACEMOS LA BUSQUEDA DE LA SUBCUENTA DONDE SUBCUENTA ES DIFERENTE DE NULO
		SELECT AB.[AccountId]												'accountId', 
				AB.[SubAccountId]											'subaccountId',
				AB.[TotalBalance]											'totalBalance',
				AB.[AvailableBalance]										'availableBalance',
				AB.[AssignedBalance]										'assignedBalance',
				ISNULL(CTE_VTB.[VCCsTotalAvailableBalance], 0)				'vccsTotalAvailableBalance',
				ISNULL(CTE_VTB.[VCCsTotalRetainedBalance], 0)				'vccsTotalRetainedBalance'
			FROM dbo.[AccountBalance] AB (NOLOCK) 
			LEFT JOIN CTE_VCCS_TOTAL_BALANCE CTE_VTB 
				ON CTE_VTB.[InstalationId] = AB.[InstallationId]
					AND CTE_VTB.[AccountId] = AB.[AccountId]
					AND CTE_VTB.[SubAccountId] = AB.[SubAccountId]
			WHERE AB.[InstallationId] = @InstallationId 
				AND AB.[AccountId] = CASE WHEN @AccountId IS NULL THEN AB.[AccountId] ELSE @AccountId END
				AND AB.[SubAccountId] = CASE WHEN @SubAccountId IS NULL THEN AB.[SubAccountId] ELSE @SubAccountId END
	END

	--CREATE TABLE #DUMMY ( a Int, b VarChar(42) )
	--SELECT * FROM #DUMMY
	
	SET NOCOUNT OFF
END
GO
PRINT N'Modificando [dbo].[GetAccountSubAccountVCCBatchSummaryByBatchTypeId]...';


GO
ALTER PROCEDURE [dbo].[GetAccountSubAccountVCCBatchSummaryByBatchTypeId] (
	@PlatformHash VARCHAR(100) = NULL,
	@InstallationId VARCHAR(50) = NULL,
	@VCC VARCHAR(50) = NULL,
	@BatchTypeName VARCHAR(50) = NULL,
	@BatchTypeId VARCHAR(50) = NULL
	)
AS
BEGIN
	-- ESTE SP TIENE LA FINALIDAD BUSCAR SI EXISTEN REGISTROS DE UN BATCHID
	SET XACT_ABORT, NOCOUNT ON

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY

		-- VERIFICAMOS QUE TENEMOS LOS PARAMETROS NECESARIOS
		IF @PlatformHash IS NULL 
			OR @InstallationId IS NULL 
			OR @VCC IS NULL
			OR @BatchTypeName IS NULL
			OR @BatchTypeId IS NULL
			-- LANZAMOS MENSAJE AVISANDO QUE FALTA ACCOUNTID
			RAISERROR('DATABABASE_ERROR_MISSING_PARAMETERS', 16, 16)

		-- HACEMOS LA BUSQUEDA POR BATCHTYPEID
		SELECT ASAVST.[BatchTypeId] 
			FROM [dbo].[AccountSubAccountVCCSummaryTransaction] ASAVST (NOLOCK)
			WHERE ASAVST.[InstallationId] = @InstallationId
				AND ASAVST.[VCC] =  @VCC
				AND ASAVST.[BatchTypeName] = @BatchTypeName
				AND ASAVST.[BatchTypeId] =  @BatchTypeId

	END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH

	SET XACT_ABORT, NOCOUNT OFF
END
GO
PRINT N'Actualizando [dbo].[AccountSubAccountDeactivate]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[AccountSubAccountDeactivate]';


GO
PRINT N'Actualizando [dbo].[ActivatePlatform]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[ActivatePlatform]';


GO
PRINT N'Actualizando [dbo].[AddAccountVCCProduct]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[AddAccountVCCProduct]';


GO
PRINT N'Actualizando [dbo].[GetAccountByHashUserToken]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetAccountByHashUserToken]';


GO
PRINT N'Actualizando [dbo].[GetAccountStatusByVcc]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetAccountStatusByVcc]';


GO
PRINT N'Actualizando [dbo].[GetAccountSubAccountExists]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetAccountSubAccountExists]';


GO
PRINT N'Actualizando [dbo].[GetAccountSubAccountRate]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetAccountSubAccountRate]';


GO
PRINT N'Actualizando [dbo].[GetAccountSubAccountVCCBalance]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetAccountSubAccountVCCBalance]';


GO
PRINT N'Actualizando [dbo].[GetAccountUser]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetAccountUser]';


GO
PRINT N'Actualizando [dbo].[GetAllAccounts]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetAllAccounts]';


GO
PRINT N'Actualizando [dbo].[GetCosts]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetCosts]';


GO
PRINT N'Actualizando [dbo].[GetInstallationIds]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetInstallationIds]';


GO
PRINT N'Actualizando [dbo].[GetPlatformAccounts]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetPlatformAccounts]';


GO
PRINT N'Actualizando [dbo].[GetRelatedAccounts]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetRelatedAccounts]';


GO
PRINT N'Actualizando [dbo].[GetRelatedSubAccounts]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetRelatedSubAccounts]';


GO
PRINT N'Actualizando [dbo].[GetVCCsByAccount]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetVCCsByAccount]';


GO
PRINT N'Actualizando [dbo].[ReportByAccountSubAccountAvailableBalanceDetail]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[ReportByAccountSubAccountAvailableBalanceDetail]';


GO
PRINT N'Actualizando [dbo].[ReportByAccountSubAccountCharge]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[ReportByAccountSubAccountCharge]';


GO
PRINT N'Actualizando [dbo].[UpdatePlatformHash]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[UpdatePlatformHash]';


GO
PRINT N'Actualizando [dbo].[CreateMainAccountBalance]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[CreateMainAccountBalance]';


GO
PRINT N'Actualizando [dbo].[SaveAccountVCCProduct]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[SaveAccountVCCProduct]';


GO
PRINT N'Actualizando [dbo].[GetAccountVcc]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetAccountVcc]';


GO
PRINT N'Actualizando [dbo].[GetBatchBalanceDetail]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetBatchBalanceDetail]';


GO
PRINT N'Actualizando [dbo].[GetMovementCurrencies]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetMovementCurrencies]';


GO
PRINT N'Actualizando [dbo].[GetPlatformCurrentBatches]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetPlatformCurrentBatches]';


GO
PRINT N'Actualizando [dbo].[CloseCancelledBatches]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[CloseCancelledBatches]';


GO
PRINT N'Actualizando [dbo].[GetPlatformByHash]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetPlatformByHash]';


GO
PRINT N'Actualizando [dbo].[GetPlatforms]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[GetPlatforms]';


GO
PRINT N'Actualizando [dbo].[SavePlatformBuildVersion]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[SavePlatformBuildVersion]';


GO
PRINT N'Actualización completada.';


GO
